@@ -59,23 +59,13 @@ int upscli_connect(UPSCONN *ups, const char *host, int port, int flags);
 
 /* --- functions that only use the new names --- */
 
-int upscli_get(UPSCONN *ups, int numq, const char **query, 
-		int *numa, char ***answer);
+int upscli_get(UPSCONN *ups, unsigned int numq, const char **query, 
+		unsigned int *numa, char ***answer);
 
-int upscli_list_start(UPSCONN *ups, int numq, const char **query);
+int upscli_list_start(UPSCONN *ups, unsigned int numq, const char **query);
 
-int upscli_list_next(UPSCONN *ups, int numq, const char **query,
-		int *numa, char ***answer);
-
-/* --- functions that only use the old names --- */
-
-int upscli_getvar(UPSCONN *ups, const char *upsname, const char *var, 
-			char *buf, size_t buflen);
-
-int upscli_getlist(UPSCONN *ups, const char *upsname, int listtype, 
-			char *buf, size_t buflen);
-
-/* --- */
+int upscli_list_next(UPSCONN *ups, unsigned int numq, const char **query,
+		unsigned int *numa, char ***answer);
 
 int upscli_sendline(UPSCONN *ups, const char *buf, size_t buflen);
 
