@@ -38,14 +38,14 @@
 #include "timehead.h"
 #include "upslog.h"
 
-	static	int	port, reopen_flag = 0;
+	static	int	port, reopen_flag = 0, exit_flag = 0;
 	static	char	*upsname, *hostname;
 	static	UPSCONN	ups;
 
 	static	FILE	*logfile;
 	static	const	char *logfn, *monhost;
 	static	sigset_t	upsl_sigmask;
-	static	char	*logformat, logbuffer[LARGEBUF];
+	static	char	logbuffer[LARGEBUF], *logformat;
 
 	static	struct	flist_t	*fhead = NULL;
 
@@ -68,13 +68,16 @@ static void reopen_log(void)
 
 static void set_reopen_flag(int sig)
 {
-	upslogx(LOG_INFO, "Signal %d: reopening log file", sig);
-
-	reopen_flag = 1;
+	reopen_flag = sig;
 }
 
-/* install handler for reloading config on SIGHUP */
-static void instsighandler(void)
+static void set_exit_flag(int sig)
+{
+	exit_flag = sig;
+}
+
+/* handlers: reload on HUP, exit on INT/QUIT/TERM */
+static void setup_signals(void)
 {
 	struct	sigaction	sa;
 
@@ -85,6 +88,14 @@ static void instsighandler(void)
 	sa.sa_flags = 0;
 	if (sigaction(SIGHUP, &sa, NULL) < 0)
 		fatal("Can't install SIGHUP handler");
+
+	sa.sa_handler = set_exit_flag;
+	if (sigaction(SIGINT, &sa, NULL) < 0)
+		fatal("Can't install SIGINT handler");
+	if (sigaction(SIGQUIT, &sa, NULL) < 0)
+		fatal("Can't install SIGQUIT handler");
+	if (sigaction(SIGTERM, &sa, NULL) < 0)
+		fatal("Can't install SIGTERM handler");
 }
 
 static void help(const char *prog)
@@ -98,7 +109,7 @@ static void help(const char *prog)
 	printf("		- Use -f \"<format>\" so your shell doesn't break it up.\n");
 	printf("  -i <interval>	- Time between updates, in seconds\n");
 	printf("  -l <logfile>	- Log file name, or - for stdout\n");
-	printf("  -s <ups>	- Monitor UPS <ups> - [<upsname>@]host[:port]\n");
+	printf("  -s <ups>	- Monitor UPS <ups> - <upsname>@<host>[:<port>]\n");
 	printf("        	- Example: -s myups@server\n");
 	printf("  -u <user>	- Switch to <user> if started as root\n");
 
@@ -116,7 +127,7 @@ static void help(const char *prog)
 	printf("\n");
 	printf("See the upslog(8) man page for more information.\n");
 
-	exit(0);
+	exit(EXIT_SUCCESS);
 }
 
 /* print current host name */
@@ -147,7 +158,7 @@ static void do_pid(const char *arg)
 
 static void do_time(const char *arg)
 {
-	int	i;
+	unsigned int	i;
 	char	timebuf[SMALLBUF], *format;
 	time_t	tod;
 
@@ -166,22 +177,10 @@ static void do_time(const char *arg)
 	free(format);
 }
 
-static void old_getvar(const char *arg)
+static void getvar(const char *var)
 {
-	char	varval[16];
 	int	ret;
-
-	ret = upscli_getvar(&ups, upsname, arg, varval, sizeof(varval));
-
-	if (ret < 0)
-		snprintfcat(logbuffer, sizeof(logbuffer), "NA");
-	else
-		snprintfcat(logbuffer, sizeof(logbuffer), "%s", varval);
-}
-
-static void new_getvar(const char *var)
-{
-	int	ret, numq, numa;
+	unsigned int	numq, numa;
 	const	char	*query[4];
 	char	**answer;
 
@@ -207,19 +206,19 @@ static void do_var(const char *arg)
 		return;
 	}
 
-	/* old variable name? */
+	/* old variable names are no longer supported */
 	if (!strchr(arg, '.')) {
-		old_getvar(arg);
+		snprintfcat(logbuffer, sizeof(logbuffer), "INVALID");
 		return;
 	}
 
-	/* no upsname = fall back on old way (until 2.0) */
+	/* a UPS name is now required */
 	if (!upsname) {
-		old_getvar(arg);
+		snprintfcat(logbuffer, sizeof(logbuffer), "INVALID");
 		return;
 	} 
 
-	new_getvar(arg);
+	getvar(arg);
 }
 
 static void do_etime(const char *arg)
@@ -267,7 +266,8 @@ static void add_call(void (*fptr)(const char *arg), const char *arg)
 /* turn the format string into a list of function calls with args */
 static void compile_format(void)
 {
-	int	i, j, found, ofs;
+	unsigned int	i;
+	int	j, found, ofs;
 	char	*cmd, *arg, *ptr;
 
 	for (i = 0; i < strlen(logformat); i++) {
@@ -368,7 +368,8 @@ static void run_flist(void)
 int main(int argc, char **argv)
 {
 	int	interval = 30, i;
-	char	*prog = NULL, *user = NULL;
+	char	*prog = NULL;
+	const	char	*user = NULL;
 	struct	passwd	*new_uid = NULL;
 
 	logformat = DEFAULT_LOGFORMAT;
@@ -405,7 +406,7 @@ int main(int argc, char **argv)
 				break;
 
 			case 'V':
-				exit(0);
+				exit(EXIT_SUCCESS);
 		}
 	}
 
@@ -449,7 +450,8 @@ int main(int argc, char **argv)
 	printf("logging status of %s to %s (%is intervals)\n", 
 		monhost, logfn, interval);
 
-	upscli_splitname(monhost, &upsname, &hostname, &port);
+	if (upscli_splitname(monhost, &upsname, &hostname, &port) != 0)
+		fatalx("Fatal error: unusable UPS definition");
 
 	if (upscli_connect(&ups, hostname, port, UPSCLI_CONN_TRYSSL) < 0)
 		fprintf(stderr, "Warning: initial connect failed: %s\n", 
@@ -467,22 +469,23 @@ int main(int argc, char **argv)
 	if ((new_uid = get_user_pwent(user)) == NULL)
 		fatal("getpwnam(%s)", user);
 
-	become_user(new_uid);
-
 	openlog("upslog", LOG_PID, LOG_FACILITY); 
 
-	if (logfile != stdout) {
+	if (logfile != stdout)
 		background();
-		instsighandler();
-	}
+
+	setup_signals();
 
 	writepid("upslog");
 
+	become_user(new_uid);
+
 	compile_format();
 
-	for (;;) {
-
+	while (exit_flag == 0) {
 		if (reopen_flag) {
+			upslogx(LOG_INFO, "Signal %d: reopening log file", 
+				reopen_flag);
 			reopen_log();
 			reopen_flag = 0;
 		}
@@ -497,7 +500,12 @@ int main(int argc, char **argv)
 		sleep(interval);
 	}
 
-	/* notreached */
+	upslogx(LOG_INFO, "Signal %d: exiting", exit_flag);
 
-	exit(1);
+	if (logfile != stdout)
+		fclose(logfile);
+
+	upscli_disconnect(&ups);
+	
+	exit(EXIT_SUCCESS);
 }
