@@ -46,9 +46,6 @@ static	int	hostsync = 15;
 	/* sum of all power values from config file */
 static	int	totalpv = 0;
 
-	/* should we go into an infinite loop upon shutdown? */
-static	int	playdead = 0;
-
 	/* default replace battery warning interval (seconds) */
 static	int	rbwarntime = 43200;
 
@@ -86,7 +83,7 @@ static	sigset_t upsm_sigmask;
 #define	shutdown_how 2
 #endif
 
-static void debug(char *format, ...)
+static void debug(const char *format, ...)
 {
 #ifdef HAVE_STDARG_H
 	va_list	args;
@@ -106,7 +103,7 @@ static void setflag(int *val, int flag)
 {
 	*val = (*val |= flag);
 }
-        
+
 static void clearflag(int *val, int flag)  
 {
 	*val = (*val ^= (*val & flag));
@@ -174,7 +171,7 @@ static void notify(const char *notice, int flags, const char *ntype,
 		}
 	}
 
-	exit(0);
+	exit(EXIT_SUCCESS);
 }
 
 static void do_notify(const utype *ups, int ntype)
@@ -414,19 +411,21 @@ static void ups_on_line(utype *ups)
 }
 
 /* create the flag file if necessary */
-static void setpdflag(void)
+static void set_pdflag(void)
 {
 	FILE	*pdf;
 
-	if (powerdownflag != NULL) {
-		pdf = fopen(powerdownflag, "w");
-		if (!pdf) {
-			upslogx(LOG_ERR, "Failed to create power down flag!");
-			return;
-		}
-		fprintf(pdf, "%s", SDMAGIC);
-		fclose(pdf);
+	if (!powerdownflag)
+		return;
+
+	pdf = fopen(powerdownflag, "w");
+	if (!pdf) {
+		upslogx(LOG_ERR, "Failed to create power down flag!");
+		return;
 	}
+
+	fprintf(pdf, "%s", SDMAGIC);
+	fclose(pdf);
 }
 
 /* the actual shutdown procedure */
@@ -454,22 +453,16 @@ static void doshutdown(void)
 		if (geteuid() != 0)
 			upslogx(LOG_WARNING, "Not root, shutdown may fail");
 
-		setpdflag();
+		set_pdflag();
 
 		ret = system(shutdowncmd);
 
 		if (ret != 0)
 			upslogx(LOG_ERR, "Unable to call shutdown command: %s\n",
-		        	shutdowncmd);
+				shutdowncmd);
 	}
 
-	/* if instructed to go into the infinite loop, then do so */
-	if (playdead == 1)
-		for (;;)
-			sleep(100);
-
-	/* hopefully not reached */
-	exit(1);
+	exit(EXIT_SUCCESS);
 }
 
 /* set forced shutdown flag so other upsmons know what's going on here */
@@ -478,32 +471,38 @@ static void setfsd(utype *ups)
 	char	buf[SMALLBUF];
 	int	ret;
 
+	/* this shouldn't happen */
+	if (!ups->upsname) {
+		upslogx(LOG_ERR, "setfsd: programming error: no UPS name set [%s]",
+			ups->sys);
+		return;
+	}
+
 	debug("Setting FSD on UPS %s\n", ups->sys);
 
-	if (ups->upsname)
-		snprintf(buf, sizeof(buf), "FSD %s\n", ups->upsname);
-	else
-		snprintf(buf, sizeof(buf), "FSD\n");
+	snprintf(buf, sizeof(buf), "FSD %s\n", ups->upsname);
 
 	ret = upscli_sendline(&ups->conn, buf, strlen(buf));
 
 	if (ret < 0) {
 		upslogx(LOG_ERR, "FSD set on UPS %s failed: %s", ups->sys,
-		        upscli_strerror(&ups->conn));
+			upscli_strerror(&ups->conn));
 		return;
 	}
 
 	ret = upscli_readline(&ups->conn, buf, sizeof(buf));
 
+	if (ret < 0) {
+		upslogx(LOG_ERR, "FSD set on UPS %s failed: %s", ups->sys,
+			upscli_strerror(&ups->conn));
+		return;
+	}
+
 	if (!strncmp(buf, "OK", 2))
 		return;
 
-	if (ret < 0)
-		upslogx(LOG_ERR, "FSD set on UPS %s failed: %s", ups->sys,
-		        upscli_strerror(&ups->conn));
-	else
-		upslogx(LOG_ERR, "FSD set on UPS %s failed: %s", ups->sys,
-		        buf);
+	/* protocol error: upsd said something other than "OK" */
+	upslogx(LOG_ERR, "FSD set on UPS %s failed: %s", ups->sys, buf);
 }
 
 static void set_alarm(void)
@@ -517,36 +516,18 @@ static void clear_alarm(void)
 	alarm(0);
 }
 
-static int old_get_var(utype *ups, const char *var, char *buf, size_t bufsize)
+static int get_var(utype *ups, const char *var, char *buf, size_t bufsize)
 {
-	debug("old_get_var: %s / %s\n", ups->sys, var);
-
-	if (!strcmp(var, "numlogins"))
-		return upscli_getvar(&ups->conn, ups->upsname, "numlogins",
- 			buf, bufsize);
-
-	if (!strcmp(var, "status"))
-		return upscli_getvar(&ups->conn, ups->upsname, "status",
-			buf, bufsize);
-
-	upslogx(LOG_ERR, "old_get_var: programming error: var=%s", var);
-	return -1;
-}
-
-static int new_get_var(utype *ups, const char *var, char *buf, size_t bufsize)
-{
-	int	ret, numq, numa;
+	int	ret;
+	unsigned int	numq, numa;
 	const	char	*query[4];
 	char	**answer;
 
-	/* can't do the new stuff without a UPS name set */
-
-	/* this shouldn't happen (we now require upsnames) */
+	/* this shouldn't happen */
 	if (!ups->upsname) {
-		upslogx(LOG_INFO, "UPS [%s]: no upsname set, forcing compatibility mode",
+		upslogx(LOG_ERR, "get_var: programming error: no UPS name set [%s]",
 			ups->sys);
-		ups->old = 1;
-		return old_get_var(ups, var, buf, bufsize);
+		return -1;
 	}
 
 	numq = 0;
@@ -565,11 +546,11 @@ static int new_get_var(utype *ups, const char *var, char *buf, size_t bufsize)
 	}
 
 	if (numq == 0) {
-		upslogx(LOG_ERR, "new_get_var: programming error: var=%s", var);
+		upslogx(LOG_ERR, "get_var: programming error: var=%s", var);
 		return -1;
 	}
 
-	debug("new_get_var: %s / %s\n", ups->sys, var);
+	debug("get_var: %s / %s\n", ups->sys, var);
 
 	ret = upscli_get(&ups->conn, numq, query, &numa, &answer);
 
@@ -578,11 +559,9 @@ static int new_get_var(utype *ups, const char *var, char *buf, size_t bufsize)
 		/* detect old upsd */
 		if (upscli_upserror(&ups->conn) == UPSCLI_ERR_UNKCOMMAND) {
 
-			upslogx(LOG_INFO, "UPS [%s]: using compatibility mode",
+			upslogx(LOG_ERR, "UPS [%s]: Too old to monitor",
 				ups->sys);
-
-			ups->old = 1;
-			return old_get_var(ups, var, buf, bufsize);
+			return -1;
 		}
 
 		/* some other error */
@@ -600,15 +579,6 @@ static int new_get_var(utype *ups, const char *var, char *buf, size_t bufsize)
 	return 0;
 }
 
-static int get_var(utype *ups, const char *var, char *buf, size_t bufsize)
-{
-	if (ups->old)
-		return old_get_var(ups, var, buf, bufsize);
-
-	/* try it the new way */
-	return new_get_var(ups, var, buf, bufsize);
-}
-
 static void slavesync(void)
 {
 	utype	*ups;
@@ -666,7 +636,7 @@ static void forceshutdown(void)
 	for (ups = firstups; ups != NULL; ups = ups->next)
 		if (flag_isset(ups->status, ST_MASTER)) {
 			isamaster = 1;
-			setfsd (ups);
+			setfsd(ups);
 		}
 
 	/* if we're not a master on anything, we should shut down now */
@@ -804,9 +774,6 @@ static void drop_connection(utype *ups)
 {
 	debug("Dropping connection to UPS [%s]\n", ups->sys);
 
-	/* reset this in case upsd gets upgraded */
-	ups->old = 0;
-
 	ups->commstate = 0;
 	ups->linestate = 0;
 	clearflag(&ups->status, ST_LOGIN);
@@ -991,7 +958,6 @@ static void addups(int reloading, const char *sys, const char *pvs,
 	tmp->lastnoncrit = 0;
 	tmp->lastrbwarn = 0;
 	tmp->lastncwarn = 0;
-	tmp->old = 0;
 
 	if (!strcasecmp(master, "master"))
 		setflag(&tmp->status, ST_MASTER);
@@ -1005,12 +971,18 @@ static void addups(int reloading, const char *sys, const char *pvs,
 
 	if (tmp->pv)
 		upslogx(LOG_INFO, "UPS: %s (%s) (power value %d)", tmp->sys, 
-		        flag_isset(tmp->status, ST_MASTER) ? "master" : "slave",
+			flag_isset(tmp->status, ST_MASTER) ? "master" : "slave",
 			tmp->pv);
 	else
 		upslogx(LOG_INFO, "UPS: %s (monitoring only)", tmp->sys);
 
-        upscli_splitname(tmp->sys, &tmp->upsname, &tmp->hostname, &tmp->port);
+	tmp->upsname = tmp->hostname = NULL;	
+
+	if (upscli_splitname(tmp->sys, &tmp->upsname, &tmp->hostname, 
+		&tmp->port) != 0) {
+		upslogx(LOG_ERR, "Error: unable to split UPS name [%s]",
+			tmp->sys);
+	}
 
 	if (!tmp->upsname)
 		upslogx(LOG_WARNING, "Warning: UPS [%s]: no upsname set!",
@@ -1272,7 +1244,7 @@ static int parse_conf_arg(int numargs, char **arg)
 }
 
 /* called for fatal errors in parseconf like malloc failures */
-void upsmon_err(const char *errmsg)
+static void upsmon_err(const char *errmsg)
 {
 	upslogx(LOG_ERR, "Fatal error in parseconf(upsmon.conf): %s", errmsg);
 }
@@ -1294,7 +1266,7 @@ static void loadconfig(void)
 			return;
 		}
 
-		fatal("%s", ctx.errmsg);
+		fatalx("%s", ctx.errmsg);
 	}
 
 	while (pconf_file_next(&ctx)) {
@@ -1308,7 +1280,7 @@ static void loadconfig(void)
 			continue;
 
 		if (!parse_conf_arg(ctx.numargs, ctx.arglist)) {
-			int	i;
+			unsigned int	i;
 			char	errmsg[SMALLBUF];
 
 			snprintf(errmsg, sizeof(errmsg), 
@@ -1319,7 +1291,7 @@ static void loadconfig(void)
 				snprintfcat(errmsg, sizeof(errmsg), " %s", 
 					ctx.arglist[i]);
 
-			upslogx(LOG_WARNING, "%s", errmsg);
+			upslogx(LOG_WARNING, errmsg);
 		}
 	}
 
@@ -1335,8 +1307,7 @@ static void sigpipe(int sig)
 /* SIGQUIT, SIGTERM handler */
 static void set_exit_flag(int sig)
 {
-	upslogx(LOG_INFO, "Signal %d: exiting", sig);
-	exit_flag = 1;
+	exit_flag = sig;
 }
 
 static void ups_free(utype *ups)
@@ -1354,7 +1325,7 @@ static void ups_free(utype *ups)
 	free(ups);
 }
 
-static void upsmon_exit(void)
+static void upsmon_cleanup(void)
 {
 	int	i;
 	utype	*utmp, *unext;
@@ -1383,8 +1354,6 @@ static void upsmon_exit(void)
 	for (i = 0; notifylist[i].name != NULL; i++)
 		if (notifylist[i].msg != notifylist[i].stockmsg)
 			free(notifylist[i].msg);
-
-	exit(0);
 }
 
 static void user_fsd(int sig)
@@ -1542,7 +1511,7 @@ static int try_connect(utype *ups)
 /* deal with the contents of STATUS or ups.status for this ups */
 static void parse_status(utype *ups, char *status)
 {
-	char	*stat, *ptr;
+	char	*statword, *ptr;
 
 	clear_alarm();
 
@@ -1562,32 +1531,32 @@ static void parse_status(utype *ups, char *status)
 	if (!strstr(status, "FSD"))
 		clearflag(&ups->status, ST_FSD);
 
-	stat = status;
+	statword = status;
 
 	/* split up the status words and parse each one separately */
-	while (stat != NULL) {
-		ptr = strchr(stat, ' ');
+	while (statword != NULL) {
+		ptr = strchr(statword, ' ');
 		if (ptr)
 			*ptr++ = '\0';
 
-		debug("    parsing: [%s]: ", stat);
+		debug("    parsing: [%s]: ", statword);
 
-		if (!strcasecmp(stat, "OL"))
+		if (!strcasecmp(statword, "OL"))
 			ups_on_line(ups);
-		if (!strcasecmp(stat, "OB"))
+		if (!strcasecmp(statword, "OB"))
 			ups_on_batt(ups);
-		if (!strcasecmp(stat, "LB"))
+		if (!strcasecmp(statword, "LB"))
 			ups_low_batt(ups);
-		if (!strcasecmp(stat, "RB"))
+		if (!strcasecmp(statword, "RB"))
 			upsreplbatt(ups);
 
 		/* do it last to override any possible OL */
-		if (!strcasecmp(stat, "FSD"))
+		if (!strcasecmp(statword, "FSD"))
 			ups_fsd(ups);
 
 		update_crittimer(ups);
 
-		stat = ptr;
+		statword = ptr;
 	} 
 
 	debug("\n");
@@ -1645,16 +1614,19 @@ static void pollups(utype *ups)
 	}
 }
 
-/* remove the power down flag if it exists and is the proper form */
-static void clearpdf(void)
+/* see if the powerdownflag file is there and proper */
+static int pdflag_status(void)
 {
 	FILE	*pdf;
 	char	buf[SMALLBUF];
 
+	if (!powerdownflag)
+		return 0;	/* unusable */
+
 	pdf = fopen(powerdownflag, "r");
 
-	if (pdf == NULL)	/* no such file, nothing to do */
-		return;
+	if (pdf == NULL)
+		return 0;	/* not there */
 
 	/* if it exists, see if it has the right text in it */
 
@@ -1669,13 +1641,59 @@ static void clearpdf(void)
 	 * solution: don't let mere mortals edit that configuration file.
 	 */
 
-	if (!strncmp(buf, SDMAGIC, strlen(SDMAGIC)))	/* ok, it's from us */
-		unlink(powerdownflag);
-	else {
-		upslogx(LOG_INFO, "%s doesn't seem to be a proper flag file.  Disabling.",
-			  powerdownflag);
+	if (!strncmp(buf, SDMAGIC, strlen(SDMAGIC)))
+		return 1;	/* exists and looks good */
+
+	return -1;	/* error: something else is in there */
+}	
+
+/* only remove the flag file if it's actually from us */
+static void clear_pdflag(void)
+{
+	int	ret;
+
+	ret = pdflag_status();
+
+	if (ret == -1)  {
+		upslogx(LOG_ERR, "POWERDOWNFLAG (%s) does not contain"
+			"the upsmon magic string - disabling!", powerdownflag);
 		powerdownflag = NULL;
+		return;
 	}
+
+	/* it's from us, so we can remove it */
+	if (ret == 1)
+		unlink(powerdownflag);
+}
+
+/* exit with success only if it exists and is proper */
+static int check_pdflag(void)
+{
+	int	ret;
+
+	ret = pdflag_status();
+
+	if (ret == -1) {
+		upslogx(LOG_ERR, "POWERDOWNFLAG (%s) does not contain "
+			"the upsmon magic string");
+		return EXIT_FAILURE;
+	}
+
+	if (ret == 0) {
+		/* not there - this is not a shutdown event */
+		printf("Power down flag is not set\n");
+		return EXIT_FAILURE;
+	}
+
+	if (ret != 1) {
+		upslogx(LOG_ERR, "Programming error: pdflag_status returned %d",
+			ret);
+		return EXIT_FAILURE;
+	}
+
+	/* only thing left - must be time for a shutdown */
+	printf("Power down flag is set\n");
+	return EXIT_SUCCESS;
 }
 
 static void help(const char *progname)
@@ -1690,10 +1708,11 @@ static void help(const char *progname)
 	printf("		 - stop: stop monitoring and exit\n");
 	printf("  -D		raise debugging level\n");
 	printf("  -h		display this help\n");
-	printf("  -i		go into an infinite loop after calling shutdown\n");
+	printf("  -K		checks POWERDOWNFLAG, sets exit code to 0 if set\n");
 	printf("  -p		always run privileged (disable privileged parent)\n");
 	printf("  -u <user>	run child as user <user> (ignored when using -p)\n");
-	exit(0);
+
+	exit(EXIT_SUCCESS);
 }
 
 /* set all the notify values to a default */
@@ -1707,7 +1726,7 @@ static void initnotify(void)
 	}
 }
 
-static void runparent(int pipe)
+static void runparent(int fd)
 {
 	int	ret;
 	char	ch;
@@ -1717,7 +1736,7 @@ static void runparent(int pipe)
 	signal(SIGUSR1, SIG_IGN);
 	signal(SIGUSR2, SIG_IGN);
 
-	ret = read(pipe, &ch, 1);
+	ret = read(fd, &ch, 1);
 
 	if (ret < 1) {
 		if (errno == ENOENT)
@@ -1730,30 +1749,32 @@ static void runparent(int pipe)
 		fatalx("upsmon parent: got bogus pipe command %c", ch);
 
 	/* have to do this here - child is unprivileged */
-	setpdflag();
+	set_pdflag();
 
 	ret = system(shutdowncmd);
 
 	if (ret != 0)
 		upslogx(LOG_ERR, "parent: Unable to call shutdown command: %s\n",
-	        	shutdowncmd);
+			shutdowncmd);
 
-	close(pipe);
-	exit(0);
+	close(fd);
+	exit(EXIT_SUCCESS);
 }
 
 /* fire up the split parent/child scheme */
-static void start_pipe(char *user)
+static void start_pipe(const char *user)
 {
 	int	ret;
 	struct	passwd	*new_uid = NULL;
 
-	/* default to the --with-user value from configure */
-	if (!user)
-		user = RUN_AS_USER;
+	/* default user = the --with-user value from configure */
+	if (user)
+		new_uid = get_user_pwent(user);
+	else
+		new_uid = get_user_pwent(RUN_AS_USER);
 
-	/* grab user data early, while we still have the console */
-	if ((new_uid = get_user_pwent(user)) == NULL)
+	/* check user data while we still have the console */
+	if (!new_uid) 
 		fatal("getpwnam(%s)", user);
 
 	ret = pipe(pipefd);
@@ -1771,7 +1792,7 @@ static void start_pipe(char *user)
 		close(pipefd[1]);
 		runparent(pipefd[0]);
 
-		exit(1);	/* NOTREACHED */
+		exit(EXIT_FAILURE);	/* NOTREACHED */
 	}
 
 	close(pipefd[0]);
@@ -1821,7 +1842,7 @@ static void delete_ups(utype *target)
 }	
 
 /* see if we can open a file */
-static int check_file(char *fn)
+static int check_file(const char *fn)
 {
 	char	chkfn[SMALLBUF];
 	FILE	*f;
@@ -1927,27 +1948,14 @@ static void check_parent(void)
 
 int main(int argc, char *argv[])  
 {
-	int	i, cmd;
+	int	i, cmd, checking_flag = 0;
 
 	cmd = 0;
 
 	printf("Network UPS Tools upsmon %s\n", UPS_VERSION);
 
-	while ((i = getopt(argc, argv, "+Ddhic:pu:V")) != EOF) {
+	while ((i = getopt(argc, argv, "+Dhic:pu:VK")) != EOF) {
 		switch (i) {
-			case 'D':
-			case 'd':
-				debuglevel++;
-				break;
-			case 'h':
-				help(argv[0]);
-				break;
-			case 'i':
-				playdead = 1;
-				break;
-			case 'p':
-				use_pipe = 0;
-				break;
 			case 'c':
 				if (!strncmp(optarg, "fsd", strlen(optarg)))
 					cmd = SIGCMD_FSD;
@@ -1960,12 +1968,26 @@ int main(int argc, char *argv[])
 				if (cmd == 0)
 					help(argv[0]);
 				break;
+			case 'D':
+				debuglevel++;
+				break;
+			case 'h':
+				help(argv[0]);
+				break;
+
+			case 'K':
+				checking_flag = 1;
+				break;
+
+			case 'p':
+				use_pipe = 0;
+				break;
 			case 'u':
 				run_as_user = xstrdup(optarg);
 				break;
 			case 'V':
 				/* just show the banner */
-				exit(0);
+				exit(EXIT_SUCCESS);
 			default:
 				help(argv[0]);
 				break;
@@ -1974,7 +1996,7 @@ int main(int argc, char *argv[])
 
 	if (cmd) {
 		sendsignal("upsmon", cmd);
-		exit(0);
+		exit(EXIT_SUCCESS);
 	}
 
 	argc -= optind;
@@ -1985,12 +2007,15 @@ int main(int argc, char *argv[])
 	initnotify();
 	loadconfig();
 
+	if (checking_flag)
+		exit(check_pdflag());
+
 	if (shutdowncmd == NULL)
 		printf("Warning: no shutdown command defined!\n");
 
 	/* we may need to get rid of a flag from a previous shutdown */
 	if (powerdownflag != NULL)
-		clearpdf();
+		clear_pdflag();
 
 	if (totalpv < minsupplies) {
 		printf("\nFatal error: insufficient power configured!\n\n");
@@ -1999,7 +2024,7 @@ int main(int argc, char *argv[])
 		printf("Minimum value (MINSUPPLIES): %d\n", minsupplies);
 
 		printf("\nEdit your upsmon.conf and change the values.\n");
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 
 	if (debuglevel < 1)
@@ -2022,16 +2047,13 @@ int main(int argc, char *argv[])
 	closelog();
 	openlog("upsmon", LOG_PID, LOG_FACILITY);
 
-	for (;;) {
+	while (exit_flag == 0) {
 		utype	*ups;
 
 		/* check flags from signal handlers */
 		if (userfsd)
 			forceshutdown();
 
-		if (exit_flag)
-			upsmon_exit();
-
 		if (reload_flag)
 			reload_conf();
 
@@ -2051,5 +2073,8 @@ int main(int argc, char *argv[])
 		sleep(sleepval);
 	}
 
-	/* NOTREACHED */
+	upslogx(LOG_INFO, "Signal %d: exiting", exit_flag);
+	upsmon_cleanup();
+
+	exit(EXIT_SUCCESS);
 }
