@@ -144,7 +144,7 @@ static void checktimers(void)
 #endif
 
 		unlink(pipefn);
-		exit(0);
+		exit(EXIT_SUCCESS);
 	}
 
 	emptyctr = 0;
@@ -259,7 +259,7 @@ static void us_serialize(int op)
 	}
 }
 
-static int open_sock(const char *fn)
+static int open_sock(void)
 {
 	int	ret, fd;
 	struct	sockaddr_un	ssaddr;
@@ -274,7 +274,6 @@ static int open_sock(const char *fn)
 
 	unlink(pipefn);
 
-	/* group gets access so upsd can be a different user but same group */
 	umask(0007);
 
 	ret = bind(fd, (struct sockaddr *) &ssaddr, sizeof ssaddr);
@@ -334,7 +333,7 @@ static int send_to_one(struct conn_t *conn, const char *fmt, ...)
 
 	ret = write(conn->fd, buf, strlen(buf));
 
-	if (ret != strlen(buf)) {
+	if ((ret < 1) || (ret != (int) strlen(buf))) {
 		upsdebugx(2, "write to fd %d failed", conn->fd);
 
 		close(conn->fd);
@@ -350,11 +349,11 @@ static void conn_add(int sockfd)
 {
 	int	acc, ret;
 	struct	conn_t	*tmp, *last;
-	struct	sockaddr_un sa;
+	struct	sockaddr_un	saddr;
 	socklen_t	salen;
 
-	salen = sizeof(sa);
-	acc = accept(sockfd, (struct sockaddr *) &sa, &salen);
+	salen = sizeof(saddr);
+	acc = accept(sockfd, (struct sockaddr *) &saddr, &salen);
 
 	if (acc < 0) {
 		upslog(LOG_ERR, "accept on unix fd failed");
@@ -517,7 +516,7 @@ static void start_daemon(int lockfd)
 	dup(0);
 	dup(0);
 
-	pipefd = open_sock(pipefn);
+	pipefd = open_sock();
 
 	if (verbose)
 		upslogx(LOG_INFO, "Timer daemon started");
@@ -546,7 +545,7 @@ static void start_daemon(int lockfd)
 
 			if (tmp->fd > maxfd)
 				maxfd = tmp->fd;
-	        }
+		}
 
 		ret = select(maxfd + 1, &rfds, NULL, NULL, &tv);
 
@@ -577,21 +576,21 @@ static void start_daemon(int lockfd)
 
 /* --- 'client' functions --- */
 
-static int try_connect(const char *pipefn)
+static int try_connect(void)
 {
 	int	pipefd, ret;
-	struct	sockaddr_un sa;
+	struct	sockaddr_un saddr;
 
-	memset(&sa, '\0', sizeof(sa));
-	sa.sun_family = AF_UNIX;
-	snprintf(sa.sun_path, sizeof(sa.sun_path), "%s", pipefn);
+	memset(&sa, '\0', sizeof(saddr));
+	saddr.sun_family = AF_UNIX;
+	snprintf(saddr.sun_path, sizeof(saddr.sun_path), "%s", pipefn);
 
 	pipefd = socket(AF_UNIX, SOCK_STREAM, 0);
 
 	if (pipefd == -1)
 		fatal("socket");
 
-	ret = connect(pipefd, (const struct sockaddr *) &sa, sizeof(sa));
+	ret = connect(pipefd, (const struct sockaddr *) &saddr, sizeof(saddr));
 
 	if (ret != -1)
 		return pipefd;
@@ -612,7 +611,7 @@ static int check_parent(const char *cmd)
 
 	for (tries = 0; tries < MAX_TRIES; tries++) {
 
-		pipefd = try_connect(pipefn);
+		pipefd = try_connect();
 
 		if (pipefd != -1)
 			return pipefd;
@@ -642,7 +641,7 @@ static int check_parent(const char *cmd)
 	}
 
 	upslog(LOG_ERR, "Failed to connect to parent and failed to create parent");
-	exit(1);
+	exit(EXIT_FAILURE);
 }
 
 static void read_timeout(int sig)
@@ -701,12 +700,13 @@ static void sendcmd(const char *cmd, const char *arg1, const char *arg2)
 		ret = write(pipefd, buf, strlen(buf));
 
 		/* if we can't send the whole thing, loop back and try again */
-		if (ret != strlen(buf)) {
+		if ((ret < 1) || (ret != (int) strlen(buf))) {
 			upslogx(LOG_ERR, "write failed, trying again");
 			close(pipefd);
 			continue;
 		}
 
+		/* ugh - probably should use select here... */
 		setup_sigalrm();
 
 		alarm(2);
@@ -784,7 +784,7 @@ static void parse_at(const char *ntype, const char *un, const char *cmd,
 		if (verbose)	
 			upslogx(LOG_INFO, "Executing command: %s", ca1);
 
-		exec_cmd((char *) ca1);
+		exec_cmd(ca1);
 		return;
 	}
 
@@ -833,7 +833,7 @@ static int conf_arg(int numargs, char **arg)
 }
 
 /* called for fatal errors in parseconf like malloc failures */
-void upssched_err(const char *errmsg)
+static void upssched_err(const char *errmsg)
 {
 	upslogx(LOG_ERR, "Fatal error in parseconf(upssched.conf): %s", errmsg);
 }
@@ -863,7 +863,7 @@ static void checkconf(void)
 			continue;
 
 		if (!conf_arg(ctx.numargs, ctx.arglist)) {
-			int	i;
+			unsigned int	i;
 			char	errmsg[SMALLBUF];
 
 			snprintf(errmsg, sizeof(errmsg), 
@@ -873,7 +873,7 @@ static void checkconf(void)
 				snprintfcat(errmsg, sizeof(errmsg), " %s", 
 					ctx.arglist[i]);
 
-			upslogx(LOG_WARNING, "%s", errmsg);
+			upslogx(LOG_WARNING, errmsg);
 		}
 	}
 
@@ -894,11 +894,11 @@ int main(int argc, char **argv)
 	if ((!upsname) || (!notify_type)) {
 		printf("Error: UPSNAME and NOTIFYTYPE must be set.\n");
 		printf("This program should only be run from upsmon.\n");
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 
 	/* see if this matches anything in the config file */
 	checkconf();
 
-	return 0;
+	exit(EXIT_SUCCESS);
 }
