@@ -137,103 +137,6 @@ void parsearg(char *var, char *value)
 	}
 }
 
-static void do_enum(const char *varname)
-{
-	char	out[SMALLBUF], temp[SMALLBUF], *val, *sel, *ptr;
-
-	snprintf(out, sizeof(out),
-		(upsname ? "ENUM %s@%s\n" : "ENUM %s\n"), varname, upsname);
-	
-	if (upscli_sendline(&ups, out, strlen(out)) < 0) {
-		fprintf(stderr, "ENUM %s failed: %s\n", varname,
-			upscli_strerror(&ups));
-		printf("Unable to get ENUM list\n");
-		return;
-	}
-
-	if (upscli_readline(&ups, temp, sizeof(temp)) < 0) {
-		fprintf(stderr, "Read ENUM %s failed: %s\n", varname,
-			upscli_strerror(&ups));
-		printf("Unable to get ENUM list\n");
-		return;
-	}
-
-	if (strncmp(temp, "ENUM", 4) != 0) {
-		printf("Bogus reply from server for %s\n", varname);
-		return;
-	}
-
-	printf("<SELECT NAME=\"UPSVAR_%s\">\n", varname);
-
-	/* 'OPTION "<val>" [SELECTED]' || 'END' */
-	if (upscli_readline(&ups, temp, sizeof(temp)) < 0) {
-		printf("<!-- Error reading OPTION list -->\n");
-
-		fprintf(stderr, "Can't read OPTION list: %s\n",
-			upscli_strerror(&ups));
-		printf("</SELECT>\n");
-		return;
-	}
-
-	while (strcmp(temp, "END") != 0) {
-
-		/* split into value and selected */
-		val = strchr(temp, '"');
-
-		/* sanity check - no " means a protocol problem */
-		if (!val) {
-			printf("<!-- Error: no quote detected in ENUM response -->\n");
-			printf("</SELECT>\n");
-			return;
-		}
-
-		/* move off the " */
-		val++;
-
-		sel = strchr(val, ' ');
-		if (sel)
-			sel++;
-		ptr = strchr(val, '"');
-
-		/* same idea */
-		if (!ptr) {
-			printf("<!-- Error: no quote detected in ENUM response -->\n");
-			printf("</SELECT>\n");
-			return;
-		}
-
-		val [ptr - val] = '\0';
-			
-		printf("<OPTION VALUE=\"%s\" ", val);
-			
-		if ((sel != NULL) && (!strcmp(sel, "SELECTED")))
-			printf("SELECTED");
-
-		printf(">%s</OPTION>\n", val);	
-
-		if (upscli_readline(&ups, temp, sizeof(temp)) < 0) {
-			printf("<!-- Error reading OPTION list -->\n");
-
-			fprintf(stderr, "Can't read OPTION list: %s\n",
-				upscli_strerror(&ups));
-			printf("</SELECT>\n");
-			return;
-		}
-	}
-
-	printf("</SELECT>\n");
-}
-
-static void do_string(const char *varname, int typelen)
-{
-	char	temp[SMALLBUF];
-
-	if (upscli_getvar(&ups, upsname, varname, temp, sizeof(temp)) == 0)
-		printf("<INPUT TYPE=\"TEXT\" NAME=\"UPSVAR_%s\" VALUE=\"%s\" "
-			"SIZE=\"%i\">\n",
-			varname, temp, typelen);
-}
-
 static void do_header(const char *title)
 {
 	printf("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\"\n");
@@ -252,7 +155,7 @@ static void start_table(void)
 	printf("<TABLE CELLPADDING=\"5\" CELLSPACING=\"0\" ALIGN=\"CENTER\" WIDTH=\"100%%\">\n");
 	printf("<TR><TH COLSPAN=2 BGCOLOR=\"#60B0B0\">\n");
 	printf("<FONT SIZE=\"+2\">Network UPS Tools upsset %s</FONT>\n", 
-	        UPS_VERSION);
+		UPS_VERSION);
 	printf("</TH></TR>\n");
 }
 
@@ -260,13 +163,13 @@ static void start_table(void)
 static void do_hidden(const char *next)
 {
 	printf("<INPUT TYPE=\"HIDDEN\" NAME=\"username\" VALUE=\"%s\">\n",
-	       username);
+		username);
 	printf("<INPUT TYPE=\"HIDDEN\" NAME=\"password\" VALUE=\"%s\">\n",
-	       password);
+		password);
 
 	if (next)
 		printf("<INPUT TYPE=\"HIDDEN\" NAME=\"function\" VALUE=\"%s\">\n", 
-		       next);
+			next);
 }
 
 /* generate SELECT chooser from hosts.conf entries */
@@ -288,7 +191,7 @@ static void upslist_arg(int numargs, char **arg)
 }
 
 /* called for fatal errors in parseconf like malloc failures */
-void upsset_hosts_err(const char *errmsg)
+static void upsset_hosts_err(const char *errmsg)
 {
 	upslogx(LOG_ERR, "Fatal error in parseconf(hosts.conf): %s", errmsg);
 }
@@ -325,7 +228,7 @@ static void do_pickups(const char *currfunc)
 		if (pconf_parse_error(&ctx)) {
 			upslogx(LOG_ERR, "Parse error: %s:%d: %s",
 				hostfn, ctx.linenum, ctx.errmsg);
-                        
+
 			continue;
 		}
 
@@ -386,7 +289,7 @@ static void error_page(const char *next, const char *title,
 	printf("</BODY></HTML>\n");
 
 	upscli_disconnect(&ups);
-	exit(0);
+	exit(EXIT_SUCCESS);
 }
 
 static void loginscreen(void)
@@ -415,13 +318,17 @@ static void loginscreen(void)
 	printf("</BODY></HTML>\n");
 
 	upscli_disconnect(&ups);
-	exit(0);
+	exit(EXIT_SUCCESS);
 }
 
 /* try to connect to upsd - generate an error page if it fails */
 static void upsd_connect(void)
 {
-	upscli_splitname(monups, &upsname, &hostname, &port);
+	if (upscli_splitname(monups, &upsname, &hostname, &port) != 0) {
+		error_page("showsettings", "UPS name is unusable",
+			"Unable to split UPS name [%s]", monups);
+		/* NOTREACHED */
+	}
 
 	if (upscli_connect(&ups, hostname, port, 0) < 0) {
 		error_page("showsettings", "Connect failure",
@@ -431,114 +338,10 @@ static void upsd_connect(void)
 	}
 }
 
-static void old_showcmds(const char *desc)
-{
-	char	*ptr, cmds[SMALLBUF], *tmp;
-
-	if (upscli_getlist(&ups, upsname, UPSCLI_LIST_CMDS, cmds, 
-		sizeof(cmds)) < 0) {
-
-		error_page("showcmds", "Protocol failure",
-		         "Unable to get list of commands - %s\n",
-		         upscli_strerror(&ups));
-
-		/* NOTREACHED */
-	}
-
-	if (strlen(cmds) == 0)
-		error_page("showcmds", "No instant commands supported",
-		          "This UPS doesn't support any instant commands.");
-
-	do_header("Instant commands");
-	printf("<FORM ACTION=\"upsset.cgi\" METHOD=\"POST\">\n");
-	start_table();
-
-	/* include the description from checkhost() if present */
-	if (desc)
-		printf("<TR><TH BGCOLOR=\"#60B0B0\"COLSPAN=2>%s</TH></TR>\n",
-		       desc);
-
-	printf("<TR BGCOLOR=\"#60B0B0\" ALIGN=\"CENTER\">\n");
-	printf("<TD>Instant commands</TD>\n");
-
-	printf("<TD>\n");
-	printf("<SELECT NAME=\"upscommand\">\n");
-
-	/* provide a dummy do-nothing default choice */
-	printf("<OPTION VALUE=\"\" SELECTED></OPTION>\n");
-
-	ptr = cmds;
-
-	while (ptr) {
-		char	buf[SMALLBUF], *q, *desc;
-
-		tmp = ptr;
-		ptr = strchr(tmp, ' ');
-		if (ptr) {
-			*ptr = '\0';
-			ptr++;
-		}
-
-		/* get a description for this command from the server */
-		snprintf(buf, sizeof(buf), "INSTCMDDESC %s\n", tmp);
-
-		/* if description retrieval fails, go with the cmd name */
-		if (upscli_sendline(&ups, buf, strlen(buf)) < 0) {
-			printf("<OPTION VALUE=\"%s\">%s</OPTION>\n", tmp, tmp);
-			continue;
-		}
-
-		if (upscli_readline(&ups, buf, sizeof(buf)) < 0) {
-			printf("<OPTION VALUE=\"%s\">%s</OPTION>\n", tmp, tmp);
-			continue;
-		}
-
-		q = strchr(buf, '"');
-
-		if (!q)	{	/* invalid/missing description, use cmd name */
-			printf("<OPTION VALUE=\"%s\">%s</OPTION>\n", tmp, tmp);
-			continue;
-		}
-
-		desc = ++q;
-		
-		q = strchr(desc, '"');
-		if (!q)	{	/* missing second quote - don't use it */
-			printf("<OPTION VALUE=\"%s\">%s</OPTION>\n", tmp, tmp);
-			continue;
-		}
-
-		*q = '\0';
-		printf("<OPTION VALUE=\"%s\">%s</OPTION>\n", tmp, desc);
-	}
-
-	printf("</SELECT>\n");
-	printf("</TD></TR>\n");
-
-	printf("<TR BGCOLOR=\"#60B0B0\">\n");
-	printf("<TD COLSPAN=\"2\" ALIGN=\"CENTER\">\n");
-	do_hidden("docmd");
-	printf("<INPUT TYPE=\"HIDDEN\" NAME=\"monups\" VALUE=\"%s\">\n", monups);
-	printf("<INPUT TYPE=\"SUBMIT\" VALUE=\"Issue command\">\n");
-	printf("<INPUT TYPE=\"RESET\" VALUE=\"Reset\">\n");
-	printf("</TD></TR>\n");
-	printf("</TABLE>\n");
-	printf("</FORM>\n");
-
-	printf("<TR><TD ALIGN=\"CENTER\">\n");
-	do_pickups("showcmds");
-	printf("</TD></TR>\n");
-
-	printf("</TABLE>\n");
-	printf("</BODY></HTML>\n");
-
-	upscli_disconnect(&ups);
-	exit(0);
-}
-
 static void print_cmd(const char *cmd)
 {
-	int	ret, numq, numa;
+	int	ret;
+	unsigned int	numq, numa;
 	char	**answer;
 	const	char	*query[4];
 
@@ -557,12 +360,21 @@ static void print_cmd(const char *cmd)
 	printf("<OPTION VALUE=\"%s\">%s</OPTION>\n", cmd, answer[3]);
 }
 
-static void new_showcmds(const char *desc)
+/* generate a list of instant commands */
+static void showcmds(void)
 {
-	int	ret, numq, numa;
+	int	ret;
+	unsigned int	numq, numa;
 	const	char	*query[2];
 	char	**answer;
 	struct	list_t	*lhead, *llast, *ltmp, *lnext;
+	char	*desc;
+
+	if (!checkhost(monups, &desc))
+		error_page("showsettings", "Access denied",
+			"Access to that host is not authorized");
+
+	upsd_connect();
 
 	llast = lhead = NULL;
 
@@ -573,12 +385,6 @@ static void new_showcmds(const char *desc)
 	ret = upscli_list_start(&ups, numq, query);
 
 	if (ret < 0) {
-
-		/* old upsd: fall back */
-
-		if (upscli_upserror(&ups) == UPSCLI_ERR_UNKCOMMAND)
-			old_showcmds(desc);
-
 		fprintf(stderr, "LIST CMD %s failed: %s\n",
 			upsname, upscli_strerror(&ups));
 
@@ -616,7 +422,7 @@ static void new_showcmds(const char *desc)
 
 	if (!lhead)
 		error_page("showcmds", "No instant commands supported",
-		          "This UPS doesn't support any instant commands.");
+			"This UPS doesn't support any instant commands.");
 
 	do_header("Instant commands");
 	printf("<FORM ACTION=\"upsset.cgi\" METHOD=\"POST\">\n");
@@ -625,7 +431,7 @@ static void new_showcmds(const char *desc)
 	/* include the description from checkhost() if present */
 	if (desc)
 		printf("<TR><TH BGCOLOR=\"#60B0B0\"COLSPAN=2>%s</TH></TR>\n",
-		       desc);
+			desc);
 
 	printf("<TR BGCOLOR=\"#60B0B0\" ALIGN=\"CENTER\">\n");
 	printf("<TD>Instant commands</TD>\n");
@@ -669,165 +475,9 @@ static void new_showcmds(const char *desc)
 	printf("</BODY></HTML>\n");
 
 	upscli_disconnect(&ups);
-	exit(0);
+	exit(EXIT_SUCCESS);
 }	
 
-/* generate a list of instant commands */
-static void showcmds(void)
-{
-	char	*desc;
-
-	if (!checkhost(monups, &desc))
-		error_page("showsettings", "Access denied",
-		          "Access to that host is not authorized");
-
-	upsd_connect();
-
-	if (!upsname)
-		old_showcmds(desc);
-	else
-		new_showcmds(desc);
-
-	/* NOTREACHED */
-}
-
-static void old_docmd(const char *desc, const char *cmd)
-{
-	char	buf[SMALLBUF];
-
-	/* send command, get response, show it */
-	if (upsname)
-		snprintf(buf, sizeof(buf), "INSTCMD %s@%s\n", cmd, upsname);
-	else
-		snprintf(buf, sizeof(buf), "INSTCMD %s\n", cmd);
-
-	if (upscli_sendline(&ups, buf, strlen(buf)) < 0) {
-		do_header("Error while issuing command");
-
-		start_table();    
-
-		printf("<TR><TD>Error sending command: %s\n</TD></TR>",
-			upscli_strerror(&ups));
-
-		printf("<TR><TD ALIGN=\"CENTER\" COLSPAN=2>\n");
-		do_pickups("showcmds");
-		printf("</TD></TR>\n");
-
-		printf("</TABLE>\n");
-
-		printf("</TD></TR></TABLE>\n");
-		printf("</BODY></HTML>\n");
-		upscli_disconnect(&ups);
-		exit(0);
-	}
-
-	if (upscli_readline(&ups, buf, sizeof(buf)) < 0) {
-		do_header("Error while reading command response");
-
-		start_table();    
-
-		printf("<TR><TD>Error reading command response: %s\n</TD></TR>",
-			upscli_strerror(&ups));
-
-		printf("<TR><TD ALIGN=\"CENTER\" COLSPAN=2>\n");
-		do_pickups("showcmds");
-		printf("</TD></TR>\n");
-
-		printf("</TABLE>\n");
-
-		printf("</TD></TR></TABLE>\n");
-		printf("</BODY></HTML>\n");
-		upscli_disconnect(&ups);
-		exit(0);
-	}
-
-	do_header("Issuing commands");
-	start_table();
-
-	printf("<TR><TD><PRE>\n");
-	printf("Sending command: %s\n", upscommand);
-	printf("Response: %s\n", buf);
-	printf("</PRE></TD></TR>\n");
-
-	printf("<TR><TD ALIGN=\"CENTER\" COLSPAN=2>\n");
-	do_pickups("showcmds");
-	printf("</TD></TR>\n");
-
-	printf("</TABLE>\n");
-	printf("</TD></TR></TABLE>\n");
-	printf("</BODY></HTML>\n");
-
-	upscli_disconnect(&ups);
-	exit(0);
-
-}
-
-static void new_docmd(const char *desc, const char *cmd)
-{
-	char	buf[SMALLBUF];
-
-	snprintf(buf, sizeof(buf), "INSTCMD %s %s\n", upsname, cmd);
-
-	if (upscli_sendline(&ups, buf, strlen(buf)) < 0) {
-		do_header("Error while issuing command");
-
-		start_table();    
-
-		printf("<TR><TD>Error sending command: %s\n</TD></TR>",
-			upscli_strerror(&ups));
-
-		printf("<TR><TD ALIGN=\"CENTER\" COLSPAN=2>\n");
-		do_pickups("showcmds");
-		printf("</TD></TR>\n");
-
-		printf("</TABLE>\n");
-
-		printf("</TD></TR></TABLE>\n");
-		printf("</BODY></HTML>\n");
-		upscli_disconnect(&ups);
-		exit(0);
-	}
-
-	if (upscli_readline(&ups, buf, sizeof(buf)) < 0) {
-		do_header("Error while reading command response");
-
-		start_table();    
-
-		printf("<TR><TD>Error reading command response: %s\n</TD></TR>",
-			upscli_strerror(&ups));
-
-		printf("<TR><TD ALIGN=\"CENTER\" COLSPAN=2>\n");
-		do_pickups("showcmds");
-		printf("</TD></TR>\n");
-
-		printf("</TABLE>\n");
-
-		printf("</TD></TR></TABLE>\n");
-		printf("</BODY></HTML>\n");
-		upscli_disconnect(&ups);
-		exit(0);
-	}
-
-	do_header("Issuing command");
-	start_table();
-
-	printf("<TR><TD><PRE>\n");
-	printf("Sending command: %s\n", upscommand);
-	printf("Response: %s\n", buf);
-	printf("</PRE></TD></TR>\n");
-
-	printf("<TR><TD ALIGN=\"CENTER\" COLSPAN=2>\n");
-	do_pickups("showcmds");
-	printf("</TD></TR>\n");
-
-	printf("</TABLE>\n");
-	printf("</TD></TR></TABLE>\n");
-	printf("</BODY></HTML>\n");
-
-	upscli_disconnect(&ups);
-	exit(0);
-}
-
 /* handle setting authentication data in the server */
 static void send_auth(const char *next)
 {
@@ -848,7 +498,7 @@ static void send_auth(const char *next)
 		/* test for old upsd that doesn't do USERNAME */
 		if (upscli_upserror(&ups) == UPSCLI_ERR_UNKCOMMAND) {
 			error_page(next, "Protocol mismatch",
-			          "upsd version too old - USERNAME not supported");
+				"upsd version too old - USERNAME not supported");
 		}
 
 		error_page(next, "Can't set user name", 
@@ -866,14 +516,13 @@ static void send_auth(const char *next)
 			"Password set failed: %s", upscli_strerror(&ups));
 }	
 
-/* take a command from the client and send it to upsd */
 static void docmd(void)
 {
-	char	*desc;
+	char	buf[SMALLBUF], *desc;
 
 	if (!checkhost(monups, &desc))
 		error_page("showsettings", "Access denied",
-		          "Access to that host is not authorized");
+			"Access to that host is not authorized");
 
 	/* the user is messing with us */
 	if (!upscommand)	
@@ -889,180 +538,72 @@ static void docmd(void)
 
 	send_auth("showcmds");
 
-	if (!upsname)
-		old_docmd(desc, upscommand);
+	snprintf(buf, sizeof(buf), "INSTCMD %s %s\n", upsname, upscommand);
 
-	if (!strchr(upscommand, '.'))
-		old_docmd(desc, upscommand);
+	if (upscli_sendline(&ups, buf, strlen(buf)) < 0) {
+		do_header("Error while issuing command");
 
-	new_docmd(desc, upscommand);
+		start_table();
 
-	/* NOTREACHED */
-}
-
-/* dump a table of all read/write variables and their current values */
-static void old_showsettings(const char *desc)
-{
-	char	vars[SMALLBUF], temp[SMALLBUF], out[SMALLBUF], *v, 
-		*type, *ptr, *vptr;
-	int	typelen;
-
-	if (upscli_getlist(&ups, upsname, UPSCLI_LIST_RW, vars, 
-		sizeof(vars)) < 0) {
-
-		error_page("showsettings", "Server protocol error",
-		         "Unable to get variable list - %s\n",
-		         upscli_strerror(&ups));
-
-		/* NOTREACHED */
-	}
-
-	if (strlen(vars) == 0) {
-		error_page("showsettings", "No read/write variables", 
-		          "This UPS has no read/write variables.");
-		/* NOTREACHED */
-	}
-
-	/* no spaces = seriously broken string */
-	if (!strchr(vars, ' ')) {		
-		error_page("showsettings", "Server protocol error", 
-		          "Unable to get variable list - broken string");
-		/* NOTREACHED */
-	}
-
-	do_header("Current settings");
-	printf("<FORM ACTION=\"upsset.cgi\" METHOD=\"POST\">\n");
-	start_table();
-
-	/* include the description from checkhost() if present */
-	if (desc)
-		printf("<TR><TH BGCOLOR=\"#60B0B0\"COLSPAN=2>%s</TH></TR>\n",
-		        desc);
-
-	printf("<TR BGCOLOR=\"#60B0B0\">\n");
-	printf("<TH>Setting</TH>\n");
-	printf("<TH>Value</TH></TR>\n");
-
-	v = vars;
-
-	while (v != NULL) {
-		vptr = strchr(v, ' ');
-		if (vptr)
-			*vptr++ = '\0';
-
-		/* get description */
-		snprintf(out, sizeof(out), "VARDESC %s\n", v);
-		if (upscli_sendline(&ups, out, strlen(out)) < 0) {
-			printf("Can't get VARDESC\n");
-
-			fprintf(stderr, "Can't get VARDESC of %s: %s\n",
-				v, upscli_strerror(&ups));
-			return;
-		}
-
-		if (upscli_readline(&ups, temp, sizeof(temp)) < 0) {
-			printf("Can't get VARDESC\n");
-
-			fprintf(stderr, "Can't get VARDESC of %s: %s\n",
-				v, upscli_strerror(&ups));
-			return;
-		}
-
-		printf("<TR BGCOLOR=\"#60B0B0\" ALIGN=\"CENTER\">\n");
-
-		/* strip off leading/trailing quotes */
-		ptr = strchr(temp, '"');
-		if (ptr != NULL) {
-			char	*ptr2;
-
-			ptr++;
-			ptr2 = strchr(ptr, '"');
-
-			if (ptr2)
-				*ptr2 = '\0';
-			else {
-				printf("Unbalanced quotes from server\n");
-				exit(0);
-			}
-		}
-
-		printf("<TD>%s</TD>\n", ptr);
-
-		/* now get variable type */
-		snprintf(out, sizeof(out),
-			(upsname ? "VARTYPE %s@%s\n" : "VARTYPE %s\n"), v, upsname);
-
-		if (upscli_sendline(&ups, out, strlen(out)) < 0) {
-			printf("Can't get VARTYPE\n");
-
-			fprintf(stderr, "Can't get VARTYPE %s: %s\n",
-				v, upscli_strerror(&ups));
-			return;
-		}
-
-		if (upscli_readline(&ups, temp, sizeof(temp)) < 0) {
-			printf("Can't get VARTYPE\n");
-
-			fprintf(stderr, "Can't get VARTYPE %s: %s\n",
-				v, upscli_strerror(&ups));
-			return;
-		}
-
-		type = NULL;
-		typelen = 0;
-
-		ptr = strchr(temp, ' ');
-		if (ptr) {
-			*ptr++ = '\0';
-			type = ptr;
-			ptr = strchr(type, ' ');
-			if (ptr) {
-				*ptr++ = '\0';
-				typelen = strtol(ptr, (char **) NULL, 10);
-			}
-		}
-
-		if (type) {
-			printf("<TD>\n");
-
-			if (!strcmp(type, "ENUM"))
-				do_enum(v);
-
-			if (!strcmp(type, "STRING"))
-				do_string(v, typelen);
-		}
-		else
-			printf("Broken type received from server\n");
+		printf("<TR><TD>Error sending command: %s\n</TD></TR>",
+			upscli_strerror(&ups));
 
+		printf("<TR><TD ALIGN=\"CENTER\" COLSPAN=2>\n");
+		do_pickups("showcmds");
 		printf("</TD></TR>\n");
 
-		v = vptr;		
+		printf("</TABLE>\n");
+
+		printf("</TD></TR></TABLE>\n");
+		printf("</BODY></HTML>\n");
+		upscli_disconnect(&ups);
+		exit(EXIT_SUCCESS);
 	}
 
-	printf("<TR BGCOLOR=\"#60B0B0\">\n");
-	printf("<TD COLSPAN=\"2\" ALIGN=\"CENTER\">\n");
-	do_hidden("savesettings");
-	printf("<INPUT TYPE=\"HIDDEN\" NAME=\"monups\" VALUE=\"%s\">\n", monups);
-	printf("<INPUT TYPE=\"SUBMIT\" VALUE=\"Save changes\">\n");
-	printf("<INPUT TYPE=\"RESET\" VALUE=\"Reset\">\n");
-	printf("</TD></TR>\n");
-	printf("</TABLE>\n");
-	printf("</FORM>\n");
+	if (upscli_readline(&ups, buf, sizeof(buf)) < 0) {
+		do_header("Error while reading command response");
 
-	printf("<TR><TD ALIGN=\"CENTER\">\n");
-	do_pickups("showsettings");
+		start_table();
+
+		printf("<TR><TD>Error reading command response: %s\n</TD></TR>",
+			upscli_strerror(&ups));
+
+		printf("<TR><TD ALIGN=\"CENTER\" COLSPAN=2>\n");
+		do_pickups("showcmds");
+		printf("</TD></TR>\n");
+
+		printf("</TABLE>\n");
+
+		printf("</TD></TR></TABLE>\n");
+		printf("</BODY></HTML>\n");
+		upscli_disconnect(&ups);
+		exit(EXIT_SUCCESS);
+	}
+
+	do_header("Issuing command");
+	start_table();
+
+	printf("<TR><TD><PRE>\n");
+	printf("Sending command: %s\n", upscommand);
+	printf("Response: %s\n", buf);
+	printf("</PRE></TD></TR>\n");
+
+	printf("<TR><TD ALIGN=\"CENTER\" COLSPAN=2>\n");
+	do_pickups("showcmds");
 	printf("</TD></TR>\n");
 
 	printf("</TABLE>\n");
+	printf("</TD></TR></TABLE>\n");
 	printf("</BODY></HTML>\n");
 
 	upscli_disconnect(&ups);
-	exit(0);
+	exit(EXIT_SUCCESS);
 }
 
 static const char *get_data(const char *type, const char *varname)
 {
-	int	ret, numq, numa;
+	int	ret;
+	unsigned int	numq, numa;
 	char	**answer;
 	const	char	*query[4];
 
@@ -1080,7 +621,7 @@ static const char *get_data(const char *type, const char *varname)
 	return answer[3];
 }
 
-static void new_do_string(const char *varname, int maxlen)
+static void do_string(const char *varname, int maxlen)
 {
 	const	char	*val;
 
@@ -1088,7 +629,7 @@ static void new_do_string(const char *varname, int maxlen)
 
 	if (!val) {
 		printf("Unavailable\n");
-		fprintf(stderr, "new_do_string: can't get current value of %s\n",
+		fprintf(stderr, "do_string: can't get current value of %s\n",
 			varname);
 		return;
 	}
@@ -1097,9 +638,10 @@ static void new_do_string(const char *varname, int maxlen)
 		"SIZE=\"%d\">\n", varname, val, maxlen);
 }
 
-static void new_do_enum(const char *varname)
+static void do_enum(const char *varname)
 {
-	int	ret, numq, numa;
+	int	ret;
+	unsigned int	numq, numa;
 	char	**answer, *val;
 	const	char	*query[4], *tmp;
 
@@ -1162,7 +704,8 @@ static void new_do_enum(const char *varname)
 
 static void do_type(const char *varname)
 {
-	int	i, ret, numq, numa;
+	int	ret;
+	unsigned int	i, numq, numa;
 	char	**answer;
 	const	char	*query[4];
 
@@ -1182,7 +725,7 @@ static void do_type(const char *varname)
 	for (i = 3; i < numa; i++) {
 
 		if (!strcasecmp(answer[i], "ENUM")) {
-			new_do_enum(varname);
+			do_enum(varname);
 			return;
 		}
 
@@ -1194,7 +737,7 @@ static void do_type(const char *varname)
 			*ptr++ = '\0';
 			len = strtol(ptr, (char **) NULL, 10);
 
-			new_do_string(varname, len);
+			do_string(varname, len);
 			return;
 		}
 
@@ -1232,23 +775,18 @@ static void print_rw(const char *upsname, const char *varname)
 
 static void showsettings(void)
 {
-	int	ret, numq, numa;
+	int	ret;
+	unsigned int	numq, numa;
 	const	char	*query[2];
 	char	**answer, *desc = NULL;
 	struct	list_t	*lhead, *llast, *ltmp, *lnext;
 
 	if (!checkhost(monups, &desc))
 		error_page("showsettings", "Access denied",
-		          "Access to that host is not authorized");
+			"Access to that host is not authorized");
 
 	upsd_connect();
 
-	/* old default upsname scheme = use old protocol commands */
-	if ((!upsname) || (!strcmp(upsname, ""))) {
-		old_showsettings(desc);
-		return;
-	}
-
 	query[0] = "RW";
 	query[1] = upsname;
 	numq = 2;
@@ -1256,13 +794,6 @@ static void showsettings(void)
 	ret = upscli_list_start(&ups, numq, query);
 
 	if (ret < 0) {
-
-		/* old upsd --> fall back on old LISTRW technique */
-		if (upscli_upserror(&ups) == UPSCLI_ERR_UNKCOMMAND) {
-			old_showsettings(desc);
-			return;
-		}
-
 		fprintf(stderr, "LIST RW %s failed: %s\n",
 			upsname, upscli_strerror(&ups));
 
@@ -1299,10 +830,10 @@ static void showsettings(void)
 	start_table();
 
 	/* include the description from checkhost() if present */
-        if (desc)
+	if (desc)
 		printf("<TR><TH BGCOLOR=\"#60B0B0\"COLSPAN=2>%s</TH></TR>\n",
 			desc);
-        
+
 	printf("<TR BGCOLOR=\"#60B0B0\">\n");
 	printf("<TH>Setting</TH>\n");
 	printf("<TH>Value</TH></TR>\n");
@@ -1339,40 +870,10 @@ static void showsettings(void)
 	printf("</BODY></HTML>\n");
 
 	upscli_disconnect(&ups);
-	exit(0);
+	exit(EXIT_SUCCESS);
 }
 
-static void old_setvar(const char *var, const char *val)
-{
-	char	buf[SMALLBUF];
-
-	upscli_getvar(&ups, upsname, var, buf, sizeof(buf));
-
-	/* don't send a SET if it hasn't chnaged */
-	if (strcmp(val, buf))
-		return;
-
-	printf("set %s to %s (was %s)\n", var, val, buf);
-
-	if (upsname)
-		snprintf(buf, sizeof(buf), "SET %s@%s %s\n", var, upsname, val);
-	else
-		snprintf(buf, sizeof(buf), "SET %s %s\n", var, val);
-
-	if (upscli_sendline(&ups, buf, strlen(buf)) < 0) {
-		printf("Error: SET failed: %s\n", upscli_strerror(&ups));
-		return;
-	}
-
-	if (upscli_readline(&ups, buf, sizeof(buf)) < 0) {
-		printf("Error: SET failed: %s\n", upscli_strerror(&ups));
-		return;
-	}
-
-	printf("response: %s\n", buf);
-}
-
-static void new_setvar(const char *var, const char *val)
+static int setvar(const char *var, const char *val)
 {
 	char	buf[SMALLBUF], enc[SMALLBUF];
 	const	char	*tmp;
@@ -1382,12 +883,12 @@ static void new_setvar(const char *var, const char *val)
 
 	if (!tmp) {
 		printf("Can't get old value for %s, aborting SET\n", var);
-		return;
+		return 0;
 	}
 
 	/* don't send a SET if it hasn't chnaged */
-	if (!strcmp(tmp, var))
-		return;
+	if (!strcmp(tmp, val))
+		return 0;
 
 	printf("set %s to %s (was %s)\n", var, val, tmp);
 
@@ -1396,31 +897,33 @@ static void new_setvar(const char *var, const char *val)
 
 	if (upscli_sendline(&ups, buf, strlen(buf)) < 0) {
 		printf("Error: SET failed: %s\n", upscli_strerror(&ups));
-		return;
+		return 0;
 	}
 
 	if (upscli_readline(&ups, buf, sizeof(buf)) < 0) {
 		printf("Error: SET failed: %s\n", upscli_strerror(&ups));
-		return;
+		return 0;
 	}
 
 	if (strncmp(buf, "OK", 2) != 0) {
 		printf("Unexpected response: %s\n", buf);
-		return;
+		return 0;
 	}
 
 	printf("OK\n");
+	return 1;
 }
 
 /* turn a form submission of settings into SET commands for upsd */
 static void savesettings(void)
 {
+	int	changed = 0;
 	char	*desc;
 	uvtype	*upsvar;
 
 	if (!checkhost(monups, &desc)) 
 		error_page("showsettings", "Access denied",
-		          "Access to that host is not authorized");
+			"Access to that host is not authorized");
 
 	upsd_connect();
 
@@ -1434,25 +937,16 @@ static void savesettings(void)
 	printf("<TR><TD><PRE>\n");
 
 	while (upsvar) {
-
-		/* fallback on old methods */
-		if (!upsname) {
-			old_setvar(upsvar->var, upsvar->value);
-			upsvar = upsvar->next;
-			continue;
-		}
-
-		/* ditto for old variable names */
-		if (!strchr(upsvar->var, '.')) {
-			old_setvar(upsvar->var, upsvar->value);
-			upsvar = upsvar->next;
-			continue;
-		}
-
-		new_setvar(upsvar->var, upsvar->value);
+		changed += setvar(upsvar->var, upsvar->value);
 		upsvar = upsvar->next;
 	}
 
+	if (changed == 0)
+		printf("No settings changed.\n");
+	else
+		printf("Updated %d setting%s.\n",
+			changed, changed == 1 ? "" : "s");
+
 	printf("</PRE></TD></TR>\n");
 
 	printf("<TR><TD ALIGN=\"CENTER\" COLSPAN=\"2\">\n");
@@ -1464,7 +958,7 @@ static void savesettings(void)
 	printf("</BODY></HTML>\n");
 
 	upscli_disconnect(&ups);
-	exit(0);
+	exit(EXIT_SUCCESS);
 }
 
 static void initial_pickups(void)
@@ -1481,10 +975,10 @@ static void initial_pickups(void)
 	printf("</BODY></HTML>\n");
 
 	upscli_disconnect(&ups);
-	exit(0);
+	exit(EXIT_SUCCESS);
 }
 
-void upsset_conf_err(const char *errmsg)
+static void upsset_conf_err(const char *errmsg)
 {
 	upslogx(LOG_ERR, "Fatal error in parseconf(upsset.conf): %s", errmsg);
 }
@@ -1509,7 +1003,7 @@ static void check_conf(void)
 
 		/* leave something in the httpd log for the admin */
 		fprintf(stderr, "upsset.conf does not exist to permit execution\n");
-		exit(0);
+		exit(EXIT_FAILURE);
 	}
 
 	while (pconf_file_next(&ctx)) {
@@ -1540,7 +1034,7 @@ static void check_conf(void)
 	/* leave something in the httpd log for the admin */
 	fprintf(stderr, "upsset.conf does not permit execution\n");
 
-	exit(0);
+	exit(EXIT_FAILURE);
 }	
 
 int main(int argc, char **argv)
