@@ -18,7 +18,6 @@
 */
 
 #include "common.h"
-#include "shared.h"
 
 #include <ctype.h>
 #include <syslog.h>
@@ -26,19 +25,19 @@
 #include <grp.h>
 
 	int	nut_debug_level = 0;
-	int     upslog_flags = UPSLOG_STDERR;
+	static	int	upslog_flags = UPSLOG_STDERR;
 
-void xbit_set(int *val, int flag)
+static void xbit_set(int *val, int flag)
 {
 	*val = (*val |= flag);
 }
 
-void xbit_clear(int *val, int flag)
+static void xbit_clear(int *val, int flag)
 {
 	*val = (*val ^= (*val & flag));
 }
 
-int xbit_test(int val, int flag)
+static int xbit_test(int val, int flag)
 {
 	return ((val & flag) == flag);
 }
@@ -80,7 +79,7 @@ void background(void)
 	close(2);
 
 	if (pid != 0) 
-		_exit(0);		/* parent */
+		_exit(EXIT_SUCCESS);		/* parent */
 
 	/* child */
 
@@ -208,7 +207,7 @@ void sendsignalfn(const char *pidfn, int sig)
 int snprintfcat(char *dst, size_t size, const char *fmt, ...)
 {
 	va_list ap;
-	int len = strlen(dst);
+	size_t len = strlen(dst);
 	int ret;
 
 	size--;
@@ -241,12 +240,16 @@ const char *xbasename(const char *file)
 	return p + 1;
 }
 
-void vupslog(int priority, const char *fmt, va_list va, int use_strerror)
+static void vupslog(int priority, const char *fmt, va_list va, int use_strerror)
 {
-	char buf[LARGEBUF];
+	int	ret;
+	char	buf[LARGEBUF];
 
-	if (vsnprintf(buf, sizeof(buf), fmt, va) >= sizeof(buf))
-		; /* overflow should be handled */
+	ret = vsnprintf(buf, sizeof(buf), fmt, va);
+
+	if ((ret < 0) || (ret >= (int) sizeof(buf)))
+		syslog(LOG_WARNING, "vupslog: vsnprintf needed more than %d bytes",
+			LARGEBUF);
 
 	if (use_strerror)
 		snprintfcat(buf, sizeof(buf), ": %s", strerror(errno));
@@ -333,7 +336,7 @@ void upsdebugx(int level, const char *fmt, ...)
 	va_end(va);
 }
 
-void vfatal(const char *fmt, va_list va, int use_strerror)
+static void vfatal(const char *fmt, va_list va, int use_strerror)
 {
 	if (xbit_test(upslog_flags, UPSLOG_STDERR_ON_FATAL))
 		xbit_set(&upslog_flags, UPSLOG_STDERR);
@@ -351,7 +354,7 @@ void fatal(const char *fmt, ...)
 	vfatal(fmt, va, 1);
 	va_end(va);
 
-	exit(1);
+	exit(EXIT_FAILURE);
 }
 
 void fatalx(const char *fmt, ...)
@@ -362,7 +365,7 @@ void fatalx(const char *fmt, ...)
 	vfatal(fmt, va, 0);
 	va_end(va);
 
-	exit(1);
+	exit(EXIT_FAILURE);
 }
 
 static const char *oom_msg = "Out of memory";
@@ -403,18 +406,17 @@ char *xstrdup(const char *string)
 	return p;
 }
 
-#ifndef HAVE_CFMAKERAW
-
-int cfmakeraw(struct termios *termios_p)
+/* modify in - strip all trailing instances of <sep> */
+void rtrim(char *in, char sep)
 {
-	termios_p->c_iflag &= ~(IGNBRK|BRKINT|PARMRK|ISTRIP
-				|INLCR|IGNCR|ICRNL|IXON);
-	termios_p->c_oflag &= ~OPOST;
-	termios_p->c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);
-	termios_p->c_cflag &= ~(CSIZE|PARENB);
-	termios_p->c_cflag |= CS8;
+	char	*p = NULL;
 
-	return 0;
+	p = &in[strlen(in) - 1];
+
+	while (p >= in) {
+		if (*p != sep)
+			return;
+
+		*p-- = '\0';
+	}
 }
-
-#endif
