@@ -1,7 +1,7 @@
 /* belkin.c - model specific routines for Belkin Smart-UPS units.
 
    Copyright (C) 2000 Marcus Mller <marcus@ebootis.de>
-     
+
    based on:
 
    apcsmart.c - model specific routines for APC smart protocol units
@@ -24,11 +24,18 @@
 */
 
 #include "main.h"
+#include "serial.h"
+
 #include "belkin.h"
 
-#define DRV_VERSION "0.11"
+#define DRV_VERSION "0.21"
 
-int init_communication(void)
+static void send_belkin_command(char cmd, const char *subcmd, const char *data)
+{
+	ser_send(upsfd, "~00%c%03d%s%s", cmd, strlen(data) + 3, subcmd, data);
+}
+
+static int init_communication(void)
 {
 	int	i;
 	int	res;
@@ -37,7 +44,7 @@ int init_communication(void)
 	res = -1;
 	for (i = 1; i <= 10 && res == -1; i++) {
 		send_belkin_command(STATUS,MANUFACTURER,"");
-    		res = get_belkin_reply(temp);
+		res = get_belkin_reply(temp);
 	}
 
 	if (res == -1 || strcmp(temp,"BELKIN")) 
@@ -46,7 +53,41 @@ int init_communication(void)
 	return 0;
 }
 
-void do_status(void)
+static char *get_belkin_field(const char *in, char *out, size_t outlen, 
+	size_t num)
+{
+	size_t	i, c = 1;
+	char	*ptr;
+
+	/* special case */
+	if (num == 1) {
+		snprintf(out, outlen, "%s", in);
+		ptr = strchr(out, ';');
+
+		if (ptr)
+			*ptr = '\0';
+
+		return out;
+	}		
+
+	for (i = 0; i < strlen(in); i++) {
+		if (in[i] == ';')
+			c++;
+
+		if (c == num) {
+			snprintf(out, outlen, "%s", &in[i + 1]);
+			ptr = strchr(out, ';');
+
+			if (ptr)
+				*ptr = '\0';
+			return out;
+		}
+	}
+
+	return NULL;
+}
+
+static int do_status(void)
 {
 	char	temp[SMALLBUF], st[SMALLBUF];
 	int	res;
@@ -55,7 +96,7 @@ void do_status(void)
 	res = get_belkin_reply(temp);
 	if (res == -1) {
 		dstate_datastale();
-		return;
+		return 0;
 	}
 
 	status_init();
@@ -72,15 +113,18 @@ void do_status(void)
 
 			send_belkin_command(STATUS,STAT_BATTERY,"");
 			res = get_belkin_reply(temp);
-			if (res == -1)
-				return;
+
+			if (res == -1) {
+				dstate_datastale();
+				return 0;
+			}
 
 			get_belkin_field(temp, st, sizeof(st), 10);
 			res = atoi(st);
 			get_belkin_field(temp, st, sizeof(st), 2);
 
 			if (*st == '1' || res < LOW_BAT) 
-				status_set("LB");		/* low battery */
+				status_set("LB");	/* low battery */
 		}
 		else
 			status_set("OL");	/* on line */
@@ -88,9 +132,41 @@ void do_status(void)
 
 	status_commit();
 	dstate_dataok();
+
+	return 1;
 }
 
-int init_ups_data(void)
+static int do_broken_rat(char *buf)
+{
+	int	ret, cnt;
+	char	tmp[8];
+
+	usleep(25000);
+
+	ret = ser_get_buf_len(upsfd, tmp, 7, 3, 0);
+
+	if (ret != 7)
+		return -1;
+
+	tmp[7] = '\0';
+	cnt = atoi(tmp + 4);
+
+	if ((cnt < 1) || (cnt > 255))
+		return -1;
+
+	usleep(5000 * cnt);
+
+	/* firmware 001 only sends 50 bytes instead of the proper 53 */
+	if (cnt == 53)
+		cnt = 50;
+
+	ret = ser_get_buf_len(upsfd, buf, 50, cnt, 0);
+	buf[cnt] = 0;
+
+	return ret;
+}	
+
+static int init_ups_data(void)
 {
 	int	res;
 	double	low, high;
@@ -106,24 +182,32 @@ int init_ups_data(void)
 	send_belkin_command(STATUS, VERSION, "");
 	res = get_belkin_reply(temp);
 	if (res == -1)
-		return (res);
+		return res;
 
 	dstate_setinfo("ups.firmware", "%s", temp);
 
+	/* deal with stupid firmware that breaks RAT */
+
 	send_belkin_command(STATUS, RATING, "");
-	res = get_belkin_reply(temp);
 
-	get_belkin_field(temp, st, sizeof(st), 8);
-	low = atof(st) / 0.88;
+	if (!strcmp(temp, "001"))
+		res = do_broken_rat(temp);
+	else
+		res = get_belkin_reply(temp);
 
-	get_belkin_field(temp, st, sizeof(st), 9);
-	high = atof(st) * 0.88;
+	if (res > 0) {
+		get_belkin_field(temp, st, sizeof(st), 8);
+		low = atof(st) / 0.88;
+
+		get_belkin_field(temp, st, sizeof(st), 9);
+		high = atof(st) * 0.88;
+
+		dstate_setinfo("input.transfer.low", "%03.1f", low);
+		dstate_setinfo("input.transfer.high", "%03.1f", high);
+	}
 
 	tcflush(upsfd,TCIOFLUSH);
 
-	dstate_setinfo("input.transfer.low", "%03.1f", low);
-	dstate_setinfo("input.transfer.high", "%03.1f", high);
-
 	dstate_addcmd("load.off");
 	dstate_addcmd("load.on");
 	upsdrv_updateinfo();
@@ -137,7 +221,8 @@ void upsdrv_updateinfo(void)
 	double	val;
 	char	temp[SMALLBUF], st[SMALLBUF];
 
-	do_status();
+	if (!do_status())
+		return;
 
 	send_belkin_command(STATUS, STAT_INPUT, "");
 	res = get_belkin_reply(temp);
@@ -148,6 +233,10 @@ void upsdrv_updateinfo(void)
 	val = atof(st) / 10;
 	dstate_setinfo("input.voltage", "%05.1f", val);
 
+	get_belkin_field(temp, st, sizeof(st), 2);
+	val = atof(st) / 10;
+	dstate_setinfo("input.frequency", "%.1f", val);
+
 	send_belkin_command(STATUS,STAT_BATTERY, "");
 	res = get_belkin_reply(temp);
 	if (res == -1)
@@ -157,6 +246,10 @@ void upsdrv_updateinfo(void)
 	val = atof(st);
 	dstate_setinfo("battery.charge", "%03.0f", val);
 
+	get_belkin_field(temp, st, sizeof(st), 9);
+	val = atof(st);
+	dstate_setinfo("battery.temperature", "%03.0f", val);
+
 	get_belkin_field(temp, st, sizeof(st), 7);
 	val = atof(st) / 10;
 	dstate_setinfo("battery.voltage", "%4.1f", val);
@@ -172,7 +265,7 @@ void upsdrv_updateinfo(void)
 
 	get_belkin_field(temp, st, sizeof(st), 2);
 	val = atof(st) / 10;
-	dstate_setinfo("input.frequency", "%.1f", val);
+	dstate_setinfo("output.frequency", "%.1f", val);
 
 	get_belkin_field(temp, st, sizeof(st), 4);
 	val = atof(st) / 10;
@@ -183,88 +276,70 @@ void upsdrv_updateinfo(void)
 	dstate_setinfo("ups.load", "%03.0f", val);
 }
 
-
-void get_belkin_field(char *temp, char *data, size_t datalen, int n)
+static int get_belkin_reply(char *buf)
 {
-	char	st[SMALLBUF];
-	int	i = 0, f = 1;
+	int	ret, cnt;
+	char	tmp[8];
 
-	strlcpy(st, temp, sizeof(st));
+	usleep(25000);
 
-	while (f < n) {
-		while (st[i] && st[i] != ';') 
-			i++;
+	/* pull first 7 bytes to get data length - like ~00S004 */
+	ret = ser_get_buf_len(upsfd, tmp, 7, 3, 0);
 
-		st[i++] = 0;
-		f++;
+	if (ret != 7) {
+		ser_comm_fail("Initial read returned %d bytes", ret);
+		return -1;
 	}
 
-	f = i;
+	tmp[7] = 0;
+	cnt = atoi(tmp + 4);
 
-	while (st[i] && st[i] != ';') 
-		i++;
+	if ((cnt < 1) || (cnt > 255))
+		return -1;
 
-	st[i] = 0;
+	/* give it time to respond to us */
+	usleep(5000 * cnt);
 
-	snprintf(data, datalen, "%s", st+f);
-}
+	ret = ser_get_buf_len(upsfd, buf, cnt, 3, 0);
 
-int get_belkin_reply(char *buf)
-{
-	int	res, cnt;
-
-	res = recvbinary(buf, 7);
-
-	if (res)
-		return res;
-
-	buf[7] = 0;
-	cnt = atoi(buf + 4);
-	res = recvbinary(buf, cnt);
 	buf[cnt] = 0;
 
-	return res;
-}
+	if (ret != cnt) {
+		ser_comm_fail("Second read returned %d bytes, expected %d",
+			ret, cnt);
+		return -1;
+	}
 
-void send_belkin_command(char command, const char *subcommand, const char *data)
-{
-	upssend("~00%c%03d%s%s", command, strlen(data) + 3, subcommand, data);
+	ser_comm_good();
+
+	return ret;
 }
 
 /* power down the attached load immediately */
 void upsdrv_shutdown(void)
 {
 	int	res;
-	char	temp[SMALLBUF], st[SMALLBUF];
 
 	res = init_communication();
-	if (res == -1) {
+	if (res == -1)
 		printf("Detection failed.  Trying a shutdown command anyway.\n");
-		send_belkin_command(CONTROL, POWER_OFF, "1;1");
-		exit(-1);
-	}
 
-	send_belkin_command(STATUS, STAT_STATUS, "");
-	res = get_belkin_reply(temp);
-	get_belkin_field(temp, st, sizeof(st), 2);
+	/* tested on a F6C525-SER: this works when OL and OB */
 
-        if (*st == '1') {
-		printf("On battery - sending shutdown comand...\n");
-		send_belkin_command(CONTROL,POWER_OFF,"1;1");
-		return;
-	}
+	/* shutdown type 2 (UPS system) */
+	send_belkin_command(CONTROL, "SDT", "2");
 
-	printf("Power restored - sending shutdown+return command...\n");
+	/* SDR means "do SDT and SDA, then reboot after n minutes" */
+	send_belkin_command(CONTROL, "SDR", "1");
 
-	/* turn off the outlet in 5 seconds */
-	send_belkin_command(CONTROL, POWER_OFF, "1;5");
+	printf("UPS should power off load in 5 seconds\n");
 
-	/* ... and turn it back on 10 seconds after that */ 
-	send_belkin_command(CONTROL, POWER_ON, "1;15");
+	/* shutdown in 5 seconds */
+	send_belkin_command(CONTROL, "SDA", "5");
 }
 
 /* handle the "load.off" with some paranoia */
-void do_off(void)
+static void do_off(void)
 {
 	static	time_t lastcmd = 0;
 	time_t	now, elapsed;
@@ -279,18 +354,17 @@ void do_off(void)
 	if ((elapsed < MINCMDTIME) || (elapsed > MAXCMDTIME)) {
 
 		/* FUTURE: tell the user (via upsd) to try it again */
-		/* msgreply(UPSMSG_REPAGAIN); */
 		return;
 	}
 #endif
 
 	upslogx(LOG_INFO, "Sending powerdown command to UPS\n");
-        send_belkin_command(CONTROL,POWER_OFF,"1;1");
+	send_belkin_command(CONTROL,POWER_OFF,"1;1");
 	usleep(1500000);
-        send_belkin_command(CONTROL,POWER_OFF,"1;1");
+	send_belkin_command(CONTROL,POWER_OFF,"1;1");
 }
 
-int instcmd(const char *cmdname, const char *extra)
+static int instcmd(const char *cmdname, const char *extra)
 {
 	if (!strcasecmp(cmdname, "load.off")) {
 		do_off();
@@ -298,7 +372,7 @@ int instcmd(const char *cmdname, const char *extra)
 	}
 
 	if (!strcasecmp(cmdname, "load.on")) {
-	        send_belkin_command(CONTROL,POWER_ON,"1;1");
+		send_belkin_command(CONTROL,POWER_ON,"1;1");
 		return STAT_INSTCMD_HANDLED;
 	}
 
@@ -306,55 +380,7 @@ int instcmd(const char *cmdname, const char *extra)
 	return STAT_INSTCMD_UNKNOWN;
 }
 
-/* install pointers to functions for msg handlers called from msgparse */
-void setuphandlers(void)
-{
-	upsh.new_instcmd = instcmd;
-}
-
-int send_string(char *data)
-{
-	tcflush(upsfd, TCIFLUSH);
-	return(write(upsfd, data, strlen(data)));
-}
-
-int recvbinary(char *buf, int buflen)
-{
-	unsigned char	in;
-	int	ret, counter = 0, retval = 0;
-	struct	sigaction sa;
-	sigset_t	bel_sigmask;
- 
-	sa.sa_handler = timeout;
-	sigemptyset(&bel_sigmask);
-	sa.sa_mask = bel_sigmask;
-	sa.sa_flags = 0;
-	sigaction(SIGALRM, &sa, NULL);
-
-	alarm(3);
-
-	while (counter < buflen) {
-		ret = read(upsfd, &in, 1);
-
-		if (ret > 0) {
-			buf[counter] = in;
-			counter++;
-			nolongertimeout();	   
-		}
-		else {
-			upslogx(LOG_DEBUG, "error reading from serial device!");
-			retval = -1;
-			break;
-		}
-	}
-	
-	alarm(0);
-	signal(SIGALRM, SIG_IGN);
-
-	return retval;
-}
-
-void set_serialDTR0RTS1(void)
+static void set_serialDTR0RTS1(void)
 {
 	int dtr_bit = TIOCM_DTR;
 	int rts_bit = TIOCM_RTS;
@@ -381,7 +407,8 @@ void upsdrv_makevartable(void)
 /* prep the serial port */
 void upsdrv_initups(void)
 {
-	open_serial(device_path, B2400);
+	upsfd = ser_open(device_path);
+	ser_set_speed(upsfd, device_path, B2400);
 	
 	set_serialDTR0RTS1();
 
@@ -398,12 +425,10 @@ void upsdrv_initinfo(void)
 		printf("Unable to detect an Belkin Smart protocol UPS on port %s\n", 
 			device_path);
 		printf("Check the cabling, port name or model name and try again\n");
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 
-	/* manufacturer ID - hardcoded in this particular module */
 	dstate_setinfo("ups.mfr", "BELKIN", 0, 0);
-
 	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
 
 	/* see what's out there */
@@ -412,9 +437,10 @@ void upsdrv_initinfo(void)
 	printf("Detected %s on %s\n", dstate_getinfo("ups.model"),
 		device_path);
 
-	setuphandlers();
+	upsh.instcmd = instcmd;
 }
 
 void upsdrv_cleanup(void)
 {
+	ser_close(upsfd, device_path);
 }
