@@ -27,9 +27,10 @@
 
 */
 
-#define DRV_VERSION "0.01"
+#define DRV_VERSION "0.02"
 
 #include "main.h"
+#include "serial.h"
 
 #define ENDCHAR		'\r'
 #define IGNCHARS	"\012"
@@ -75,7 +76,7 @@ void  upsdrv_initinfo (void)
 	    dstate_setinfo("ups.model", "Micro Ferrups (MD) %d", fc.va);
 	    break;
           default:
-	    exit(1); /* Will never get here, upsdrv_initups() will catch */
+	    exit(EXIT_FAILURE); /* Will never get here, upsdrv_initups() will catch */
         } 
 	fprintf(stderr, "Best Power %s detected\n", 
 		dstate_getinfo("ups.model"));
@@ -93,15 +94,15 @@ time^M^M^JFeb 20, 22:13:32^M^J^M^J=>id^M^JUnit ID "ME3.1K12345"^M^J^M^J=>
 ----------------------------------------------------
 */
 
-int execute(char *cmd, char *result, int resultsize) 
+static int execute(const char *cmd, char *result, int resultsize) 
 {
   int ret;
   char buf[256];
   
-  upssend(cmd);
-  upsrecv(buf, sizeof(buf), '\012', "");
-  ret=upsrecv(result, resultsize, '\015', "\012");
-  upsrecv(buf, sizeof(buf), '>', "");
+  ser_send(upsfd, cmd);
+  ser_get_line(upsfd, buf, sizeof(buf), '\012', "", 3, 0);
+  ret = ser_get_line(upsfd, result, resultsize, '\015', "\012", 3, 0);
+  ser_get_line(upsfd, buf, sizeof(buf), '>', "", 3, 0);
   return ret;
 
 }
@@ -206,8 +207,8 @@ void upsdrv_updateinfo(void)
         }
 	break;
       default: /* Will never happen, caught in upsdrv_initups() */
-        fprintf(stderr, "Uknown model in upsdrv_updateinfo()\n");
-        exit(1);
+        fprintf(stderr, "Unknown model in upsdrv_updateinfo()\n");
+        exit(EXIT_FAILURE);
     }
     /* Compute battery percent left based on battery voltages. */
     battpercent = ((vbatt - fc.emptyvolts) 
@@ -286,7 +287,7 @@ void upsdrv_updateinfo(void)
 }
 
 
-void ups_sync(void)
+static void ups_sync(void)
 {
   char	buf[256];
 
@@ -300,7 +301,7 @@ void ups_sync(void)
     fprintf(stderr, "UPS Time: %s\n", buf);
   } else {
     fprintf(stderr, "Error connecting to UPS.\n");
-    exit(1);
+    exit(EXIT_FAILURE);
   }
 }
 
@@ -308,18 +309,13 @@ void ups_sync(void)
 void upsdrv_shutdown(void)
 {
 /* NB: hard-wired password */
-  upssend("pw377\r");
-  upssend("off 1 a\r");	/* power off in 1 second and restart when line power returns */
+  ser_send(upsfd, "pw377\r");
+  ser_send(upsfd, "off 1 a\r");	/* power off in 1 second and restart when line power returns */
 }
 
 /* list flags and values that you want to receive via -x */
 void upsdrv_makevartable(void)
 {
-        /* allow '-x xyzzy' */
-        /* addvar(VAR_FLAG, "xyzzy", "Enable xyzzy mode"); */
-
-        /* allow '-x foo=<some value>' */
-        /* addvar(VAR_VALUE, "foo", "Override foo setting"); */
 }
 
 void upsdrv_help(void)
@@ -336,12 +332,12 @@ void upsdrv_banner(void)
 static void sync_serial(void) {
 	char buffer[10];
 
-	upssend("\r");
-  	upsrecv(buffer,sizeof(buffer), '\r', "\012");
-	upsrecv(buffer, sizeof(buffer), ENDCHAR, IGNCHARS);
+	ser_send(upsfd, "\r");
+	ser_get_line(upsfd, buffer, sizeof(buffer), '\r', "\012", 3, 0);
+	ser_get_line(upsfd, buffer, sizeof(buffer), ENDCHAR, IGNCHARS, 3, 0);
 
-	while (upsrecv(buffer,sizeof(buffer), '>', "\012") <= 0) {
-		upssend("\r");
+	while (ser_get_line(upsfd, buffer, sizeof(buffer), '>', "\012", 3, 0) <= 0) {
+		ser_send(upsfd, "\r");
 	}
 }
 
@@ -379,7 +375,8 @@ void upsdrv_initups ()
 {
   char	temp[256], fcstring[512];
 
-  open_serial(device_path, B1200);
+  upsfd = ser_open(device_path);
+  ser_set_speed(upsfd, device_path, B1200);
   setup_serial();
   ups_sync();
 
@@ -387,14 +384,14 @@ void upsdrv_initups ()
   /* Obtain Model */
   if (execute("id\r", fcstring, sizeof(fcstring)) < 0) {
     fprintf(stderr, "Failed execute in ups_ident()\n");
-    exit(1);
+    exit(EXIT_FAILURE);
   }
   
   /* response is a one-line packed string starting with $ */
   if (memcmp(fcstring, "Unit", 4)) {
     fprintf(stderr, "Bad response from formatconfig command in ups_ident()\n");
     fprintf(stderr, "id: %s\n", fcstring);
-    exit(1);
+    exit(EXIT_FAILURE);
   }
 
   if (debugging)
@@ -446,7 +443,7 @@ void upsdrv_initups ()
       break;
     default:
       fprintf(stderr, "Uknown model %s in ups_ident()\n", temp);
-      exit(1);
+      exit(EXIT_FAILURE);
       break;
   }
 
@@ -456,4 +453,5 @@ void upsdrv_initups ()
 
 void upsdrv_cleanup(void)
 {
+  ser_close(upsfd, device_path);
 }
