@@ -18,90 +18,88 @@
 */
 
 #include "main.h"
+#include "serial.h"
 
 #define ENDCHAR  13	/* replies end with CR */
-#define UPSDELAY  5
-#define MAXTRIES 10
+#define MAXTRIES 5
+#define UPSDELAY 50000	/* 50 ms delay required for reliable operation */
 
-#define DRV_VERSION "0.60"
+#define SER_WAIT_SEC	3	/* allow 3.0 sec for ser_get calls */
+#define SER_WAIT_USEC	0
+
+#define DRV_VERSION "1.02"
 
-static	char	*modelname, *va;
 static	float	lowvolt = 0, highvolt = 0, voltrange = 0;
-static	int	linenorm = 0, poll_failures = 0;
+static	int	linenorm = 0;
 static	int	inverted_bypass_bit = 0;
 
-static void setmodel(const char *abbr, const char *va)
+static void model_set(const char *abbr, const char *rating)
 {
 	if (!strcmp(abbr, "FOR")) {
 		dstate_setinfo("ups.mfr", "%s", "Best Power");
-		dstate_setinfo("ups.model", "Fortress %s", va);
+		dstate_setinfo("ups.model", "Fortress %s", rating);
 		return;
 	}
 
 	if (!strcmp(abbr, "FTC")) {
 		dstate_setinfo("ups.mfr", "%s", "Best Power");
-		dstate_setinfo("ups.model", "Fortress Telecom %s", va);
+		dstate_setinfo("ups.model", "Fortress Telecom %s", rating);
 		return;
 	}
 
 	if (!strcmp(abbr, "PRO")) {
 		dstate_setinfo("ups.mfr", "%s", "Best Power");
-		dstate_setinfo("ups.model", "Patriot Pro %s", va);
+		dstate_setinfo("ups.model", "Patriot Pro %s", rating);
 		inverted_bypass_bit = 1;
 		return;
 	}
 
 	if (!strcmp(abbr, "PR2")) {
 		dstate_setinfo("ups.mfr", "%s", "Best Power");
-		dstate_setinfo("ups.model", "Patriot Pro II %s", va);
+		dstate_setinfo("ups.model", "Patriot Pro II %s", rating);
 		inverted_bypass_bit = 1;
 		return;
 	}
 
 	if (!strcmp(abbr, "325")) {
 		dstate_setinfo("ups.mfr", "%s", "Sola Australia");
-		dstate_setinfo("ups.model", "Sola 325 %s", va);
+		dstate_setinfo("ups.model", "Sola 325 %s", rating);
 		return;
 	}
 
 	if (!strcmp(abbr, "520")) {
 		dstate_setinfo("ups.mfr", "%s", "Sola Australia");
-		dstate_setinfo("ups.model", "Sola 520 %s", va);
+		dstate_setinfo("ups.model", "Sola 520 %s", rating);
 		return;
 	}
 
 	if (!strcmp(abbr, "620")) {
 		dstate_setinfo("ups.mfr", "%s", "Sola Australia");
-		dstate_setinfo("ups.model", "Sola 620 %s", va);
+		dstate_setinfo("ups.model", "Sola 620 %s", rating);
 		return;
 	}
 
 	if (!strcmp(abbr, "AX1")) {
 		dstate_setinfo("ups.mfr", "%s", "Best Power");
-		dstate_setinfo("ups.model", "Axxium Rackmount %s", va);
+		dstate_setinfo("ups.model", "Axxium Rackmount %s", rating);
 		return;
 	}
 
 	dstate_setinfo("ups.mfr", "%s", "Unknown");
-	dstate_setinfo("ups.model", "Unknown %s (%s)", abbr, va);
+	dstate_setinfo("ups.model", "Unknown %s (%s)", abbr, rating);
 
 	printf("Unknown model detected - please report this ID: '%s'\n", abbr);
 }
 
-int instcmd(const char *cmdname, const char *extra)
+static int instcmd(const char *cmdname, const char *extra)
 {
 	if (!strcasecmp(cmdname, "test.battery.stop")) {
-		upssendchar('C');
-		upssendchar('T');
-		upssendchar(13);
-
+		ser_send_pace(upsfd, UPSDELAY, "CT\r");
 		return STAT_INSTCMD_HANDLED;
 	}
 
 	if (!strcasecmp(cmdname, "test.battery.start")) {
-		upssendchar('T');
-		upssendchar(13);
-
+		ser_send_pace(upsfd, UPSDELAY, "T\r");
 		return STAT_INSTCMD_HANDLED;
 	}
 
@@ -109,153 +107,26 @@ int instcmd(const char *cmdname, const char *extra)
 	return STAT_INSTCMD_UNKNOWN;
 }
 
-void upsdrv_initinfo(void)
-{
-	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
-	dstate_addcmd("test.battery.start");
-	dstate_addcmd("test.battery.stop");
-
-	setmodel(modelname, va);
-	printf("Detected %s %s on %s\n", dstate_getinfo("ups.mfr"), 
-		dstate_getinfo("ups.model"), device_path);
-
-	upsh.new_instcmd = instcmd;
-
-	/* paranoia - cancel any shutdown that might already be running */
-	upssendchar('C');
-	upssendchar(13);
-}
-
-static void setup_serial(void)
-{	
-	struct	termios	tio;
-
-	if (tcgetattr(upsfd, &tio) == -1)
-		fatal("tcgetattr");
-
-	tio.c_iflag = IXON | IXOFF;
-	tio.c_oflag = 0;
-	tio.c_cflag = (CS8 | CREAD | HUPCL | CLOCAL);
-	tio.c_lflag = 0;
-	tio.c_cc[VMIN] = 1;
-	tio.c_cc[VTIME] = 0; 
-
-#ifdef HAVE_CFSETISPEED
-	cfsetispeed(&tio, B2400);
-	cfsetospeed(&tio, B2400);
-#else
-#error This system lacks cfsetispeed() and has no other means to set the speed
-#endif
-
-	if (tcsetattr(upsfd, TCSANOW, &tio) == -1)
-		fatal("tcsetattr");
-}
-
-static void ups_sync(void)
-{
-	char	buf[256];
-	int	i, ret;
-
-	printf("Syncing with UPS: ");
-	fflush(stdout);
-
-	for (i = 0; i < MAXTRIES; i++) {
-		printf(".");
-		fflush(stdout);
-		upssend("%s", "\rQ1\r");
-		printf(".");
-		fflush(stdout);
-		sleep(UPSDELAY);
-		printf(".");
-		fflush(stdout);
-
-		ret = upsrecv(buf, sizeof(buf), ENDCHAR, "");
-
-		/* return once we get something that looks usable */
-		if ((ret > 0) && (buf[0] == '(')) {
-			printf(" done\n");
-			return;
-		}
-	}
-
-	fatalx("\nFailed - giving up...");
-}
-
-static int ups_on_line(void)
-{
-	int	i, ret;
-	char	temp[256], stat[32];
-
-	printf("Checking line status: ");
-	fflush(stdout);
-
-	for (i = 0; i < MAXTRIES; i++) {
-		upssend("\rQ1\r");
-		printf(".");
-		fflush(stdout);
-		sleep(UPSDELAY);
-		printf(".");
-		fflush(stdout);
-
-		ret = upsrecv(temp, sizeof(temp), ENDCHAR, "");
-
-		/* Q1 must return 46 bytes starting with a ( */
-		if ((ret > 0) && (temp[0] == '(') && (strlen(temp) == 46)) {
-
-			sscanf(temp, "%*s %*s %*s %*s %*s %*s %*s %s", stat);
-
-			if (stat[0] == '0')
-				return 1;	/* on line */
-
-			return 0;	/* on battery */
-		}
-	}
-
-	upslogx(LOG_ERR, "Status read failed: assuming on battery");
-
-	return 0;	/* on battery */
-}
-
-void upsdrv_shutdown(void)
-{
-	printf("The UPS will shut down in approximately one minute.\n");
-
-	if (ups_on_line())
-		printf("The UPS will restart in about one minute.\n");
-	else
-		printf("The UPS will restart when power returns.\n");
-
-	upssend("%s", "S01R0001\r");
-}
 
 static int get_ident(char *buf, size_t bufsize)
 {
 	int	i, ret;
 
-	printf("Identifying UPS: ");
-	fflush(stdout);
-
 	for (i = 0; i < MAXTRIES; i++) {
-		printf(".");
-		fflush(stdout);
+		ser_send_pace(upsfd, UPSDELAY, "\rID\r");
 
-		upssend("%s", "\r\rID\r");
-		printf(".");
+		ret = ser_get_line(upsfd, buf, bufsize, ENDCHAR, "", 
+			SER_WAIT_SEC, SER_WAIT_USEC);
 
-		fflush(stdout);
-		sleep(UPSDELAY);
-		printf(".");
-		fflush(stdout);
-
-		ret = upsrecv(buf, bufsize, ENDCHAR, "");
+		if (ret > 0)
+			upsdebugx(2, "get_ident: got [%s]", buf);
 
 		/* buf must start with ( and be in the range [25-27] */
 		if ((ret > 0) && (buf[0] != '(') && (strlen(buf) >= 25) &&
-			(strlen(buf) <= 27)) {
-
-			printf(" done\n");
+			(strlen(buf) <= 27))
 			return 1;
-		}
+
+		sleep(1);
 	}
 
 	upslogx(LOG_INFO, "Giving up on hardware detection after %d tries",
@@ -266,13 +137,13 @@ static int get_ident(char *buf, size_t bufsize)
 
 static void ups_ident(void)
 {
-	char	buf[256], *ptr, *com;
+	char	buf[256], *ptr, *com, *model, *rating, *bvs;
 	int	i;
 
 	if (!get_ident(buf, sizeof(buf)))
-		fatalx("Unable to get initial hardware info string");
+		fatalx("Unable to detect a Best/SOLA or Phoenix protocol UPS");
 
-	modelname = va = NULL;
+	model = rating = NULL;
 
 	/* FOR,750,120,120,20.0,27.6 */
 	ptr = buf;
@@ -284,16 +155,15 @@ static void ups_ident(void)
 			*com = '\0';
 
 		switch (i) {
-			case 0: modelname = xstrdup(ptr);
+			case 0: model = ptr;
 				break;
-			case 1: va = xstrdup(ptr);
+			case 1: rating = ptr;
 				break;
 			case 2: linenorm = atoi(ptr);
 				break;
 			case 4: lowvolt = atof(ptr);
 				break;
 			case 5: highvolt = atof(ptr);
-				voltrange = highvolt - lowvolt;
 				break;
 			default:
 				break;
@@ -302,67 +172,160 @@ static void ups_ident(void)
 		if (com)
 			ptr = com + 1;
 	}
+
+	if ((!model) || (!rating))
+		fatalx("Didn't get a valid ident string");
+
+	bvs = getval("nombattvolt");
+
+	if (bvs) {
+		float	nomvolt;
+
+		nomvolt = strtod(bvs, (char **) NULL);
+
+		if (nomvolt > 0)
+			highvolt = nomvolt;
+	}
+
+	dstate_setinfo("battery.voltage.nominal", "%2.1f", highvolt);
+
+	voltrange = highvolt - lowvolt;
+
+	model_set(model, rating);
 }
 
-static void pollfail(char *why)
+static void ups_sync(void)
 {
-	poll_failures++;
+	char	buf[256];
+	int	i, ret;
 
-	/* ignore the first few since these UPSes tend to drop characters */
-	if (poll_failures == 3)
-		upslogx(LOG_ERR, why);
+	for (i = 0; i < MAXTRIES; i++) {
+		ser_send_pace(upsfd, UPSDELAY, "\rQ1\r");
 
-	return;
+		ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, "", 
+			SER_WAIT_SEC, SER_WAIT_USEC);
+
+		/* return once we get something that looks usable */
+		if ((ret > 0) && (buf[0] == '('))
+			return;
+
+		usleep(250000);
+	}
+
+	fatalx("Unable to detect a Best/SOLA or Phoenix protocol UPS");
+}
+
+void upsdrv_initinfo(void)
+{
+	ups_sync();
+	ups_ident();
+
+	printf("Detected %s %s on %s\n", dstate_getinfo("ups.mfr"), 
+		dstate_getinfo("ups.model"), device_path);
+
+	/* paranoia - cancel any shutdown that might already be running */
+	ser_send_pace(upsfd, UPSDELAY, "C\r");
+
+	upsh.instcmd = instcmd;
+
+	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
+	dstate_addcmd("test.battery.start");
+	dstate_addcmd("test.battery.stop");
+}
+
+static int ups_on_line(void)
+{
+	int	i, ret;
+	char	temp[256], pstat[32];
+
+	for (i = 0; i < MAXTRIES; i++) {
+		ser_send_pace(upsfd, UPSDELAY, "\rQ1\r");
+
+		ret = ser_get_line(upsfd, temp, sizeof(temp), ENDCHAR, "", 
+			SER_WAIT_SEC, SER_WAIT_USEC);
+
+		/* Q1 must return 46 bytes starting with a ( */
+		if ((ret > 0) && (temp[0] == '(') && (strlen(temp) == 46)) {
+
+			sscanf(temp, "%*s %*s %*s %*s %*s %*s %*s %s", pstat);
+
+			if (pstat[0] == '0')
+				return 1;	/* on line */
+
+			return 0;	/* on battery */
+		}
+
+		sleep(1);
+	}
+
+	upslogx(LOG_ERR, "Status read failed: assuming on battery");
+
+	return 0;	/* on battery */
+}	
+
+void upsdrv_shutdown(void)
+{
+	printf("The UPS will shut down in approximately one minute.\n");
+
+	if (ups_on_line())
+		printf("The UPS will restart in about one minute.\n");
+	else
+		printf("The UPS will restart when power returns.\n");
+
+	ser_send_pace(upsfd, UPSDELAY, "S01R0001\r");
 }
 
 void upsdrv_updateinfo(void)
 {
 	char	involt[16], outvolt[16], loadpct[16], acfreq[16], 
-		battvolt[16], upstemp[16], stat[16], buf[256];
+		battvolt[16], upstemp[16], pstat[16], buf[256];
 	float	bvoltp;
 	int	ret;
-	static	int	pass = 0;
 
-	/* this UPS needs a huge delay on queries, so chop up the updates */
+	ret = ser_send_pace(upsfd, UPSDELAY, "\rQ1\r");
 
-	/* half the time, send the Q1, the other half, read it */
-
-	if (pass == 0) {
-		upssend("%s", "\rQ1\r");
-		pass = 1;
-		return;
-	}
-
-	pass = 0;
-
-	ret = upsrecv(buf, sizeof(buf), ENDCHAR, "");
-
-	if (strlen(buf) < 46) {
-		pollfail("Poll failed: short read from UPS");
+	if (ret < 1) {
+		ser_comm_fail("ser_send_pace failed");
 		dstate_datastale();
 		return;
 	}
 
-	if (strlen(buf) > 46) {
-		pollfail("Poll failed: oversized read from UPS");
+	/* these things need a long time to respond completely */
+	usleep(200000);
+
+	ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, "", 
+		SER_WAIT_SEC, SER_WAIT_USEC);
+
+	if (ret < 1) {
+		ser_comm_fail(NULL);
+		dstate_datastale();
+		return;
+	}
+
+	if (ret < 46) {
+		ser_comm_fail("Poll failed: short read (got %d bytes)", ret);
+		dstate_datastale();
+		return;
+	}
+
+	if (ret > 46) {
+		ser_comm_fail("Poll failed: response too long (got %d bytes)",
+			ret);
 		dstate_datastale();
 		return;
 	}
 
 	if (buf[0] != '(') {
-		pollfail("Poll failed: invalid start character");
+		ser_comm_fail("Poll failed: invalid start character (got %02x)",
+			buf[0]);
 		dstate_datastale();
 		return;
 	}
 
-	/* only say this if it got high enough to log a failure note */
-	if (poll_failures >= 3)
-		upslogx(LOG_NOTICE, "UPS poll succeeded");
-
-	poll_failures = 0;
+	ser_comm_good();
 
 	sscanf(buf, "%*c%s %*s %s %s %s %s %s %s", involt, outvolt, 
-		loadpct, acfreq, battvolt, upstemp, stat);
+		loadpct, acfreq, battvolt, upstemp, pstat);
 	
 	bvoltp = ((atof (battvolt) - lowvolt) / voltrange) * 100.0;
 
@@ -379,12 +342,12 @@ void upsdrv_updateinfo(void)
 
 	status_init();
 
-	if (stat[0] == '0') {
+	if (pstat[0] == '0') {
 		status_set("OL");		/* on line */
 
 		/* only allow these when OL since they're bogus when OB */
 
-		if (stat[2] == (inverted_bypass_bit ? '0' : '1')) {
+		if (pstat[2] == (inverted_bypass_bit ? '0' : '1')) {
 			/* boost or trim in effect */
 			if (atof(involt) < atof(outvolt))
 				status_set("BOOST");
@@ -397,7 +360,7 @@ void upsdrv_updateinfo(void)
 		status_set("OB");		/* on battery */
 	}
 
-	if (stat[1] == '1')
+	if (pstat[1] == '1')
 		status_set("LB");		/* low battery */
 
 	status_commit();
@@ -410,6 +373,7 @@ void upsdrv_help(void)
 
 void upsdrv_makevartable(void)
 {
+	addvar(VAR_VALUE, "nombattvolt", "Override nominal battery voltage");
 }
 
 void upsdrv_banner(void)
@@ -420,18 +384,11 @@ void upsdrv_banner(void)
 
 void upsdrv_initups(void)
 {
-	upssend_delay = 100000;
-	open_serial(device_path, B2400);
-	setup_serial();
-
-	/* don't let upscommon warn about it since it happens way too often */
-	/* this driver does its own checking with better handling */
-        flag_timeoutfailure = -1;
-
-	ups_sync();
-	ups_ident();
+	upsfd = ser_open(device_path);
+	ser_set_speed(upsfd, device_path, B2400);
 }
 
 void upsdrv_cleanup(void)
 {
+	ser_close(upsfd, device_path);
 }
