@@ -20,60 +20,58 @@
 */
 
 #include "main.h"
+#include "serial.h"
 
-#define ENDCHAR  13	/* replies end with CR */
-#define UPSDELAY  5
-#define MAXTRIES 10
+#define DRV_VERSION	"0.07"
+
+#define ENDCHAR		13	/* replies end with CR */
+#define UPSDELAY 	5
+#define MAXTRIES	10
+#define SENDDELAY	100000	/* 100 ms delay between chars on transmit */
+#define SER_WAIT_SEC	3	/* allow 3.0 sec for responses */
+#define SER_WAIT_USEC	0
 
 static	float	lowvolt = 0, highvolt = 0, voltrange = 0;
 static	int	poll_failures = 0;
 static	int	inverted_bypass_bit = 0;
 
 
-int instcmd(const char *cmdname, const char *extra)
+static int instcmd(const char *cmdname, const char *extra)
 {
 	/* Stop battery test */
 	if (!strcasecmp(cmdname, "test.battery.stop")) {
-			upssendchar('C');
-			upssendchar('T');
-			upssendchar(13);
-			return STAT_INSTCMD_HANDLED;
+		ser_send_pace(upsfd, SENDDELAY, "CT\r");
+		return STAT_INSTCMD_HANDLED;
 	}
 
 	/* Start battery test */
 	if (!strcasecmp(cmdname, "test.battery.start")) {
-			upssendchar('T');
-			upssendchar(13);
-			return STAT_INSTCMD_HANDLED;
+		ser_send_pace(upsfd, SENDDELAY, "T\r");
+		return STAT_INSTCMD_HANDLED;
 	}
 
-  upslogx(LOG_NOTICE, "instcmd: unknown command [%s]", cmdname);
+	upslogx(LOG_NOTICE, "instcmd: unknown command [%s]", cmdname);
 	return STAT_INSTCMD_UNKNOWN;
 }
 
 void upsdrv_initinfo(void)
 {
+	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
 	dstate_setinfo("ups.mfr", "Centralion");
 	dstate_setinfo("ups.model", "Blazer");
-	dstate_setinfo("input.voltage", "");
-	dstate_setinfo("input.frequency", "");
-	dstate_setinfo("output.voltage", "");
-	dstate_setinfo("battery.charge", "");
-	dstate_setinfo("battery.voltage", "");
-	dstate_setinfo("ups.load", "");
   
 	dstate_addcmd("test.battery.start");
 	dstate_addcmd("test.battery.stop");
 
 	printf("Detected UPS on %s\n", device_path);
 
-	upsh.new_instcmd = instcmd;
+	upsh.instcmd = instcmd;
 
 	/* paranoia - cancel any shutdown that might already be running */
-	upssendchar('C');
-	upssendchar(13);
+	ser_send_pace(upsfd, SENDDELAY, "C\r");
 }
 
+/* is this really necessary to make this hardware work? */
 static void setup_serial(void)
 {
 	struct	termios	tio;
@@ -99,7 +97,7 @@ static void setup_serial(void)
 		fatal("tcsetattr");
 }
 
-void ups_sync(void)
+static void ups_sync(void)
 {
 	char	buf[256];
 	int	tries = 0, ret;
@@ -114,16 +112,20 @@ void ups_sync(void)
 
 		printf(".");
 		fflush(stdout);
-		upssend("%s", "\r");
-		upssend("%s", "Q1");
-		upssend("%s", "\r");
+
+		ret = ser_send_pace(upsfd, SENDDELAY, "\rQ1\r");
+
+		if (ret < 1)
+			continue;
+		
 		printf(".");
 		fflush(stdout);
 		sleep(UPSDELAY);
 		printf(".");
 		fflush(stdout);
 
-		ret = upsrecv(buf, sizeof(buf), ENDCHAR, "");
+		ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, "",
+			SER_WAIT_SEC, SER_WAIT_USEC);
 
 		if ((ret > 0) && (buf[0] == '('))
 			break;
@@ -134,11 +136,10 @@ void ups_sync(void)
 
 void upsdrv_shutdown(void)
 {
-	upssend("%s", "S01R0001");
-	upssend("%s", "\r");	/* end sequence */
+	ser_send_pace(upsfd, SENDDELAY, "S01R0001\r");
 }
 
-void ups_ident(void)
+static void ups_ident(void)
 {
 	char	buf[256];
 	int	tries = 0, ret;
@@ -157,16 +158,19 @@ void ups_ident(void)
 
 		printf(".");
 		fflush(stdout);
-		upssend("%s", "\r");
-		upssend("%s", "F");
-		upssend("%s", "\r");
+		ret = ser_send_pace(upsfd, SENDDELAY, "\rF\r");
+
+		if (ret < 1)
+			continue;
+
 		printf(".");
 		fflush(stdout);
 		sleep(UPSDELAY);
 		printf(".");
 		fflush(stdout);
 
-		ret = upsrecv(buf, sizeof(buf), ENDCHAR, "");
+		ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, "",
+			SER_WAIT_SEC, SER_WAIT_USEC);
 
 		if ((ret > 0) && (buf[0] == '#') && (strlen(buf) == 21))
 			break;
@@ -177,7 +181,7 @@ void ups_ident(void)
 	sscanf(buf, "%*c %f %d %f %f", &ratevolt, &ratecurrent, 
 		&lowvolt, &ratefreq);
 	upsdebugx(2, "UPS is rated at %.2fV, %dA, %.2fHz.\n",
-    ratevolt, ratecurrent, ratefreq);
+		ratevolt, ratecurrent, ratefreq);
 
 	/* Not sure exactly how the battery voltage from the "F" command
 	 * should be handled.  The lowest voltage I've seen before the
@@ -189,7 +193,7 @@ void ups_ident(void)
 	voltrange = highvolt - lowvolt;
 }
 
-void pollfail(char *why)
+static void pollfail(const char *why)
 {
 	poll_failures++;
 
@@ -203,25 +207,38 @@ void pollfail(char *why)
 void upsdrv_updateinfo(void)
 {
 	char	utility[16], outvolt[16], loadpct[16], acfreq[16], 
-		battvolt[16], upstemp[16], stat[16], buf[256];
+		battvolt[16], upstemp[16], pstat[16], buf[256];
 	float	bvoltp;
 	int	ret;
 
-	upssend("%s", "\rQ1");
-	upssend("%s", "\r");
+	ret = ser_send_pace(upsfd, SENDDELAY, "\rQ1\r");
+
+	if (ret < 1) {
+		pollfail("Poll failed: send status request failed");
+		dstate_datastale();
+		return;
+	}
+
 	sleep(UPSDELAY); 
 
-	ret = upsrecv(buf, sizeof(buf), ENDCHAR, "");
+	ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, "",
+		SER_WAIT_SEC, SER_WAIT_USEC);
+
+	if (ret < 1) {
+		pollfail("Poll failed: read failed");
+		dstate_datastale();
+		return;
+	}
 
 	if (strlen(buf) < 46) {
 		pollfail("Poll failed: short read from UPS");
-    dstate_datastale();
+		dstate_datastale();
 		return;
 	}
 
 	if (strlen(buf) > 46) {
 		pollfail("Poll failed: oversized read from UPS");
-    dstate_datastale();
+		dstate_datastale();
 		return;
 	}
 
@@ -237,14 +254,14 @@ void upsdrv_updateinfo(void)
 	poll_failures = 0;
 
 	sscanf(buf, "%*c%s %*s %s %s %s %s %s %s", utility, outvolt, 
-		loadpct, acfreq, battvolt, upstemp, stat);
+		loadpct, acfreq, battvolt, upstemp, pstat);
 	
 	bvoltp = ((atof (battvolt) - lowvolt) / voltrange) * 100.0;
 
 	if (bvoltp > 100.0)
 		bvoltp = 100.0;
 
-  dstate_setinfo("input.voltage", "%s", utility);
+	dstate_setinfo("input.voltage", "%s", utility);
 	dstate_setinfo("input.frequency", "%s", acfreq);
 	dstate_setinfo("output.voltage", "%s", outvolt);
 	dstate_setinfo("battery.charge", "%02.1f", bvoltp);
@@ -253,12 +270,12 @@ void upsdrv_updateinfo(void)
 
 	status_init();
 
-	if (stat[0] == '0') {
+	if (pstat[0] == '0') {
 		status_set("OL");		/* on line */
 
 		/* only allow these when OL since they're bogus when OB */
 
-		if (stat[2] == (inverted_bypass_bit ? '0' : '1')) {
+		if (pstat[2] == (inverted_bypass_bit ? '0' : '1')) {
 			/* boost or trim in effect */
 			if (atof(utility) < atof(outvolt))
 				status_set("BOOST");
@@ -271,11 +288,11 @@ void upsdrv_updateinfo(void)
 		status_set("OB");		/* on battery */
 	}
 
-	if (stat[1] == '1')
+	if (pstat[1] == '1')
 		status_set("LB");		/* low battery */
 
 	status_commit();
-  dstate_dataok();
+	dstate_dataok();
 }
 
 void upsdrv_help(void)
@@ -288,18 +305,16 @@ void upsdrv_makevartable(void)
 
 void upsdrv_banner(void)
 {
-	printf("Network UPS Tools - Blazer UPS driver 0.06 (%s)\n", UPS_VERSION);
+	printf("Network UPS Tools - Blazer UPS driver %s (%s)\n", 
+		DRV_VERSION, UPS_VERSION);
 }
 
 void upsdrv_initups(void)
 {
-	upssend_delay = 100000;
-	open_serial(device_path, B2400);
-	setup_serial();
+	upsfd = ser_open(device_path);
+	ser_set_speed(upsfd, device_path, B2400);
 
-	/* don't let upscommon warn about it since it happens way too often */
-	/* this driver does its own checking with better handling */
-	flag_timeoutfailure = -1;
+	setup_serial();
 
 	ups_sync();
 	ups_ident();
@@ -307,5 +322,5 @@ void upsdrv_initups(void)
 
 void upsdrv_cleanup(void)
 {
+	ser_close(upsfd, device_path);
 }
-
