@@ -19,32 +19,28 @@
 */
 
 #include "main.h"
+#include "serial.h"
 
 #include <sys/ioctl.h>
-#include <sys/termios.h>
 
 #include "timehead.h"
 
-#define DRV_VERSION "0.22"
+#define DRV_VERSION "1.00"
 
 /* window for repeating dangerous command (shutdown.stayoff) */
-#define MINCMDTIME      3
-#define MAXCMDTIME      15
+#define MINCMDTIME	3
+#define MAXCMDTIME	15
 
-/* limit the amount of spew that goes in the syslog when we lose the UPS */
-#define CP_ERR_LIMIT 10		/* start limiting after 10 in a row	*/
-#define CP_ERR_RATE 100		/* then only print every 100th error	*/
-
-	struct	sigaction sa;
-	sigset_t cp_sigmask;
-	unsigned int	poll_failures = 0;
+#define UPSDELAY 50000
 
 /* ups frequency */
-float frequency(unsigned char in)
+static float frequency(unsigned char in)
 {
-	float freq[22] = { 63.0, 62.7, 62.4, 62.1, 61.8, 61.4, 61.1, 60.8, 60.5, 60.2, 60.0, 59.7, 59.4, 59.1, 58.8, 58.5, 58.3, 58.0, 57.7, 57.4, 57.2, 57.0 };
+	float freq[22] = { 63.0, 62.7, 62.4, 62.1, 61.8, 61.4, 61.1, 60.8, 
+		60.5, 60.2, 60.0, 59.7, 59.4, 59.1, 58.8, 58.5, 58.3, 58.0,
+		57.7, 57.4, 57.2, 57.0 };
 	int i, j;
-        
+
 	for (i = 0, j = 168; i < 23; j++, i++)
 		if (in == j)
 			return (float)freq[i];
@@ -53,10 +49,16 @@ float frequency(unsigned char in)
 }
 
 /* adjust bizarre UPS data to observed voltage data */
-int voltconvert(unsigned char in)
+static int voltconvert(unsigned char in)
 {
-	int v_end[43] = { 36, 51, 55, 60, 65, 70, 75, 80, 85, 91, 98, 103, 108, 113, 118, 123, 128, 133, 138, 143, 148, 153, 158, 163, 168, 173, 178, 183, 188, 193, 198, 203, 208, 213, 218, 223, 228, 233, 238, 243, 248, 253, 255 };
-	int v_adj[43] = {  3,  4,  5,  4,  3,  2,  1,  0, -1, -2, -3,  -4,  -5,  -6,  -7,  -8,  -9, -10, -11, -12, -13, -14, -15, -16, -17, -18, -19, -20, -21, -22, -23, -24, -25, -26, -27, -28, -29, -30, -31, -32, -33, -34, -35 };
+	int v_end[43] = { 36, 51, 55, 60, 65, 70, 75, 80, 85, 91, 98, 103, 
+		108, 113, 118, 123, 128, 133, 138, 143, 148, 153, 158, 163, 
+		168, 173, 178, 183, 188, 193, 198, 203, 208, 213, 218, 223, 
+		228, 233, 238, 243, 248, 253, 255 };
+	int v_adj[43] = {  3,  4,  5,  4,  3,  2,  1,  0, -1, -2, -3,  -4,  
+		-5,  -6,  -7,  -8,  -9, -10, -11, -12, -13, -14, -15, -16, 
+		-17, -18, -19, -20, -21, -22, -23, -24, -25, -26, -27, -28, 
+		-29, -30, -31, -32, -33, -34, -35 };
 	int	i;
 
 	if (in < 27)
@@ -72,9 +74,11 @@ int voltconvert(unsigned char in)
 }
 
 /* map UPS data to realistic percentages */
-int battconvert(unsigned char in)
+static int battconvert(unsigned char in)
 {
-	int b_val[26] = {0, 1, 1, 2, 3, 4, 6, 8, 10, 12, 15, 18, 22, 26, 30, 35, 40, 46, 52, 58, 66, 73, 81, 88, 99, 100 }; /* XXX - for load of 0 */
+	/* these may only be valid for a load of 0 */
+	int b_val[26] = {0, 1, 1, 2, 3, 4, 6, 8, 10, 12, 15, 18, 22, 26, 30, 
+		35, 40, 46, 52, 58, 66, 73, 81, 88, 99, 100 };
 
 	if (in > 185)
 		return 100;
@@ -87,12 +91,12 @@ int battconvert(unsigned char in)
 
 /* Model mapping */
 struct {
-	int     first;
-	int     second;
-	char	*pcode;
-	char    *model;
-}     modelmap[] = {
-      
+	int	first;
+	int	second;
+	const	char	*pcode;		/* product code - presently ignored */
+	const	char	*model;
+}	modelmap[] = {
+
 	{ 51, 51, "OP850", "850AVR"	},	/* O33 */
 	{ 52, 53, "OP1500", "1500AVR"	},	/* O45 */
 	{ 52, 51, "OP1250", "1250AVR"	},	/* O43 */
@@ -156,7 +160,7 @@ struct {
 	{   0,   0, 0,  0 },
 };
 
-float tempconvert(unsigned char in)
+static float tempconvert(unsigned char in)
 {
 	int	i, j, found, count;
 
@@ -185,42 +189,30 @@ float tempconvert(unsigned char in)
 	return 0;
 }
 
-void sendtoups(char ch)
+static int confirm_write(const unsigned char *buf, size_t buflen)
 {
-	write(upsfd, &ch, 1);
-	usleep(50000);
-}
+	int	ret;
+	char	verify[16];
+	unsigned int	i;
 
-static void read_timeout(int sig)
-{
-	/* ignore this */
-	return;
-}
+	ret = ser_send_buf_pace(upsfd, UPSDELAY, buf, buflen);
 
-static int confirm_write(const unsigned char *buf, int buflen)
-{
-	int	i, ret;
-	unsigned	char	ch;
-
-	for (i = 0; i < buflen; i++) {
-		ret = write(upsfd, &buf[i], 1);
-
-		if (ret < 1)
-			return 0;
-
-		usleep(50000);
+	if (ret < 1) {
+		upsdebugx(1, "confirm_write: ser_send_buf_pace failed");
+		return 0;
 	}
 
-	ch = 13;
-	write(upsfd, &ch, 1);
+	/* don't try to read back the \r */
+	ret = ser_get_buf_len(upsfd, verify, buflen - 1, 5, 0);
 
-	for (i = 0; i < buflen; i++) {
+	if ((ret < 1) || (ret < ((int) buflen - 1))) {
+		upsdebugx(1, "confirm_write: ret=%d, needed %d",
+			ret, buflen - 1);
+		return 0;
+	}
 
-		alarm(2);
-		ret = read(upsfd, &ch, 1);
-		alarm(0);
-
-		if (ch != buf[i]) {
+	for (i = 0; i < buflen - 1; i++) {
+		if (buf[i] != verify[i]) {
 			upsdebugx(1, "mismatch at position %d",	i);
 			return 0;
 		}
@@ -234,27 +226,19 @@ static int confirm_write(const unsigned char *buf, int buflen)
 /* provide a quick status check to select the right shutdown command */
 static int ups_on_line(void)
 {
-	int	ret, count;
-	char	buf[SMALLBUF], ch;
+	int	ret;
+	char	buf[SMALLBUF];
 
-	sendtoups('D');
-	sendtoups(13);
+	ser_send_pace(upsfd, UPSDELAY, "D\r");
 
-	count = 0;
+	/* give it a chance to reply completely */
+	usleep(100000);
 
-	memset(buf, '\0', sizeof(buf));
-	while (count < 14) {
+	ret = ser_get_buf_len(upsfd, buf, 14, 3, 0);
 
-		alarm(1);
-		ret = read(upsfd, &ch, 1);
-		alarm(0);
-
-		if (ret < 1) {
-			upslogx(LOG_ERR, "Status read failed: assuming on battery");
-			return 0;
-		}
-
-		buf[count++] = ch;
+	if (ret < 14) {
+		upslogx(LOG_ERR, "Status read failed: assuming on battery");
+		return 0;
 	}
 
 	if (buf[9] & 128)
@@ -263,36 +247,20 @@ static int ups_on_line(void)
 	return 1;	/* on line */
 }
 
-static void setup_sigalrm(void)
-{
-	sigemptyset(&cp_sigmask);
-	sa.sa_mask = cp_sigmask;
-	sa.sa_flags = 0;
-	sa.sa_handler = read_timeout;
-	sigaction(SIGALRM, &sa, NULL);
-}
-
 /* power down the attached load immediately */
 void upsdrv_shutdown(void)
 {
 	int	i, ret, sdlen;
-	char	ch, buf[256];
+	char	buf[256];
 	unsigned char	sdbuf[16];
 
-	setup_sigalrm();
-
 	/* get this thing's attention */
 	for (i = 0; i < 10; i++) {
 		printf("Trying to wake up the ups... ");
 		fflush(stdout);
 
-		alarm(5);
-		
-		ch = 13;
-		write(upsfd, &ch, 1);
-
-		ret = read(upsfd, buf, sizeof(buf));
-		alarm(0);
+		ser_send_char(upsfd, 13);
+		ret = ser_get_buf_len(upsfd, buf, 1, 5, 0);
 
 		if (ret > 0) {
 			printf("OK\n");
@@ -304,13 +272,15 @@ void upsdrv_shutdown(void)
 
 	usleep(250000);
 
+	memset(sdbuf, '\0', sizeof(sdbuf));
+
 	if (ups_on_line() == 1) {
 
 		/* this does not come back when on battery! */
 
 		printf("Online: sending 7 byte command (back in about 45 sec)\n");
 
-		sdlen = 7;
+		sdlen = 8;
 		sdbuf[0] = 'S';
 
 		sdbuf[1] = 0x00;		/* how long until shutdown */
@@ -322,6 +292,7 @@ void upsdrv_shutdown(void)
 						/* 1 = about 45 seconds */
 
 		sdbuf[6] = 'W';
+		sdbuf[7] = '\r';
 	
 	} else {
 
@@ -331,83 +302,60 @@ void upsdrv_shutdown(void)
 
 		printf("On battery: sending 4 byte command (back when line power returns)\n");
 
-		sdlen = 4;
+		sdlen = 5;
 		sdbuf[0] = 'S';
 		sdbuf[1] = 0x00;
 		sdbuf[2] = 0x00;
 		sdbuf[3] = 'W';
+		sdbuf[4] = '\r';
 	}
 
-	printf("Sending command...");
-	fflush(stdout);
-
 	for (i = 0; i < 10; i++) {
+		printf("Sending command...");
+		fflush(stdout);
 
 		if (confirm_write(sdbuf, sdlen)) {
 			printf(" confirmed\n");
 			break;
 		}
 
-		printf("failed, retrying...");
+		printf("failed, retrying...\n");
 		fflush(stdout);
 	}
 }
 
-static void pollfail(const char *why)
-{
-	poll_failures++;
-
-	if ((poll_failures == CP_ERR_LIMIT) || 
-		((poll_failures % CP_ERR_RATE) == 0)) {
-		upslogx(LOG_WARNING, "Warning: excessive poll failures, "
-			"limiting error reporting");
-	}
-
-	if ((poll_failures < CP_ERR_LIMIT) ||
-		((poll_failures % CP_ERR_RATE) == 0)) {
-		upslogx(LOG_ERR, "UPS status unavailable: %s", why);
-	}
-}
-
 void upsdrv_updateinfo(void)
 {
-	int     ret, count;
-	char	ch, buf[SMALLBUF];
+	int	ret;
+	char	buf[SMALLBUF];
 
-	sendtoups('D');
-	sendtoups(13);
+	ser_send_pace(upsfd, UPSDELAY, "D\r");
 
-	count = 0;
+	/* give it a chance to reply completely */
+	usleep(100000);
 
-	memset(buf, '\0', sizeof(buf));
-	while (count < 14) {
+	ret = ser_get_buf_len(upsfd, buf, 14, 3, 0);
 
-		alarm(1);
-		ret = read(upsfd, &ch, 1);
-		alarm(0);
-
-		if (ret < 1) {
-			pollfail("Short read from UPS");
-			dstate_datastale();
-			return;
-		}
-
-		buf[count++] = ch;
+	if (ret < 14) {
+		ser_comm_fail("Short read from UPS");
+		dstate_datastale();
+		return;
 	}
 
 	if (buf[0] != '#') {
-		upslogx(LOG_ERR, "Invalid start char 0x%02x", buf[0] & 0xff);
+		ser_comm_fail("Invalid start char 0x%02x", buf[0] & 0xff);
 		dstate_datastale();
 		return;
 	}
 
 	if ((buf[4] != 46) || (buf[8] != 46)) {
-		upslogx(LOG_ERR, "Invalid separator in response");
+		ser_comm_fail("Invalid separator in response (0x%02x, 0x%02x)", 
+			buf[4], buf[8]);
 		dstate_datastale();
 		return;
 	}
 
-	poll_failures = 0;
+	ser_comm_good();
 
 	dstate_setinfo("input.frequency",  "%2.1f", frequency(buf[7]));
 	dstate_setinfo("ups.temperature", "%2.1f", tempconvert(buf[6]));
@@ -434,36 +382,24 @@ void upsdrv_updateinfo(void)
 
 static int get_ident(char *buf, size_t bufsize)
 {
-	int	ret, tries, count;
-	char	ch;
+	int	ret, tries;
 
 	for (tries = 0; tries < 3; tries++) {
-		sendtoups('F');
-		sendtoups(13);
+		ret = ser_send_pace(upsfd, UPSDELAY, "F\r");
 
-		memset(buf, '\0', bufsize);
+		if (ret != 2)
+			continue;
 
-		alarm(1);
-		ret = read(upsfd, &ch, 1);
-		alarm(0);
+		/* give it a chance to reply completely */
+		usleep(400000);
 
-		count = 0;
-		while (ret == 1) {
-			buf[count++] = ch;
-	
-			if ((buf[0] == '.') && (ch == 13))
-				return 1;
+		ret = ser_get_line(upsfd, buf, bufsize, '\r', "", 3, 0);
 
-			/* reading way too much */
-			if (count == bufsize) {
-				memset(buf, '\0', bufsize);
-				count = 0;
-			}				
+		if (ret < 1)
+			continue;
 
-			alarm(1);
-			ret = read(upsfd, &ch, 1);
-			alarm(0);
-		}
+		if (buf[0] == '.')
+			return 1;
 
 		/* if we got here, then the read failed somehow */
 	}
@@ -482,20 +418,21 @@ static int detect_hardware(void)
 		fatalx("Unable to get initial hardware info string");
 
 	if (buf[0] != '.') {
-		upslogx(LOG_ERR, "Invalid start model string 0x%02x", buf[0] & 0xff);
+		upslogx(LOG_ERR, "Invalid start model string 0x%02x", 
+			buf[0] & 0xff);
 		return -1;
 	}
 
 	for (i = 0; modelmap[i].model != NULL; i++) {
 		if (buf[3] == modelmap[i].first &&
 			buf[4] == modelmap[i].second) {
-              
+
 			dstate_setinfo("ups.model", "%s", modelmap[i].model);
 			foundmodel = 1;
 			break;
 		}
 	}
-           
+
 	if (!foundmodel)
 		dstate_setinfo("ups.model", "Unknown model - %c%c", 
 			buf[3], buf[4]);
@@ -509,7 +446,7 @@ static int detect_hardware(void)
 static int instcmd_btest(void)
 {
 	int	i, clen;
-	char	cbuf[8];
+	unsigned char	cbuf[8];
 
 	clen = 2;
 	cbuf[0] = 'T';
@@ -527,7 +464,7 @@ static int instcmd_btest(void)
 static int instcmd_stayoff(void)
 {
 	int	i, clen;
-	char	cbuf[8];
+	unsigned char	cbuf[8];
 	double	elapsed;
 	time_t	now;
 	static	time_t	last = 0;
@@ -538,19 +475,20 @@ static int instcmd_stayoff(void)
 	last = now;
 
 	/* for safety this must be repeated in a small window of time */
-        if ((elapsed < MINCMDTIME) || (elapsed > MAXCMDTIME)) {
+	if ((elapsed < MINCMDTIME) || (elapsed > MAXCMDTIME)) {
 		upsdebugx(1, "instcmd_shutdown: outside window (%2.0f)",
 			elapsed);
-                return STAT_INSTCMD_HANDLED;            /* FUTURE: again */
-        }
+		return STAT_INSTCMD_HANDLED;		/* FUTURE: again */
+	}
 
 	/* this is a one-way trip unless you happen to be on battery */
 
-	clen = 4;		
+	clen = 5;
 	cbuf[0] = 'S';
 	cbuf[1] = 0x00;
 	cbuf[2] = 0x00;		
 	cbuf[3] = 'W';
+	cbuf[4] = '\r';
 
 	for (i = 0; i < 3; i++)
 		if (confirm_write(cbuf, clen))
@@ -561,7 +499,7 @@ static int instcmd_stayoff(void)
 	return STAT_INSTCMD_HANDLED;		/* FUTURE: failed */
 }	
 
-int instcmd(const char *cmdname, const char *extra)
+static int instcmd(const char *cmdname, const char *extra)
 {
 	if (!strcasecmp(cmdname, "test.battery.start"))
 		return instcmd_btest();
@@ -575,15 +513,13 @@ int instcmd(const char *cmdname, const char *extra)
 /* install pointers to functions for msg handlers called from msgparse */
 static void setuphandlers(void)
 {
-	upsh.new_instcmd = instcmd;
+	upsh.instcmd = instcmd;
 }
 
 void upsdrv_banner(void)
 {
 	printf("Network UPS Tools - CyberPower driver %s (%s)\n", 
 		DRV_VERSION, UPS_VERSION);
-
-	experimental_driver = 1;
 }
 
 void upsdrv_help(void)
@@ -599,25 +535,13 @@ void upsdrv_initups(void)
 {
 	int dtr_bit = TIOCM_DTR;
 	int rts_bit = TIOCM_RTS;
-	struct termios tio;
 
-	open_serial(device_path, B1200);
+	upsfd = ser_open(device_path);
+	ser_set_speed(upsfd, device_path, B1200);
 
 	/* dtr high, rts high */
 	ioctl(upsfd, TIOCMBIS, &rts_bit);
 	ioctl(upsfd, TIOCMBIS, &dtr_bit);
-
-	tcgetattr(upsfd, &tio);
-	tio.c_cflag = 0 | CS8 | CLOCAL | ~CRTSCTS | CREAD;
-	cfmakeraw(&tio);
-	tio.c_cflag &= ~(PARODD|CSTOPB|HUPCL|CRTSCTS);
-
-	cfsetispeed(&tio, B1200);
-	cfsetospeed(&tio, B1200);
-	tcsetattr(upsfd, TCSANOW, &tio);
-
-	/* avoid getting stuck in read() */
-	setup_sigalrm();
 }
 
 void upsdrv_initinfo(void)
@@ -626,7 +550,7 @@ void upsdrv_initinfo(void)
 		printf("Unable to detect a CyberPower UPS on port %s\n", 
 			device_path);
 		printf("Check the cabling, port name or model name and try again\n");
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 
 	dstate_setinfo("ups.mfr", "CyberPower", 0, 0);
@@ -645,4 +569,5 @@ void upsdrv_initinfo(void)
 
 void upsdrv_cleanup(void)
 {
+	ser_close(upsfd, device_path);
 }
