@@ -21,10 +21,21 @@
 #define DSTATE_H_SEEN 1
 
 #include "parseconf.h"
+#include "upshandler.h"
 
 #define DS_LISTEN_BACKLOG 16
 #define DS_MAX_READ 256		/* don't read forever from upsd */
 
+/* return values for instcmd and set calls */
+
+#define STAT_INSTCMD_HANDLED    0x0001		/* all we do for now */
+#define STAT_INSTCMD_UNKNOWN    0x0002
+/* FUTURE: add details: pass/fail, etc */
+
+#define STAT_SET_HANDLED        0x0010
+#define STAT_SET_UNKNOWN        0x0011
+/* FUTURE: same */ 
+
 /* track client connections */
 struct conn_t {
 	int     fd;
@@ -32,8 +43,10 @@ struct conn_t {
 	void    *next;
 };
 
+	extern	struct	ups_handler	upsh;
+
 void dstate_init(const char *prog, const char *port);
-void dstate_poll_fds(int interval);
+int dstate_poll_fds(int interval, int extrafd);
 int dstate_setinfo(const char *var, const char *fmt, ...);
 int dstate_addenum(const char *var, const char *fmt, ...);
 void dstate_setflags(const char *var, int flags);
@@ -50,4 +63,20 @@ const struct cmdlist_t *dstate_getcmdlist(void);
 void dstate_dataok(void);
 void dstate_datastale(void);
 
+int dstate_is_stale(void);
+
+/* clean out the temp space for a new pass */
+void status_init(void);
+
+/* add a status element */
+void status_set(const char *buf);
+
+/* write the temporary status_buf into ups.status */
+void status_commit(void);
+
+/* similar functions for ups.alarm */
+void alarm_init(void);
+void alarm_set(const char *buf);
+void alarm_commit(void);
+
 #endif	/* DSTATE_H_SEEN */
