@@ -33,20 +33,42 @@
 
 #include "config.h"
 #include "proto.h"
-#include "shared.h"
-#include "shared-tables.h"
-#include "shared-tables-init.h"
 #include "version.h"
-#include "upscommon.h"
 #include "common.h"
 #include "parseconf.h"
 #include "dstate.h"
 #include "extstate.h"
 #include "state.h"
 
-	int	exit_flag = 0;		/* used by upscommon */
+#define DC_PROMPT "\nCommand (? for help): "
 
-	PCONF_CTX	ctx;
+	/* signal handling */
+	static	int	exit_flag = 0;
+	static	sigset_t		dc_sigmask;
+	static	struct	sigaction	dc_sa;
+
+	static	PCONF_CTX	ctx;
+
+static void set_exit_flag(int sig)
+{
+	exit_flag = sig;
+}
+
+static void setup_signals(void)
+{
+	sigemptyset(&dc_sigmask);
+	dc_sa.sa_mask = dc_sigmask;
+	dc_sa.sa_flags = 0;
+
+	dc_sa.sa_handler = set_exit_flag;
+	sigaction(SIGTERM, &dc_sa, NULL);
+	sigaction(SIGINT, &dc_sa, NULL);
+	sigaction(SIGQUIT, &dc_sa, NULL);
+
+	dc_sa.sa_handler = SIG_IGN;
+	sigaction(SIGHUP, &dc_sa, NULL);
+	sigaction(SIGPIPE, &dc_sa, NULL);
+}
 
 static void initinfo(void)
 {
@@ -64,20 +86,23 @@ static void do_help(void)
 	printf("? / h / help	- show this help\n");
 	printf("ac CMD		- add command name CMD\n");
 	printf("ae VAR ENUM	- add enum value ENUM to VAR\n");
+	printf("alc             - call dstate alarm_commit()\n");
+	printf("ali             - call dstate alarm_init()\n");
+	printf("als ALARM       - pass ALARM to dstate alarm_set()\n");
 	printf("d VAR		- delete variable VAR\n");
 	printf("dc CMD		- delete command name CMD\n");
 	printf("de VAR ENUM	- delete enum value ENUM from VAR\n");
 	printf("l / list	- show current variables in use\n");
 	printf("lc / listcmd	- show current commands\n");
 	printf("s / set VAR VAL - set variable VAR to VAL\n");
-	printf("                  example: s STATUS OL\n");
+	printf("                  example: s ups.status OL\n");
 	printf("sa VAR NUM	- set auxdata value NUM on variable VAR\n");
 	printf("sf VAR FLAG	- set flag FLAG on variable VAR\n");
-	printf("stale / dataok	- toggle staleness flag\n");
+	printf("stale / dataok	- control staleness flag\n");
 	printf("q / quit	- quit\n");
 	printf("\n");
 	printf("Values can use \"quotes\" to store values with embedded spaces.\n");
-	printf("Example: s status \"OL LB\"\n");
+	printf("Example: s ups.status \"OL LB\"\n");
 }
 
 static void do_set(const char *var, const char *val)
@@ -161,15 +186,13 @@ static void do_list(void)
 /* for future cleanup activities */
 static void do_quit(void)
 {
-	pconf_finish(&ctx);
-
-	exit(0);
+	exit(EXIT_SUCCESS);
 }
 
 static void fake_shutdown(void)
 {
 	printf("dummycons: not a driver, nothing to shutdown...\n");
-	exit(0);
+	exit(EXIT_SUCCESS);
 }
 
 static void help(const char *progname)
@@ -183,7 +206,7 @@ static void help(const char *progname)
 	printf("  -u USER	- switch to USER if started as root\n");
 	printf("  <portname>	- fake port name (for socket))\n");
 
-	exit(0);
+	exit(EXIT_SUCCESS);
 }
 
 static int parse_args(int numargs, char **arg)
@@ -224,9 +247,27 @@ static int parse_args(int numargs, char **arg)
 		return 1;
 	}
 
+	if (!strcmp(arg[0], "ali")) {
+		printf("alarm_init()\n");
+		alarm_init();
+		return 1;
+	}
+
+	if (!strcmp(arg[0], "alc")) {
+		printf("alarm_commit()\n");
+		alarm_commit();
+		return 1;
+	}
+
 	if (numargs < 2)
 		return 0;
 
+	if (!strcmp(arg[0], "als")) {
+		printf("alarm_set(%s)\n", arg[1]);
+		alarm_set(arg[1]);
+		return 1;
+	}
+
 	if ((!strcmp(arg[0], "d")) || (!strcmp(arg[0], "delete"))) {
 		if (!dstate_delinfo(arg[1]))
 			printf("Can't delete (%s) - not found\n", arg[1]);
@@ -315,22 +356,29 @@ static int parse_args(int numargs, char **arg)
 	return 0;
 }
 
-int dc_setvar(const char *varname, const char *val)
+static int dc_setvar(const char *varname, const char *val)
 {
 	printf("SETVAR from socket: [%s] [%s]\n", varname, val);
 	return STAT_SET_UNKNOWN;
 }
 
-int dc_instcmd(const char *cmdname, const char *extra)
+static int dc_instcmd(const char *cmdname, const char *extra)
 {
 	printf("INSTCMD from socket: [%s]\n", cmdname);
 	return STAT_INSTCMD_UNKNOWN;
 }
 
+static void exit_cleanup(void)
+{
+	pconf_finish(&ctx);
+	dstate_free();
+}
+
 int main(int argc, char **argv)
 {
-	int	res, i;
-	char	buf[128], *portname, *prog, *user;
+	int	ret, i;
+	char	buf[128], *prog;
+	const	char	*portname, *user;
 	struct	passwd	*new_uid = NULL;
 
 	printf("Network UPS Tools - Dummy console UPS driver %s\n", UPS_VERSION);
@@ -353,7 +401,7 @@ int main(int argc, char **argv)
 				break;
 
 			case 'V':
-				exit(0);
+				exit(EXIT_SUCCESS);
 		}
 	}
 
@@ -381,41 +429,21 @@ int main(int argc, char **argv)
 	memset(&upsh, '\0', sizeof(upsh));
 
 	/* install dummy handlers for SET and INSTCMD */
-	upsh.new_setvar = dc_setvar;
-	upsh.new_instcmd = dc_instcmd;
+	upsh.setvar = dc_setvar;
+	upsh.instcmd = dc_instcmd;
 
 	dstate_init("dummycons", portname);
 
-	printf("\nCommand (? for help): ");
+	printf(DC_PROMPT);
 	fflush(stdout);
 
 	pconf_init(&ctx, NULL);
 
-	for (;;) {
-		struct	timeval	tv;
-		fd_set	rfd;		
+	while (exit_flag == 0) {
+		ret = dstate_poll_fds(30, fileno(stdin));
 
-		tv.tv_sec = 2;
-		tv.tv_usec = 0;
-		FD_ZERO(&rfd);
-		FD_SET(fileno(stdin), &rfd);
-
-		res = select(fileno(stdin) + 1, &rfd, NULL, NULL, &tv);
-
-		if (exit_flag == 1)
-			do_quit();
-
-		if (res < 0)
-			fatal("select");
-
-		/* FIXME - this should be checked in a single select above */
-
-		/* this is where we would use another dstate_poll function *
-		 * that takes an arg from the caller - another fd to check */
-		dstate_poll_fds(0);
-
-		/* idle loop */
-		if (res == 0)
+		/* idle */
+		if (ret == 0)
 			continue;
 
 		fgets(buf, sizeof(buf), stdin);
@@ -431,8 +459,11 @@ int main(int argc, char **argv)
 			}
 		}
 
-		printf("\nCommand (? for help): ");
+		printf(DC_PROMPT);
 		fflush(stdout);
 
-	}	/* for (;;) */
+	}	/* while (exit_flag == 0) */
+
+	upslogx(LOG_INFO, "Signal %d: exiting", exit_flag);
+	exit(EXIT_SUCCESS);
 }
