@@ -28,24 +28,29 @@
 
 
 #include "main.h"
+#include "serial.h"
 
-#define UPSDELAY 3
-#define MAXTRIES 10
-#define UPSMFR "Energy Sistem"
+#define UPSDELAY	3
+#define MAXTRIES	10
+#define UPSMFR		"Energy Sistem"
+#define SENDDELAY	100		/* 100 usec between chars on send */
+#define SER_WAIT_SEC	3		/* max 3.0 sec window for reads */
+#define SER_WAIT_USEC	0
 
 #define UPSTYPE "UPS Smart/Advanced"
-char UPSFIRM[20] = " ";
-int UPSFIRMSET = 0;
+
+static	char UPSFIRM[20] = " ";
+static	int UPSFIRMSET = 0;
 
 #define Q1  1
 #define F  2
 
-#define DRIVERVERSION "0.20"
+#define DRIVERVERSION "0.22"
 
-int     DEBUG;
-int     type;
-char    strwakedelay[5];
-char    strdelayshut[3];
+static	int     DEBUG;
+static	int     type;
+static	char    strwakedelay[5];
+static	char    strdelayshut[3];
 
 
 #define MINVOLTGESSFACT12 0.91667
@@ -53,8 +58,8 @@ char    strdelayshut[3];
 #define MAXVOLTGESSFACT12 1.16667
 #define MAXVOLTGESSFACT24 1.14167
 
-float   minbattvolt = -1;
-float   maxbattvolt = -1;
+static	float   minbattvolt = -1;
+static	float   maxbattvolt = -1;
 
 
 /**********************************************************************
@@ -63,7 +68,8 @@ float   maxbattvolt = -1;
 * If the number is out of the range 'min-max' the output string goes to defval
 ***********************************************************************/
 
-void parse_str_num(const char *org, char *dest, int prec, int numdec, float min, float max, float defval) {
+static void parse_str_num(const char *org, char *dest, int prec, 
+	int numdec, float min, float max, float defval) {
       float num;
       char myfmtstr[7] = "%04.0f";
            
@@ -77,7 +83,7 @@ void parse_str_num(const char *org, char *dest, int prec, int numdec, float min,
       sprintf(dest,myfmtstr,num);
 }
 
-void get_battpct(char * org, char * dest, float minv, float maxv) {
+static void get_battpct(char * org, char * dest, float minv, float maxv) {
    float curvolt;
    curvolt = atof(org);
    curvolt = (((curvolt < minv ? minv : curvolt) - minv) / (maxv - minv))*100;
@@ -95,7 +101,7 @@ void get_battpct(char * org, char * dest, float minv, float maxv) {
  * Returns NULL on reaching the end of the string.
  * 
  ********************************************************************/
-char *StringSplit( char *source, char *word, int maxlen )
+static char *StringSplit( char *source, char *word, int maxlen )
 {
     int     i;
     int     len;
@@ -125,7 +131,7 @@ char *StringSplit( char *source, char *word, int maxlen )
  * "word" has to be exacly the size of "source".
  * 
  ********************************************************************/
-void StringStrip( char *source, char *word )
+static void StringStrip( char *source, char *word )
 {
     int     wc=0;
     int     i;
@@ -153,7 +159,7 @@ void StringStrip( char *source, char *word )
  * command. Sets the INFO_STATUS value ( OL, OB, ... )
  * 
  ********************************************************************/
-void parseFlags( char *flags )
+static void parseFlags( char *flags )
 {
 
     status_init();
@@ -189,7 +195,7 @@ void parseFlags( char *flags )
  * Also sets various values (IPFreq ... )
  * 
  ********************************************************************/
-void query1( char *buf )
+static void query1( char *buf )
 {
     #define WORDMAXLEN 255
     char    value[WORDMAXLEN];
@@ -267,7 +273,7 @@ void query1( char *buf )
  * On this ups seems to be static data from firmware.
  * 
  ********************************************************************/
-void queryF( char *buf )
+static void queryF( char *buf )
 {
     #define WORDMAXLEN 255
     char    value[WORDMAXLEN];
@@ -356,7 +362,7 @@ void queryF( char *buf )
  * 
  ********************************************************************/
 
-int ups_ident( int displaymsg )
+static int ups_ident( int displaymsg )
 {
     char    buf[255];
     int     ret;
@@ -366,8 +372,11 @@ int ups_ident( int displaymsg )
        fflush(stdout);
           
     /* Check presence of Q1 */
-    ret = upssend( "Q1\x0D" );
-    ret = upsrecv( buf, 250, '\x0D', "");
+    ret = ser_send_pace(upsfd, SENDDELAY, "Q1\r");
+
+    ret = ser_get_line(upsfd, buf, sizeof(buf), '\r', "",
+	SER_WAIT_SEC, SER_WAIT_USEC);
+
     ret = strlen( buf );
     if( ret != 46 ) 
     {
@@ -386,8 +395,11 @@ int ups_ident( int displaymsg )
     }
 
     /* COMANDO F */
-    ret = upssend( "F\x0D" );
-    ret = upsrecv( buf, 250, '\x0D', "");
+    ret = ser_send_pace(upsfd, SENDDELAY, "F\r");
+
+    ret = ser_get_line(upsfd, buf, sizeof(buf), '\r', "",
+	SER_WAIT_SEC, SER_WAIT_USEC);
+
     ret = strlen( buf );
     if( ret == 21 ) 
     {
@@ -402,51 +414,55 @@ int ups_ident( int displaymsg )
 
 /*************** instcmd *************************/
 
-/* void instcmd(int auxcmd, int dlen, char *data) { */
-int instcmd(const char *cmdname, const char *extra) { 
-    char aux[50];
-     aux[0] = '\0';
+static int instcmd(const char *cmdname, const char *extra)
+{
+	int	ret;
+	char	aux[50];
 
         if (!strcasecmp(cmdname, "shutdown.stop")) {
-             upssend("C\x0D");
-             upslogx(LOG_INFO,"Cancel shutdown sent to UPS.");
-             return STAT_INSTCMD_HANDLED;
+		ret = ser_send_pace(upsfd, SENDDELAY, "C\r");
+		upslogx(LOG_INFO,"Cancel shutdown sent to UPS.");
+		return STAT_INSTCMD_HANDLED;
         }     
 
         if (!strcasecmp(cmdname, "test.battery.start")) {
-             upssend("T\x0D");
-             upslogx(LOG_INFO,"Start 10 Seconds battery test.");
-             return STAT_INSTCMD_HANDLED;
+		ret = ser_send_pace(upsfd, SENDDELAY, "T\r");
+		upslogx(LOG_INFO,"Start 10 Seconds battery test.");
+		return STAT_INSTCMD_HANDLED;
         }     
 
         if (!strcasecmp(cmdname, "shutdown.return")) {
-             sprintf(aux, "S%sR%s\x0D", strdelayshut, strwakedelay);
-             upssend(aux);
-             upslogx(LOG_INFO,"Sending shutdown command '%s' to UPS", aux);
-             return STAT_INSTCMD_HANDLED;
+		snprintf(aux, sizeof(aux), "S%sR%s\r", 
+			strdelayshut, strwakedelay);
+		ret = ser_send_pace(upsfd, SENDDELAY, aux);
+             
+		upslogx(LOG_INFO,"Sending shutdown command '%s' to UPS", aux);
+		return STAT_INSTCMD_HANDLED;
         }     
 
         if (!strcasecmp(cmdname, "shutdown.stayoff")) {
-             sprintf(aux, "S%sR0000\x0D", strdelayshut);
-             upssend(aux);
-             upslogx(LOG_INFO,"Sending shutdown command '%s' to UPS", aux);
-             return STAT_INSTCMD_HANDLED;
+		snprintf(aux, sizeof(aux), "S%sR0000\r", strdelayshut);
+		ret = ser_send_pace(upsfd, SENDDELAY, aux);
+
+		upslogx(LOG_INFO,"Sending shutdown command '%s' to UPS", aux);
+		return STAT_INSTCMD_HANDLED;
         }     
 
         if (!strcasecmp(cmdname, "load.off")) {
-             upssend("C\x0D");         
-             upssend("S00R0000\x0D");
-             upslogx(LOG_INFO,"Turning off load on UPS.");
-             return STAT_INSTCMD_HANDLED;
+		ret = ser_send_pace(upsfd, SENDDELAY, "C\r");
+		ret = ser_send_pace(upsfd, SENDDELAY, "S00R0000\r");
+		upslogx(LOG_INFO,"Turning off load on UPS.");
+		return STAT_INSTCMD_HANDLED;
         }     
 
         if (!strcasecmp(cmdname, "load.on")) {
-             upssend("C\x0D");
-             upslogx(LOG_INFO,"Turning on load on UPS.");
-             return STAT_INSTCMD_HANDLED;
+		ret = ser_send_pace(upsfd, SENDDELAY, "C\r");
+		upslogx(LOG_INFO,"Turning on load on UPS.");
+		return STAT_INSTCMD_HANDLED;
         }     
-        upslogx(LOG_INFO,"Unknown command '%s'", cmdname);   
-        return STAT_INSTCMD_UNKNOWN;         
+
+	upslogx(LOG_INFO,"Unknown command '%s'", cmdname);   
+	return STAT_INSTCMD_UNKNOWN;         
 }
 
 
@@ -462,8 +478,7 @@ void upsdrv_help( void )
 
 }
 
-/* void ups_set_var(int type, int dlen, char *data) { */
-int ups_set_var(const char* varname, const char *val) {
+static int ups_set_var(const char* varname, const char *val) {
 
     
     /* Note Command for instant shutdown with no restart: S00R0000 */
@@ -490,6 +505,8 @@ int ups_set_var(const char* varname, const char *val) {
  ********************************************************************/
 void upsdrv_initinfo(void)
 {
+    dstate_setinfo("driver.version.internal", "%s", DRIVERVERSION);
+
   /* Q1 Query */ 
     dstate_setinfo( "ups.mfr", UPSMFR); /* INFO_MFR*/
     dstate_setinfo( "ups.model", UPSTYPE); /*INFO_MODEL*/
@@ -524,8 +541,8 @@ void upsdrv_initinfo(void)
     dstate_addcmd("load.off"); /* (INFO_INSTCMD, "", 0, CMD_ON);*/
     dstate_addcmd("load.on"); /* (INFO_INSTCMD, "", 0, CMD_ON);*/
 
-    upsh.new_instcmd = instcmd; 
-    upsh.new_setvar = ups_set_var;
+    upsh.instcmd = instcmd; 
+    upsh.setvar = ups_set_var;
 
 
 }
@@ -554,10 +571,10 @@ void upsdrv_updateinfo(void)
      /* F found ? */
     if( type & F ) 
     {
-        upssend( "F\x0D" );
+        ser_send_pace(upsfd, SENDDELAY, "%s", "F\x0D" );
         sleep( UPSDELAY );
         buf[0] = '\0';
-        upsrecv( buf, 250, '\x0D', "\n");
+	ser_get_line(upsfd, buf, sizeof(buf), '\r', "\n", 3, 0);
         ret = strlen( buf );
         if( ret != 21 ) 
         {
@@ -574,10 +591,10 @@ void upsdrv_updateinfo(void)
     /* Q1 found ? */
     if( type & Q1 )
     {
-        upssend( "Q1\x0D" );
+        ser_send_pace(upsfd, SENDDELAY, "%s", "Q1\x0D" );
         sleep( UPSDELAY );
         buf[0] = '\0';
-        upsrecv( buf, 250, '\x0D', "\n");
+	ser_get_line(upsfd, buf, sizeof(buf), '\r', "\n", 3, 0);
         ret = strlen( buf );
         if( ret != 46 ) 
         {
@@ -607,8 +624,8 @@ void upsdrv_updateinfo(void)
 void upsdrv_shutdown(void)
 {
     upslogx( LOG_INFO, "Sending inmediate shutdown to UPS.");
-    upssend( "C\x0D");
-    upssend( "S00R0001\x0D" );
+    ser_send_pace(upsfd, SENDDELAY, "%s", "C\x0D");
+    ser_send_pace(upsfd, SENDDELAY, "%s", "S00R0001\x0D" );
 }
 
 /********************************************************************
@@ -662,8 +679,8 @@ void upsdrv_initups(void)
     
     DEBUG = 0;
 	/* setup serial port */
-    upssend_delay = 100; 
-	open_serial( device_path, B2400 );
+    upsfd = ser_open(device_path);
+    ser_set_speed(upsfd, device_path, B2400);
    
     
     strcpy(strwakedelay,"0003");
@@ -687,7 +704,7 @@ void upsdrv_initups(void)
     if( ! good )
     {        
         printf( "No UPS Smart UPS found!\n" );
-        exit( 1 );
+        exit(EXIT_FAILURE);
     }
        
     printf( "Energy Sistem UPS Smart found\n" );
@@ -695,8 +712,8 @@ void upsdrv_initups(void)
     /* Cancel Shutdown */
     if( getval("CS")) 
     {
-       upssend( "C\x0D" );
-       exit( 1 );
+       ser_send_pace(upsfd, SENDDELAY, "%s", "C\x0D" );
+       exit(EXIT_FAILURE);
     }
     if ( getval("DEBUG")) 
     {
@@ -722,16 +739,6 @@ void upsdrv_initups(void)
     }          
 }
 
-/********************************************************************
- *
- * tell main how many items in the infostructure are needed.
- * 
- ********************************************************************/
-int upsdrv_infomax( void )
-{
-    return 24;
-}
-
 /********************************************************************
  * From the masterguard.c Author:
  *   VIM Preferences.
@@ -745,4 +752,5 @@ int upsdrv_infomax( void )
 
 void upsdrv_cleanup(void)
 {
+	ser_close(upsfd, device_path);
 }
