@@ -18,40 +18,43 @@
 */
 
 #include "main.h"
+#include "serial.h"
 #include <sys/ioctl.h>
 
+#define DRV_VERSION "0.02"
 
-	unsigned char	upstype = 0;
 
-void init_serial(void)
+static	unsigned char	upstype = 0;
+
+static void init_serial(void)
 {
         int     clr_bit = TIOCM_DTR | TIOCM_RTS;
         ioctl(upsfd, TIOCMBIC, &clr_bit);
 }
 
-int Code(int tries)
+static int Code(int tries)
 {
 	unsigned char cRecv;
 	do {
-		upssendchar(208);
-		upsrecvchars((char*) &cRecv,1);
+		ser_send_char(upsfd, 208);
+		ser_get_char(upsfd, &cRecv, 3, 0);
 		if (cRecv==208)
 			return 1;
 	} while (--tries>0);
 	return 0;
 }
 
-int InitUpsType()
+static int InitUpsType(void)
 {
 	if (Code(1)) {
-		upssendchar(173);
-		upsrecvchars((char*) &upstype,1);
+		ser_send_char(upsfd, 173);
+		ser_get_char(upsfd, &upstype, 3, 0);
 		return 1;
 	} else
 		return 0;
 }
 
-char *GetTypeUpsName()
+static const char *GetTypeUpsName(void)
 {
         switch(upstype)
         {
@@ -87,15 +90,15 @@ void upsdrv_updateinfo(void)
 		return;
 	}
 	/*Line status*/
-	upssendchar(175);
-	upsrecvchars((char*)recBuf,1);
+	ser_send_char(upsfd, 175);
+	ser_get_char(upsfd, recBuf, 3, 0);
 	if ((recBuf[0] & 1) !=0)
 		standby=1;
 	else 
 		battery=(recBuf[0] &4) !=0;
 	if (Code(1)) {  /*Accumulator voltage value*/
-		upssendchar(189);
-		upsrecvchars((char*)recBuf,1);
+		ser_send_char(upsfd, 189);
+		ser_get_char(upsfd, recBuf, 3, 0);
 		acuV=((unsigned long)recBuf[0])*150;
 		acuV/=255;
 	} else {
@@ -104,8 +107,8 @@ void upsdrv_updateinfo(void)
 		return;
 	}
 	if (Code(1)) {  /*Line voltage*/
-		upssendchar(245);
-		upsrecvchars((char*) recBuf,2);
+		ser_send_char(upsfd, 245);
+		ser_get_buf_len(upsfd, recBuf, 2, 3, 0);
 		lineV=(recBuf[0]*100+recBuf[1]*25600)/372;
 	} else {
 		upslog(LOG_INFO, "Code failed");
@@ -149,14 +152,14 @@ void upsdrv_shutdown(void)
 		upslogx(LOG_INFO, "Code failed: %m\n");
 		return;
 	}
-	upssendchar(28);
-	upssendchar(1);  /* 1.28 sec */
+	ser_send_char(upsfd, 28);
+	ser_send_char(upsfd, 1);  /* 1.28 sec */
 	if (!Code(1)) {
 		upslogx(LOG_INFO, "Code failed: %m\n");
 		return;
 	}
-	upssendchar(13);
-	upssendchar(8);
+	ser_send_char(upsfd, 13);
+	ser_send_char(upsfd, 8);
 }
 
 void upsdrv_help(void)
@@ -166,25 +169,24 @@ void upsdrv_help(void)
 /* list flags and values that you want to receive via -x */
 void upsdrv_makevartable(void)
 {
-	/* allow '-x xyzzy' */
-	/* addvar(ARG_FLAG, "xyzzy", "Enable xyzzy mode"); */
-
-	/* allow '-x foo=<some value>' */
-	/* addvar(ARG_VALUE, "foo", "Override foo setting"); */
 }
 
 void upsdrv_banner(void)
 {
-	printf("Network UPS Tools - Ever UPS driver 0.01 (%s)\n\n", UPS_VERSION);
+	printf("Network UPS Tools - Ever UPS driver %s (%s)\n\n", 
+		DRV_VERSION, UPS_VERSION);
 }
 
 void upsdrv_initups(void)
 {
-	open_serial (device_path,B300);
+	upsfd = ser_open(device_path);
+	ser_set_speed(upsfd, device_path, B300);
+
 	init_serial();
 	InitUpsType();
 }
 
 void upsdrv_cleanup(void)
 {
+	ser_close(upsfd, device_path);
 }
