@@ -18,15 +18,19 @@
 */
 
 #include "main.h"
+#include "serial.h"
+
 #include "fentonups.h"
 
-#define ENDCHAR 13	/* replies end with CR */
+#define ENDCHAR 	13	/* replies end with CR */
+#define SER_WAIT_SEC	3	/* allow 3.0 sec for ser_get calls */
+#define SER_WAIT_USEC	0
 
-#define	DRV_VERSION "1.11"
+#define	DRV_VERSION "1.21"
 
 static	int	cap_upstemp = 0;
 static	float	lowvolt = 0, voltrange;
-static	int	lownorm, highnorm, poll_failures = 0;
+static	int	lownorm, highnorm;
 
 /* handle devices which don't give a properly formatted I string */
 static int check_mtab2(const char *raw)
@@ -67,7 +71,7 @@ static int check_mtab2(const char *raw)
 
 static void guessmodel(const char *raw)
 {
-	char	mch, mstr[256];
+	char	mch, *mstr;
 
 	/* first see if it's in the mtab2 */
 	if (check_mtab2(raw))
@@ -75,7 +79,7 @@ static void guessmodel(const char *raw)
 
 	mch = raw[17];
 
-	strlcpy(mstr, &raw[18], sizeof(mstr));
+	mstr = xstrdup(&raw[18]);
 	mstr[10] = '\0';	/* 10 chars max, per the protocol */
 
 	/* trim whitespace */
@@ -105,17 +109,29 @@ static void guessmodel(const char *raw)
 				"please report this ID string: %s", raw);
 			break;
 	}
+
+	free(mstr);
 }
 
-int instcmd(const char *cmdname, const char *extra)
+static int instcmd(const char *cmdname, const char *extra)
 {
+	int	ret;
+
 	if (!strcasecmp(cmdname, "test.battery.start")) {
-		upssend("T\r");
+		ret = ser_send(upsfd, "T\r");
+
+		if (ret != 2)
+			upslog(LOG_ERR, "instcmd: ser_send failed");
+
 		return STAT_INSTCMD_HANDLED;
 	}
 
 	if (!strcasecmp(cmdname, "test.battery.stop")) {
-		upssend("CT\r");
+		ret = ser_send(upsfd, "CT\r");
+
+		if (ret != 2)
+			upslog(LOG_ERR, "instcmd: ser_send failed");
+
 		return STAT_INSTCMD_HANDLED;
 	}
 
@@ -130,11 +146,16 @@ static char *get_id(void)
 
 	/* try to get the initial UPS data */
 	for (i = 0; i < 5; i++) {
-		upssend("I\r");
-		ret = upsrecv(temp, sizeof(temp), ENDCHAR, "");
+		ret = ser_send(upsfd, "I\r");
+
+		if (ret != 2)
+			upslog(LOG_ERR, "get_id: ser_send failed");
+
+		ret = ser_get_line(upsfd, temp, sizeof(temp), ENDCHAR, "", 
+			SER_WAIT_SEC, SER_WAIT_USEC);
 
 		if (ret < 1) {
-			upslog(LOG_ERR, "Short read during UPS id sequence");
+			upslogx(LOG_ERR, "Short read during UPS id sequence");
 			continue;
 		}
 
@@ -148,25 +169,18 @@ static char *get_id(void)
 		return xstrdup(temp);
 	}
 
-	fatalx("Unable to detect a Fenton or Megatec protocol UPS");
-
-	/* NOTREACHED */
 	return NULL;
 }
 
 void upsdrv_initinfo(void)
 {
+	int	modelnum, i, ret;
 	char	temp[256], model[32], *raw;
-	int	modelnum, i;
-
-	/* dummy read attempt to sync - throw it out */
-	upssend("I\r");
-	upsrecv(temp, sizeof(temp), ENDCHAR, "");
 
 	raw = get_id();
 
-	if (!raw)	/* shouldn't happen */
-		fatalx("Didn't get a usable ID string");
+	if (!raw)
+		fatalx("Unable to detect a Fenton or Megatec protocol UPS");
 
 	snprintf(temp, sizeof(temp), "%s", raw);
 
@@ -220,58 +234,62 @@ void upsdrv_initinfo(void)
 	free(raw);
 
 	/* paranoia - cancel any shutdown that might already be running */
-	upssend("C\r");
+	ret = ser_send(upsfd, "C\r");
 
-	upsh.new_instcmd = instcmd;
+	if (ret != 2)
+		upslog(LOG_ERR, "upsdrv_initinfo: ser_send failed");
+
+	upsh.instcmd = instcmd;
 
 	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
 }
 
-static void pollfail(char *why)
-{
-	poll_failures++;
-
-	/* don't spew into the syslog forever */
-	if (poll_failures < 3)
-		upslogx(LOG_ERR, why);
-
-	return;
-}
-
 void upsdrv_updateinfo(void)
 {
 	char	temp[256], involt[16], loadpct[16], acfreq[16], battvolt[16],
-		upstemp[16], stat[16], outvolt[16];
+		upstemp[16], pstat[16], outvolt[16];
 	int	util, ret;
 	double	bvoltp;
 
-	upssend("Q1\r");
+	ret = ser_send(upsfd, "Q1\r");
 
-	ret = upsrecv(temp, sizeof(temp), ENDCHAR, "");
+	if (ret != 3)
+		upslog(LOG_ERR, "upsdrv_updateinfo: ser_send failed");
 
-	/* sanity checks for poll data */
-	if (strlen(temp) < 46) {
-		pollfail("Poll failed: short read from UPS");
+	/* give it a chance to return the full line */
+	usleep(300000);
+
+	ret = ser_get_line(upsfd, temp, sizeof(temp), ENDCHAR, "", 
+		SER_WAIT_SEC, SER_WAIT_USEC);
+
+	if (ret < 1) {
+		ser_comm_fail(NULL);
 		dstate_datastale();
 		return;
 	}
 
-	if (strlen(temp) > 46) {
-		pollfail("Poll failed: oversized read from UPS");
+	/* sanity checks for poll data */
+	if (ret < 46) {
+		ser_comm_fail("Poll failed: short read (got %d bytes)", ret);
+		dstate_datastale();
+		return;
+	}
+
+	if (ret > 46) {
+		ser_comm_fail("Poll failed: response too long (got %d bytes)", 
+			ret);
 		dstate_datastale();
 		return;
 	}
 
 	if (temp[0] != '(') {
-		pollfail("Poll failed: invalid start character");
+		ser_comm_fail("Poll failed: invalid start character (got %02x)",
+			temp[0]);
 		dstate_datastale();
 		return;
 	}
 
-	if (poll_failures > 0)
-		upslogx(LOG_NOTICE, "UPS poll succeeded");
-
-	poll_failures = 0;		
+	ser_comm_good();
 
 	/* (MMM.M NNN.N PPP.P QQQ RR.R S.SS TT.T  b7b6b5b4b3b2b1b0<cr>
 	 *
@@ -281,7 +299,7 @@ void upsdrv_updateinfo(void)
 	 */
 
 	sscanf(temp, "%*c%s %*s %s %s %s %s %s %s", involt, outvolt, loadpct,
-		acfreq, battvolt, upstemp, stat);
+		acfreq, battvolt, upstemp, pstat);
 
 	dstate_setinfo("input.voltage", "%s", involt);
 	dstate_setinfo("output.voltage", "%s", outvolt);
@@ -298,11 +316,11 @@ void upsdrv_updateinfo(void)
 
 	util = atoi(involt);
 
-	if (stat[0] == '0') {
+	if (pstat[0] == '0') {
 		status_set("OL");		/* on line */
 
 		/* only allow these when OL since they're bogus when OB */
-		if (stat[2] == '1') {		/* boost or trim in effect */
+		if (pstat[2] == '1') {		/* boost or trim in effect */
 			if (util < lownorm)
 				status_set("BOOST");
 
@@ -314,7 +332,7 @@ void upsdrv_updateinfo(void)
 		status_set("OB");		/* on battery */
 	}
 
-	if (stat[1] == '1')
+	if (pstat[1] == '1')
 		status_set("LB");		/* low battery */
 
 	status_commit();
@@ -328,29 +346,58 @@ void upsdrv_updateinfo(void)
 	dstate_dataok();
 }
 
+static int ups_on_line(void)
+{
+	int	ret;
+	char	temp[256], pstat[32];
+
+	ret = ser_send(upsfd, "Q1\r");
+
+	if (ret != 3) {
+		printf("Status request send failed, assuming on battery state");
+		return 0;
+	}
+
+	ret = ser_get_line(upsfd, temp, sizeof(temp), ENDCHAR, "", 
+		SER_WAIT_SEC, SER_WAIT_USEC);
+
+	if (ret < 1) {
+		printf("Status request read failed, assuming on battery state\n");
+		return 0;
+	}
+
+	if (strlen(temp) != 46) {
+		printf("Status request read returned garbage (%d bytes), "
+			"assuming on battery state\n", strlen(temp));
+		return 0;
+	}
+
+	sscanf(temp, "%*s %*s %*s %*s %*s %*s %*s %s", pstat);	
+
+	if (pstat[0] == '1')
+		return 0;	/* on battery */
+
+	return 1;	/* on line */
+}
+
 /* power down the attached load immediately */
 void upsdrv_shutdown(void)
 {
-	char	temp[256], stat[32];
-
-	/* basic idea: find out line status and send appropriate command */
-
-	upssend("Q1\r");
-	upsrecv(temp, sizeof(temp), ENDCHAR, "");
-	sscanf(temp, "%*s %*s %*s %*s %*s %*s %*s %s", stat);
+	/* line status determines which command is used */
 
 	/* on battery: send S01<cr>, ups will return by itself on utility */
 	/* on line: send S01R0003<cr>, ups will cycle and return soon */
 
-	upssend("S01");
-
-	if (stat[0] == '0') {			/* on line */
+	if (ups_on_line) {
 		printf("On line, sending shutdown+return command...\n");
-		upssend("R0003");
-	} else
-		printf("On battery, sending normal shutdown command...\n");
 
-	upssendchar(13);	/* end sequence */
+		/* return values below 3 misbehave on some hardware revs */
+		ser_send(upsfd, "S01R0003\r");
+		return;
+	}
+
+	printf("On battery, sending normal shutdown command...\n");
+	ser_send(upsfd, "S01\r");
 }
 
 void upsdrv_help(void)
@@ -369,9 +416,11 @@ void upsdrv_banner(void)
 
 void upsdrv_initups(void)
 {
-	open_serial(device_path, B2400);
+	upsfd = ser_open(device_path);
+	ser_set_speed(upsfd, device_path, B2400);
 }
 
 void upsdrv_cleanup(void)
 {
+	ser_close(upsfd, device_path);
 }
