@@ -18,8 +18,8 @@
 */
 
 struct {
-	char	*mtext;
-	char	*desc;
+	const	char	*mtext;
+	const	char	*desc;
 	float	lowvolt;
 	float	voltrange;
 	int	lowxfer;
@@ -66,9 +66,9 @@ struct {
 /* devices which don't implement the I string the same way */
 
 struct {
-	char	*id;
-	char	*mfr;
-	char	*model;
+	const	char	*id;
+	const	char	*mfr;
+	const	char	*model;
 	float	lowvolt;
 	float	voltrange;
 	int	lowxfer;
