@@ -17,15 +17,17 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
-#define DRV_VERSION "1.10"
+#define DRV_VERSION "1.30"
 
 #include <sys/ioctl.h>
 #include <sys/termios.h>
 
 #include "main.h"
+#include "serial.h"
+
 #include "genericups.h"
 
-	int	upstype = -1;
+	static	int	upstype = -1;
 
 static void parse_output_signals(const char *value, int *line) 
 { 
@@ -157,40 +159,59 @@ static void set_ups_type(void)
 	
 	upstype = atoi(getval("upstype"));
 
-	for (i = 0; upstab[i].mfr != NULL; i++)
+	for (i = 0; upstab[i].mfr != NULL; i++) {
 		if (upstype == i) {
 			printf("UPS type: %s\n", upstab[i].desc);
 			return;
 		}
+	}
 
 	printf("\nFatal error: unknown UPS type number\n");
 	listtypes();
-	exit(1);
+
+	exit(EXIT_FAILURE);
 }			
 
 /* power down the attached load immediately */
 void upsdrv_shutdown(void)
 {
-	int	flags, ret;
+	int	flags, ret;;
 
 	if (upstype == -1)
-		fatalx("No upstype set - see help text / man page!\n");
+		fatalx("No upstype set - see help text / man page!");
 
 	flags = upstab[upstype].line_sd;
 
 	if (flags == -1)
-		fatalx("No shutdown command defined for this model!\n");
+		fatalx("No shutdown command defined for this model!");
 
 	if (flags == TIOCM_ST) {
-#ifdef HAVE_TCSENDBREAK
-		tcsendbreak (upsfd, 4901);
-#else
-		fatalx("Need to send a BREAK, but don't have tcsendbreak!\n");
+
+#ifndef HAVE_TCSENDBREAK
+		fatalx("Need to send a BREAK, but don't have tcsendbreak!");
 #endif
-	} else {
-		ret = ioctl(upsfd, TIOCMSET, &flags);
+
+		ret = tcsendbreak(upsfd, 4901);
+
 		if (ret != 0)
-			perror("ioctl");
+			fatal("tcsendbreak");
+
+		return;
+	}
+
+	ret = ioctl(upsfd, TIOCMSET, &flags);
+	if (ret != 0)
+		fatal("ioctl TIOCMSET");
+
+	if (getval("sdtime")) {
+		int	sdtime;
+
+		sdtime = strtol(getval("sdtime"), (char **) NULL, 10);
+
+		upslogx(LOG_INFO, "Holding shutdown signal for %d seconds...\n",
+			sdtime);
+
+		sleep(sdtime);
 	}
 }
 
@@ -201,7 +222,7 @@ void upsdrv_help(void)
 
 void upsdrv_banner(void)
 {
-        printf("Network UPS Tools - Generic UPS driver %s (%s)\n", 
+	printf("Network UPS Tools - Generic UPS driver %s (%s)\n", 
 		DRV_VERSION, UPS_VERSION);
 }
 
@@ -215,15 +236,20 @@ void upsdrv_makevartable(void)
 	addvar(VAR_VALUE, "OL", "Override on line signal"); 
 	addvar(VAR_VALUE, "LB", "Override low battery signal"); 
 	addvar(VAR_VALUE, "SD", "Override shutdown setting"); 
+	addvar(VAR_VALUE, "sdtime", "Hold time for shutdown value (seconds)");
 }
 
 void upsdrv_initups(void)
 {
 	set_ups_type();
 
-	open_serial_simple(device_path, 0, upstab[upstype].line_norm);
+	upsfd = ser_open(device_path);
+
+	if (ioctl(upsfd, TIOCMSET, &upstab[upstype].line_norm))
+		fatal("ioctl TIOCMSET");
 }
 
 void upsdrv_cleanup(void)
 {
+	ser_close(upsfd, device_path);
 }
