@@ -18,9 +18,9 @@
 */
 
 struct {
-	char	*mfr;			/* value for INFO_MFR	*/
-	char	*model;			/* value for INFO_MODEL	*/
-	char	*desc;			/* used in -h listing	*/
+	const	char	*mfr;			/* value for INFO_MFR	*/
+	const	char	*model;			/* value for INFO_MODEL	*/
+	const	char	*desc;			/* used in -h listing	*/
 	int	line_norm;
 	int	line_ol, val_ol;
 	int	line_bl, val_bl;
