@@ -90,23 +90,15 @@ void upsdrv_banner(void)
 	experimental_driver = 1;
 }
 
-static inline int find_collection(int fd, unsigned usage) 
+static inline int find_application(int fd, int usage) 
 {
+	int	i = 0, ret;
 
-/* this isn't in all 2.4 kernels... */
-#ifdef HIDIOCGCOLLECTIONINFO
-	struct hiddev_collection_info cinfo;
-
-	cinfo.index = 0;
-	cinfo.usage = 0;
-
-	ioctl(fd, HIDIOCGCOLLECTIONINFO, &cinfo);
-	return (cinfo.usage == usage);
-#else
-	upslogx(LOG_WARNING, "HIDIOCGCOLLECTIONINFO not known at compile-time, winging it...");
-	return 1;
-#endif
+	while ((ret = ioctl(fd, HIDIOCAPPLICATION, i)) > 0 &&
+	       ret != usage) 
+		i++;
 
+	return (ret == usage);
 }
 
 static void parse_event(struct hiddev_event *ev)
@@ -285,7 +277,8 @@ void upsdrv_updateinfo(void)
 {
 	fd_set	fdset;
 	struct	timeval	tv;
-	int	rd, i;
+	int	rd;
+	unsigned int	i;
 	struct	hiddev_event ev[64];
 
 	FD_ZERO(&fdset);
@@ -401,10 +394,11 @@ int setvalue(int type, int value)
 }
 
 /* handler for settable variables in UPS*/
-void setvar (int auxcmd, int dlen, char *data)
+int setvar(const char *varname, const char *val)
 {
-	;	
 	/* int setvalue(int type, int value) */
+
+	return STAT_SET_UNKNOWN;
 }
 
 void upsdrv_initinfo(void) 
@@ -544,15 +538,15 @@ void upsdrv_initups(void)
 
 	open_usb(device_path, O_RDONLY);
 
-	ioctl(fd, HIDIOCINITREPORT, 0);
-
-	if (!find_collection(fd, UPS_USAGE))
+	if ((!find_application(fd, UPS_USAGE)) &&
+		(!find_application(fd, POWER_USAGE)))
 		fatalx("%s is not a UPS\n", device_path);
 
 	ioctl(fd, HIDIOCGNAME(sizeof(name)), name);
 	printf("Detected %s\n", name);
 	printf("on port %s\n", device_path);
 
+	ioctl(fd, HIDIOCINITREPORT, 0);
 }
 
 void upsdrv_cleanup(void)
