@@ -20,7 +20,7 @@
 
 */
 
-#define DRV_VERSION "0.02"
+#define DRV_VERSION "0.03"
 
 #define xDEBUG
 
@@ -31,7 +31,9 @@
 #endif
 
 #include "main.h"
-#include <math.h>
+#include "serial.h"
+
+#include <math.h>		/* for sqrt */
 #include <string.h>
 
 /*#define ENDCHAR	'&'*/
@@ -55,14 +57,14 @@ void upsdrv_initinfo(void)
 	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
 }
 
-static char *getpacket(int *we_know){
+static const char *getpacket(int *we_know){
   fd_set readfds;
   struct timeval tv;
   int bytes_per_packet=0;
   int ret;
-  static char *packet_id=NULL;
+  static const char *packet_id=NULL;
   static char buf[256];
-  char *s;
+  const char *s;
   ssize_t r;
 
   
@@ -122,7 +124,7 @@ static char *getpacket(int *we_know){
 	return NULL;
  }
  D(if (s != buf) printf("overlapping packet received\n");)
- if (strlen(s) < bytes_per_packet) {
+ if ((int) strlen(s) < bytes_per_packet) {
 		    D(printf("incomplete packet information\n");)
 		    return NULL; 
  }
@@ -138,7 +140,8 @@ static char *getpacket(int *we_know){
 void upsdrv_updateinfo(void)
 {
   static  float high_volt=-1, low_volt=999;
-  char	*buf=NULL,buf2[17];
+  const char	*buf=NULL;
+  char buf2[17];
   int i;
   static int bytes_per_packet=0;
 
@@ -242,11 +245,6 @@ void upsdrv_help(void)
 /* list flags and values that you want to receive via -x */
 void upsdrv_makevartable(void)
 {
-	/* allow '-x xyzzy' */
-	/* addvar(VAR_FLAG, "xyzzy", "Enable xyzzy mode"); */
-
-	/* allow '-x foo=<some value>' */
-	/* addvar(VAR_VALUE, "foo", "Override foo setting"); */
 }
 
 void upsdrv_banner(void)
@@ -257,11 +255,11 @@ void upsdrv_banner(void)
 
 void upsdrv_initups(void)
 {
-	 upssend_delay = 100; 
-	 open_serial(device_path, B9600);
-         flag_timeoutfailure = -1;
+	upsfd = ser_open(device_path);
+	ser_set_speed(upsfd, device_path, B2400);
 }
 
 void upsdrv_cleanup(void)
 {
+	ser_close(upsfd, device_path);
 }
