@@ -20,45 +20,47 @@
 #include "main.h"
 #include "dstate.h"
 
-	unsigned int sddelay = 90;	/* wait 90 seconds for shutdown */
-	const char *progname = NULL, *upsname = NULL;
-	const char *device_name = NULL, *device_path = NULL;
-	int	do_forceshutdown = 0;
-	vartab_t	*vartab_h = NULL;
+	/* data which may be useful to the drivers */	
+	int	upsfd = -1;
+	char	*device_path = NULL;
+	const	char	*progname = NULL, *upsname = NULL, 
+			*device_name = NULL;
+
+	/* may be set by the driver to wake up while in dstate_poll_fds */
+	int	extrafd = -1;
+
+	/* for ser_open */
+	int	do_lock_port = 1;
+
+	/* set by the drivers */
 	int	experimental_driver = 0;
 	int	broken_driver = 0;
 
-	/* stuff from upscommon */
-	extern	int	do_lock_port;
-	extern	char	*pidfn;
-
-	/* stuff from common */
-	extern	int	upslog_flags;
-
 	/* for detecting -a values that don't match anything */
-	int	upsname_found = 0;
+	static	int	upsname_found = 0;
+
+	static vartab_t	*vartab_h = NULL;
 
 	/* variables possibly set by the global part of ups.conf */
 	static unsigned int	poll_interval = 2;
+	static char		*chroot_path = NULL, *user = NULL;
 
-	/* set by upscommon's signal handlers */
+	/* signal handling */
 	int	exit_flag = 0;
+	static	sigset_t		main_sigmask;
+	static	struct	sigaction	main_sa;
+
+	/* everything else */
+	static	char	*pidfn = NULL;
 
 /* power down the attached load immediately */
 static void forceshutdown(void)
 {
 	upslogx(LOG_NOTICE, "Initiating UPS shutdown");
-	printf("Initiating forced UPS shutdown!\n");
 
+	/* the driver must not block in this function */
 	upsdrv_shutdown();
-
-	if (sddelay > 0) {
-		printf("Waiting for poweroff...\n");
-		sleep(sddelay);
-		printf("Hmm, did the shutdown fail?  Oh well...\n");
-		exit(1);
-	}
-	exit(0);
+	exit(EXIT_SUCCESS);
 }
 
 static void help(void)
@@ -67,21 +69,18 @@ static void help(void)
 
 	printf("\nusage: %s [OPTIONS] [<device>]\n\n", progname);
 
-	printf(
-		"  -a <id>        - autoconfig using ups.conf section <id>\n"
-		"  -d <num>       - wait <num> seconds after sending shutdown command\n"
-		"                 - note: -x and -d after -a override ups.conf settings\n"
-		"  -D             - raise debugging level\n"
-		"  -h             - display this help\n"
-		"  -k             - force shutdown\n"
-		"  -r <dir>       - chroot to <dir>\n"
-		"  -u <user>      - switch to <user> (if started as root)\n"
-		"  -x <var>=<val> - set driver variable <var> to <val>\n"
-		"                 - example: -x cable=940-0095B\n"
-		"\n"
-		"  <device>       - /dev entry corresponding to UPS port\n"
-		"                 - Only optional when using -a!\n"
-		"\n");
+	printf("  -a <id>        - autoconfig using ups.conf section <id>\n");
+	printf("                 - note: -x after -a overrides ups.conf settings\n");
+	printf("  -D             - raise debugging level\n");
+	printf("  -h             - display this help\n");
+	printf("  -k             - force shutdown\n");
+	printf("  -r <dir>       - chroot to <dir>\n");
+	printf("  -u <user>      - switch to <user> (if started as root)\n");
+	printf("  -x <var>=<val> - set driver variable <var> to <val>\n");
+	printf("                 - example: -x cable=940-0095B\n\n");
+
+	printf("  <device>       - /dev entry corresponding to UPS port\n");
+	printf("                 - Only optional when using -a!\n\n");
 
 	if (vartab_h) {
 		tmp = vartab_h;
@@ -91,7 +90,7 @@ static void help(void)
 		while (tmp) {
 			if (tmp->vartype == VAR_VALUE)
 				printf("%40s : -x %s=<value>\n", 
-				       tmp->desc, tmp->var);
+					tmp->desc, tmp->var);
 			else
 				printf("%40s : -x %s\n", tmp->desc, tmp->var);
 			tmp = tmp->next;
@@ -100,7 +99,7 @@ static void help(void)
 
 	upsdrv_help();
 
-	exit(1);
+	exit(EXIT_SUCCESS);
 }
 
 /* store these in dstate as driver.(parameter|flag) */
@@ -163,7 +162,7 @@ static void storeval(const char *var, char *val)
 	printf("Look in the man page or call this driver with -h for a list of\n");
 	printf("valid variable names and flags.\n");
 
-	exit(1);
+	exit(EXIT_SUCCESS);
 }
 
 /* retrieve the value of variable <var> if possible */
@@ -236,7 +235,7 @@ static int main_arg(char *var, char *val)
 	if (!val)
 		return 0;
 
-	/* variables for main: port and sddelay */
+	/* variables for main: port */
 
 	if (!strcmp(var, "port")) {
 		device_path = xstrdup(val);
@@ -246,8 +245,7 @@ static int main_arg(char *var, char *val)
 	}
 
 	if (!strcmp(var, "sddelay")) {
-		sddelay = atoi(val);
-		dstate_setinfo("driver.parameter.sddelay", "%s", val);
+		upslogx(LOG_INFO, "Obsolete value sddelay found in ups.conf");
 		return 1;	/* handled */
 	}
 
@@ -257,7 +255,7 @@ static int main_arg(char *var, char *val)
 
 	/* only for upsd (at the moment) - ignored here */
 	if (!strcmp(var, "desc"))
-		return 1;       /* handled */
+		return 1;	/* handled */
 
 	return 0;	/* unhandled, pass it through to the driver */
 }
@@ -269,6 +267,21 @@ static void do_global_args(const char *var, const char *val)
 		return;
 	}
 
+	if (!strcmp(var, "chroot")) {
+		if (chroot_path)
+			free(chroot_path);
+
+		chroot_path = xstrdup(val);
+	}
+
+	if (!strcmp(var, "user")) {
+		if (user)
+			free(user);
+
+		user = xstrdup(val);
+	}
+
+
 	/* unrecognized */
 }
 
@@ -276,7 +289,7 @@ void do_upsconf_args(char *confupsname, char *var, char *val)
 {
 	char	tmp[SMALLBUF];
 
-	/* ignore global stuff in here */
+	/* handle global declarations */
 	if (!confupsname) {
 		do_global_args(var, val);
 		return;
@@ -310,6 +323,12 @@ void do_upsconf_args(char *confupsname, char *var, char *val)
 		return;
 	}
 
+	/* allow per-driver overrides of the global setting */
+	if (!strcmp(var, "pollinterval")) {
+		poll_interval = atoi(val);
+		return;
+	}
+
 	/* everything else must be for the driver */
 
 	storeval(var, val);
@@ -341,7 +360,7 @@ static void splitxarg(char *inbuf)
 }
 
 /* dump the list from the vartable for external parsers */
-void listxarg(void)
+static void listxarg(void)
 {
 	vartab_t	*tmp;
 
@@ -364,14 +383,74 @@ void listxarg(void)
 	}
 }
 
+static void vartab_free(void)
+{
+	vartab_t	*tmp, *next;
+
+	tmp = vartab_h;
+
+	while (tmp) {
+		next = tmp->next;
+
+		if (tmp->var)
+			free(tmp->var);
+		if (tmp->val)
+			free(tmp->val);
+		if (tmp->desc)
+			free(tmp->desc);
+		free(tmp);
+
+		tmp = next;
+	}
+}
+
+static void exit_cleanup(void)
+{
+	upsdrv_cleanup();
+
+	if (chroot_path)
+		free(chroot_path);
+	if (device_path)
+		free(device_path);
+	if (user)
+		free(user);
+	if (pidfn) {
+		unlink(pidfn);
+		free(pidfn);
+	}
+
+	dstate_free();
+	vartab_free();
+}
+
+static void set_exit_flag(int sig)
+{
+	exit_flag = sig;
+}
+
+static void setup_signals(void)
+{
+	sigemptyset(&main_sigmask);
+	main_sa.sa_mask = main_sigmask;
+	main_sa.sa_flags = 0;
+
+	main_sa.sa_handler = set_exit_flag;
+	sigaction(SIGTERM, &main_sa, NULL);
+	sigaction(SIGINT, &main_sa, NULL);
+	sigaction(SIGQUIT, &main_sa, NULL);
+
+	main_sa.sa_handler = SIG_IGN;
+	sigaction(SIGHUP, &main_sa, NULL);
+	sigaction(SIGPIPE, &main_sa, NULL);
+}
+
 int main(int argc, char **argv)
 {
-	char	*chroot_path = NULL, *user = NULL;
 	struct	passwd	*new_uid = NULL;
-	int	i;
+	int	i, do_forceshutdown = 0;
 
 	/* pick up a default from configure --with-user */
-	user = RUN_AS_USER;
+	user = xstrdup(RUN_AS_USER);	/* xstrdup: this gets freed at exit */
 
 	upsdrv_banner();
 
@@ -391,15 +470,12 @@ int main(int argc, char **argv)
 			case 'a':
 				upsname = optarg;
 
-				read_upsconf(1);
+				read_upsconf();
 
 				if (!upsname_found)
 					upslogx(LOG_WARNING, "Warning: Section %s not found in ups.conf",
 						optarg);
 				break;
-			case 'd':
-				sddelay = atoi(optarg);
-				break;
 			case 'D':
 				nut_debug_level++;
 				break;
@@ -412,16 +488,16 @@ int main(int argc, char **argv)
 				break;
 			case 'L':
 				listxarg();
-				exit(0);
+				exit(EXIT_SUCCESS);
 			case 'r':
-				chroot_path = optarg;
+				chroot_path = xstrdup(optarg);
 				break;
 			case 'u':
-				user = optarg;
+				user = xstrdup(optarg);
 				break;
 			case 'V':
 				/* already printed the banner, so exit */
-				exit(0);
+				exit(EXIT_SUCCESS);
 			case 'x':
 				splitxarg(optarg);
 				break;
@@ -446,7 +522,7 @@ int main(int argc, char **argv)
 
 	/* allow argv to override the ups.conf entry if specified */
 	else {
-		device_path = argv[0];
+		device_path = xstrdup(argv[0]);
 		device_name = xbasename(device_path);
 	}
 
@@ -463,7 +539,7 @@ int main(int argc, char **argv)
 	if (broken_driver) {
 		printf("Fatal error: broken driver.  It probably needs to be converted.\n");
 		printf("Search for 'broken_driver = 1' in the source for more details.\n");
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 
 	if (do_forceshutdown)
@@ -490,7 +566,7 @@ int main(int argc, char **argv)
 	upsdrv_updateinfo();
 
 	/* now we can start servicing requests */
-        dstate_init(progname, device_name);
+	dstate_init(progname, device_name);
 
 	/* publish the top-level data: version number, driver name */
 	dstate_setinfo("driver.version", "%s", UPS_VERSION);
@@ -504,19 +580,15 @@ int main(int argc, char **argv)
 	/* safe to do this now that the parent has exited */
 	atexit(exit_cleanup);
 
-	for (;;) {
+	while (exit_flag == 0) {
 		upsdrv_updateinfo();
 
-		/* FUTURE: make this delay configurable */
-		dstate_poll_fds(poll_interval);
-
-		/* deal with flags from the signal handlers */
-		if (exit_flag == 1) {
-			upsdrv_cleanup();
-			exit(0);
-		}
+		dstate_poll_fds(poll_interval, extrafd);
 	}
 
-	/* NOTREACHED */
-	return 0;
+	/* if we get here, the exit flag was set by a signal handler */
+
+	upslogx(LOG_INFO, "Signal %d: exiting", exit_flag);
+
+	exit(EXIT_SUCCESS);
 }
