@@ -24,10 +24,12 @@
  */
 
 #include <sys/ioctl.h>
+#include <sys/termios.h>
 #include <string.h>
 
 #include "config.h"
 #include "main.h"
+#include "serial.h"
 #include "timehead.h"
 #include "mge-shut.h"
 
@@ -37,7 +39,7 @@
 
 #define DRIVER_NAME    "MGE UPS SYSTEMS/SHUT driver"
 
-#define MAX_TRY 3
+#define MAX_TRY 4
 
 /* global variables */
 int commstatus = 0;
@@ -46,8 +48,6 @@ int ondelay = DEFAULT_ONDELAY;
 int offdelay = DEFAULT_OFFDELAY;
 int notification = DEFAULT_NOTIFICATION;
 
-extern int  sddelay;         /* from main.c --- ignored for now  */
-
 #define SD_RETURN	0
 #define SD_STAYOFF	1
 
@@ -56,10 +56,10 @@ int sdtype = SD_RETURN;
 #define BYTESWAP(in) (((in & 0xFF) << 8) + ((in & 0xFF00) >> 8))
 
 /* realign packet data according to Endianess */
-void align_request(hid_packet_t *sd)
+static void align_request(hid_packet_t *sd)
 {
 #if WORDS_BIGENDIAN
-        /* Sparc/Mips/... are big endian, USB little endian */
+        /* Sparc/Mips/... are big endian, USB/SHUT little endian */
 	(*sd).wValue    = BYTESWAP((*sd).wValue);
 	(*sd).wIndex    = BYTESWAP((*sd).wIndex);
 	(*sd).wLength   = BYTESWAP((*sd).wLength);
@@ -135,8 +135,8 @@ void upsdrv_initinfo (void)
 	dstate_addcmd("test.battery.stop");
 
 	/* install handlers */
-	upsh.new_setvar = hid_set_value; /* setvar; */
-	upsh.new_instcmd = instcmd;
+	upsh.setvar = hid_set_value; /* setvar; */
+	upsh.instcmd = instcmd;
 	
 }
 
@@ -204,9 +204,6 @@ void upsdrv_shutdown (void)
 void upsdrv_help (void)
 {
 	upsdebugx(2, "entering upsdrv_help");
-	printf("\n");
-	printf("Option -d is ignored.\n");
-	printf("\n");
 }
 
 /* --------------------------------------------------------------- */
@@ -250,7 +247,8 @@ void upsdrv_initups (void)
 	upsdebugx(2, "entering upsdrv_initups()");
 	
 	/* initialize serial port */
-	open_serial (device_path, B2400);
+	upsfd = ser_open(device_path);
+	ser_set_speed(upsfd, device_path, B2400);
 	setline (1);
   
 	/* get battery lowlevel */
@@ -284,6 +282,7 @@ void upsdrv_initups (void)
 
 void upsdrv_cleanup(void)
 {
+	ser_close(upsfd, device_path);
 }
 
 /* --------------------------------------------------------------- */
@@ -475,7 +474,7 @@ int shut_wait_ack (void)
  * return -1 on error, -2 on timeout, nb_bytes_readen on success
  * 
  *********************************************************************/
-int char_read (char *bytes, int size, int read_timeout)
+static int char_read (char *bytes, int size, int read_timeout)
 {
 	struct timeval serial_timeout;
 	fd_set readfs;
@@ -519,17 +518,17 @@ int char_read (char *bytes, int size, int read_timeout)
  * returns 0 on success, -1 on error, -2 on timeout
  * 
  **********************************************************************/
-int serial_read (int read_timeout, unsigned char *read)
+int serial_read (int read_timeout, unsigned char *readbuf)
 {
 	static unsigned char cache[512];
 	static unsigned char *cachep = cache;
 	static unsigned char *cachee = cache;
 	int recv;
-	*read = '\0';
+	*readbuf = '\0';
 	
 	/* if still data in cache, get it */
 	if (cachep < cachee) { 
-		*read = *cachep++;
+		*readbuf = *cachep++;
 		return 0;
 		/* return (int) *cachep++; */
 	}
@@ -545,7 +544,7 @@ int serial_read (int read_timeout, unsigned char *read)
 	
 	if (recv) {
 		upsdebugx(5,"received: %02x", *cachep); 
-		*read = *cachep++;
+		*readbuf = *cachep++;
 		return 0;
 	}
 	return -1;
@@ -575,29 +574,28 @@ int serial_send (unsigned char *buf, int len)
 /* Get and parse UPS status */
 void  shut_ups_status(void)
 {
-	int try = 0, retcode = 0;
+  int try = 0, retcode = 0;
 
 	/* clear status buffer before begining */
 	status_init();
 	
 	/* Ensure to have at least basic status */
-	while (try < MAX_TRY) {
-		if((retcode =hid_get_value("UPS.PowerSummary.PresentStatus.ACPresent")) != 0 ) {
-			try = MAX_TRY;
-			if(hData.Value == 1) {
-				status_set("OL");
-			}
-			else {
-				status_set("OB");
-			}
-		} else { /* retry to get data */
-			try++;
-		}
+        while (try < MAX_TRY) {
+	  if((retcode = hid_get_value("UPS.PowerSummary.PresentStatus.ACPresent")) != 0 ) {
+	    try = MAX_TRY;
+	    if(hData.Value == 1){
+	      status_set("OL");
+	    } else {
+	      status_set("OB");
+	    }
+	  } else { /* retry to get data */
+	    try++;
+	  }
 	}
 	/* If we didn't succeed in getting status, set it to OB (FIXME: do a global parse?!) */
 	if (try == MAX_TRY && retcode == 0)
-		status_set("OB");
-  
+	  status_set("OB");
+
 	if(hid_get_value("UPS.PowerSummary.PresentStatus.Discharging") != 0 ) {
 		if(hData.Value == 1)
 			status_set("DISCHRG");
@@ -840,20 +838,20 @@ int shut_get_descriptor(int desctype, unsigned char *pkt, int reportlen)
  * return string size on success, -1 on failure, -2 on NACK
  *
  *********************************************************************/
-int shut_get_string(int index, char *string, int stringlen)
+int shut_get_string(int strindex, char *string, int stringlen)
 {
 	hid_packet_t HIDRequest;
 	hid_data_u   data;
 	int retcode;
 	unsigned char buf[MAX_STRING];
 	
-	upsdebugx (3, "entering shut_get_string(%02x)", index);
+	upsdebugx (3, "entering shut_get_string(%02x)", strindex);
 	
 	HIDRequest.bmRequestType = REQUEST_TYPE_USB;
 	HIDRequest.bRequest = 0x06;
-	HIDRequest.wValue = index+(STRING_DESCRIPTOR<<8);
+	HIDRequest.wValue = strindex+(STRING_DESCRIPTOR<<8);
 	HIDRequest.wIndex = 0x0000;
-	HIDRequest.wLength = (stringlen<<8); //(reportlen&0xFF)&(reportlen>>8);
+	HIDRequest.wLength = (stringlen<<8); /* (reportlen&0xFF)&(reportlen>>8); */
 	
 	align_request(&HIDRequest);
 	
@@ -1051,7 +1049,7 @@ int hid_init_device()
 }
 
 /* translate HID string path to numeric path and return path depth */
-ushort lookup_path(char *HIDpath, HIDData *data)
+ushort lookup_path(const char *HIDpath, HIDData *data)
 {
 	ushort i = 0, cond = 1;
 	int cur_usage;
@@ -1116,7 +1114,7 @@ int hid_lookup_usage(char *name)
 }
 
 /* Get an item value from a HID path */
-int hid_get_value(char *item_path)
+int hid_get_value(const char *item_path)
 {
 	int i, retcode;
 	
@@ -1329,7 +1327,7 @@ int hid_set_value(const char *varname, const char *val)
 	}
 
 	/* Checking item writability and HID Path */
-	if (!shut_info_p->flags & FLAG_RW) {
+	if (!shut_info_p->flags & ST_FLAG_RW) {
 		upsdebugx(2, "hid_ups_set: not writable %s", varname);
 		return STAT_SET_UNKNOWN;
 	}
