@@ -2,7 +2,7 @@
 
    Driver for MGE UPS SYSTEMS units with serial UTalk protocol
 
-   This driver is a collaborative effort by (Copyright (C) 2002 - 2003):
+   This driver is a collaborative effort by (Copyright (C) 2002 - 2004):
 
      Hans Ekkehard Plesser <hans.plesser@itf.nlh.no>
      Arnaud Quette <arnaud.quette@mgeups.com>
@@ -29,6 +29,7 @@
 #include <ctype.h>
 #include <sys/ioctl.h>
 #include "main.h"
+#include "serial.h"
 #include "mge-utalk.h"
 
 /* --------------------------------------------------------------- */
@@ -36,7 +37,7 @@
 /* --------------------------------------------------------------- */
 
 #define DRIVER_NAME    "MGE UPS SYSTEMS/U-Talk driver"
-#define DRIVER_VERSION "0.81.0"
+#define DRIVER_VERSION "0.80.3"
 
 /* delay after sending each char to UPS (in MICROSECONDS) */
 #define MGE_CHAR_DELAY 0
@@ -126,9 +127,9 @@ void upsdrv_initups(void)
 {
 	char buf[BUFFLEN];
 	int RTS = TIOCM_RTS;
-	int  bytes_rcvd;
 	
-	open_serial(device_path, B2400);
+	upsfd = ser_open(device_path);
+	ser_set_speed(upsfd, device_path, B2400);
 
 	/* read command line/conf variables */
 	if (getval ("LowBatt"))
@@ -142,39 +143,26 @@ void upsdrv_initups(void)
 
 	if (testvar ("oldmac"))
 		RTS = ~TIOCM_RTS;
-
+	
 	/* Init serial line */
 	ioctl(upsfd, TIOCMBIC, &RTS);
 	enable_ups_comm();
-	
-	/* Get and set values (if exists) given on the command line */
-	bytes_rcvd = mge_command(buf, sizeof(buf), "Bl ?");
-	if(bytes_rcvd > 0 && buf[0] != '?') {
-		mge_command(buf, sizeof(buf), "Bl %d",  mge_ups.LowBatt);
-		if(strcmp(buf, "OK"))
-			upsdebugx(1, "UPS response to %d%% Low Batt Level was %s",
-				mge_ups.LowBatt, buf);
-	} else
-		upsdebugx(1, "initups: LowBatt unavailable");
-	
-	bytes_rcvd = mge_command(buf, sizeof(buf), "Sm ?");
-	if(bytes_rcvd > 0 && buf[0] != '?') {
-		mge_command(buf, sizeof(buf), "Sm %d",  mge_ups.OnDelay);
-		if(strcmp(buf, "OK"))
-			upsdebugx(1, "UPS response to %d min ON delay was %s",
-				mge_ups.OnDelay, buf);
-	} else
-		upsdebugx(1, "initups: OnDelay unavailable");
-	
-	bytes_rcvd = mge_command(buf, sizeof(buf), "Sn ?");
-	if(bytes_rcvd > 0 && buf[0] != '?') {
-		mge_command(buf, sizeof(buf), "Sn %d",  mge_ups.OffDelay);
-		if(strcmp(buf, "OK"))
-			upsdebugx(1, "UPS response to %d min OFF delay was %s",
-				mge_ups.OffDelay, buf);
-	} else
-		upsdebugx(1, "initups: OffDelay unavailable");
 
+	/* set values given on the command line */
+	mge_command(buf, sizeof(buf), "Bl %d",  mge_ups.LowBatt);
+	if(strcmp(buf, "OK"))
+		upsdebugx(1, "UPS response to %d%% Low Batt Level was %s",
+			mge_ups.LowBatt, buf);
+	
+	mge_command(buf, sizeof(buf), "Sm %d",  mge_ups.OnDelay);
+	if(strcmp(buf, "OK"))
+		upsdebugx(1, "UPS response to %d min ON delay was %s",
+			mge_ups.OnDelay, buf);
+	
+	mge_command(buf, sizeof(buf), "Sn %d",  mge_ups.OffDelay);
+	if(strcmp(buf, "OK"))
+		upsdebugx(1, "UPS response to %d min OFF delay was %s",
+			mge_ups.OffDelay, buf);
 }
 
 /* --------------------------------------------------------------- */
@@ -269,17 +257,11 @@ void upsdrv_initinfo(void)
 			item->ok = TRUE;
 			extract_info(buf, item, infostr, sizeof(infostr));
 			dstate_setinfo(item->type, "%s", infostr);
-			dstate_setflags(item->type, item->flags);
 			upsdebugx(1, "initinfo: %s == >%s<", item->type, infostr);
-
-			/* Set max length for strings */
-			if (item->flags & ST_FLAG_STRING)
-				dstate_setaux(item->type, item->length);
 		}
 	} /* for item */
 
 	/* commands ----------------------------------------------- */
-	/* FIXME: check if available before adding! */
 	dstate_addcmd("load.off");
 	dstate_addcmd("load.on");
 	dstate_addcmd("shutdown.return");
@@ -290,8 +272,8 @@ void upsdrv_initinfo(void)
 	dstate_addcmd("bypass.stop");
 
 	/* install handlers */
-	upsh.new_setvar = setvar;
-	upsh.new_instcmd = instcmd;
+	upsh.setvar = setvar;
+	upsh.instcmd = instcmd;
 }
 
 /* --------------------------------------------------------------- */
@@ -494,7 +476,7 @@ int setvar(const char *varname, const char *val)
     
 		/* Execute command */
 		mge_command(temp, sizeof(temp), cmd);
-		upslogx(LOG_INFO, "setvar: UPS response to Set %s to %s was %s", varname, val, temp);
+		upsdebugx(1, "setvar: UPS response to Set %s to %s was %s", varname, val, temp);
 	} else
 		upsdebugx(1, "setvar: Variable %s not supported by UPS", varname);
 	
@@ -509,7 +491,7 @@ static void enable_ups_comm(void)
 	mge_command(NULL, 0, "Z");   
 	mge_command(NULL, 0, "Ax 1");   
 	usleep(MGE_CONNECT_DELAY);
-	upsflushin(0, nut_debug_level, "?\r\n");  /* ignore ? as well " */
+	ser_flush_in(upsfd, "?\r\n", nut_debug_level);
 }
 
 /* --------------------------------------------------------------- */
@@ -630,7 +612,6 @@ static int get_ups_status(void)
 				status_set("LB");
 			}
 
-				
 			if ( !(buf[9] == '1') )
 				status_set("OFF");
 		} /* if strlen */  
@@ -745,12 +726,13 @@ static int mge_command(char *reply, int replylen, const char *fmt, ...)
 	else
 		usleep(MGE_REPLY_DELAY);
 
-	bytes_rcvd = upsrecv(reply, replylen, 
-		MGE_REPLY_ENDCHAR, MGE_REPLY_IGNCHAR);
+	bytes_rcvd = ser_get_line(upsfd, reply, replylen,
+		MGE_REPLY_ENDCHAR, MGE_REPLY_IGNCHAR, 3, 0);
 	
 	return bytes_rcvd;
 }
 
 void upsdrv_cleanup(void)
 {
+	ser_close(upsfd, device_path);
 }
