@@ -1,22 +1,11 @@
 /*
  * powercom.h - defines for the newpowercom.c driver
  *
- * Read docs/powercom.txt for other models and manufactures
- *
  * $Id: - will be filled in on next CVS add/update $
  *
  * Copyrights:
  * (C) 2002 Simon Rozman <simon@rozman.net>
- *  Added support for Egys
- *
- * (C) 2001 Peter Bieringer <pb@bieringer.de>
- *  Porting old style "powercom" to new style "newpowercom", cleanup header file
- *
- * (C) 2000  Shaul Karl <shaulk@israsrv.net.il>
- *  Creating old style "powercom"
- *   Heavily based on
- *    ups-trust425+625.c - model specific routines for Trust UPS 425/625
- *    Copyright (C) 1999  Peter Bieringer <pb@bieringer.de>
+ * (C) 1999  Peter Bieringer <pb@bieringer.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,12 +21,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  *
- * Changes
- * 20011208/PB: add support for subtypes
- * 20020629/SR: add support for calc. parameters
- * 20020701/SK: subtypes as structures
- * 20030506/SK: replaced the previous validation method with
- *              validation_byte[] and replaced subtype by type.
  */
 
 /* C-libary includes */
@@ -50,19 +33,17 @@
 
 /* nut includes */
 #include "timehead.h"
-#include "shared-tables.h"
 
 /* general constants */
 enum general {
-	SEC_FOR_POWERKILL         = 10U,
-	MAX_NUM_OF_BYTES_FROM_UPS = 16U,
+	MAX_NUM_OF_BYTES_FROM_UPS = 16U
 };
 
 /* values for sending to UPS */
 enum commands {
 	SEND_DATA    = '\x01',
 	BATTERY_TEST = '\x03',
-	SHUTDOWN     = '\xbc',
+	SHUTDOWN     = '\xbc'
 };
 
 /* location of data in received string */
@@ -72,7 +53,7 @@ enum data {
 	INPUT_VOLTAGE    = 2U,
 	OUTPUT_VOLTAGE   = 3U,
 	INPUT_FREQUENCY  = 4U,
-	OUTPUT_FREQUENCY = 6U,
+	OUTPUT_FREQUENCY = 6U
 };
 
 /* status bits */
@@ -86,30 +67,46 @@ enum status {
 	OVERLOAD      = 32U,
 	OFF           = 128U,
 	STATUS_A      = 9U,
-	STATUS_B      = 10U,
+	STATUS_B      = 10U
 };
 
 /* supported types */
 struct type {
-	char          *name;
+	const char    *name;
 	unsigned char num_of_bytes_from_ups;
-	void          (*setup_flow_control)(void);
+	
+	struct method_of_flow_control {
+	    char *name;
+	    void (*setup_flow_control)(void);
+	} flowControl;
 	
 	struct validation_byte {
-	    unsigned char index_of_byte, required_value;
+	    unsigned int index_of_byte, required_value;
 	    /* An example might explain the intention better then prose.
 	     * Suppose we want to validate the data with:
 	     *     powercom_raw_data[5] == 0x80
-	     * then we will set index_of_byte to 5 and required_value to
-	     * 0x80: { 5, 0x80 }.
+	     * then we will set index_of_byte to 5U and required_value to
+	     * 0x80U: { 5U, 0x80U }.
 	     */
 	} validation[3];
 	/* The validation array is of length 3 because 3 is longest 
 	 * validation sequence for any type.
 	 */
 	
-	/* parameters to calculate input and output freq., one pair for each
-	 * type:
+	/* Some UPSs must have a minutes and a seconds arguments for 
+	 * the SHUTDOWN commands while others are known to work with the
+	 * seconds argument alone.
+	 */
+	struct deley_for_power_kill {
+	    unsigned int  delay[2];   /* { minutes, seconds } */
+	    unsigned char minutesShouldBeUsed;
+	    /* 'n' in case the minutes value, which is deley[0], should
+		 * be skipped and not sent to the UPS. 
+		 */
+	} shutdown_arguments;
+	
+	/* parameters to calculate input and output freq., one pair for
+	 * each type:
 	 *  Each pair defines parameters for 1/(A*x+B) to calculate freq.
 	 *  from raw data
 	 */
