@@ -47,6 +47,7 @@
 #include <sys/ioctl.h>
 
 #include "main.h"
+#include "serial.h"
 #include "safenet.h"
 
 #define DRV_VERSION	"0.03"
@@ -73,8 +74,8 @@ static int safenet_command(char *command)
 	 * read back the status line. Unless we just send a shutdown command,
 	 * it will return the actual status.
 	 */
-	upssend(command);
-	upsrecv(reply, sizeof(reply), ENDCHAR, IGNCHARS);
+	ser_send_pace(upsfd, UPSDELAY, command);
+	ser_get_line(upsfd, reply, sizeof(reply), ENDCHAR, IGNCHARS, 1, 0);
 
 	upsdebugx(3, "S : %s", ((strlen(reply) > 0) ? reply : "[empty]"));
 
@@ -85,13 +86,12 @@ static int safenet_command(char *command)
 	 */
 	if ((strlen(reply) != 11) || (reply[0] != '$') || (strspn(reply+1, "AB") != 10))
 	{
-		if (retry == 2)
+		if (retry > 2)
 		{
-			upslogx(LOG_INFO, "Status read failed");
+			ser_comm_fail("Status read failed");
 			dstate_datastale();
 		}
-
-		if (retry < 3)
+		else
 		{
 			retry++;
 		}
@@ -99,6 +99,11 @@ static int safenet_command(char *command)
 		return(0);
 	}
 
+	if (retry > 2)
+	{
+		ser_comm_good();
+	}
+
 	retry = 0;
 
 	for (i=0; i<10; i++)
@@ -210,7 +215,7 @@ static void shutdown_reboot(int delay, int restart)
 	/*
 	 * This is ugly! This UPS has a strange mapping of numerals (A=0, B=1,..,J=9)
 	 * so we do this here. Anyone for an easier way?
-	 */
+	 */ 
 	snprintf(command, sizeof(command), "ZAF%c%c%cR%c%c%c%cO\r", ((delay / 100) + 'A'),
 		(((delay % 100) / 10) + 'A'), ((delay % 10) + 'A'),
 		((restart / 1000) + 'A'), (((restart % 1000) / 100) + 'A'),
@@ -351,7 +356,7 @@ void upsdrv_initinfo(void)
 	dstate_addcmd ("shutdown.reboot");
 	dstate_addcmd ("shutdown.reboot.graceful");
 
-	upsh.new_instcmd = instcmd;
+	upsh.instcmd = instcmd;
 }
 
 /*
@@ -451,7 +456,8 @@ void upsdrv_initups(void)
 	/*
 	 * Open and lock the serial port and set the speed to 1200 baud.
 	 */
-	open_serial(device_path, B1200);
+	upsfd = ser_open(device_path);
+	ser_set_speed(upsfd, device_path, B1200);
 
 	/*
 	 * Set DTR and clear RTS to provide power for the serial interface.
@@ -462,4 +468,5 @@ void upsdrv_initups(void)
 
 void upsdrv_cleanup(void)
 {
+	ser_close(upsfd, device_path);
 }
