@@ -58,8 +58,11 @@ void upsdrv_initinfo(void)
 			dstate_addcmd(su_info_p->info_type);
 
 	/* setup handlers for instcmd and setvar functions */
-	upsh.new_setvar = su_setvar;
-	upsh.new_instcmd = su_instcmd;
+	upsh.setvar = su_setvar;
+	upsh.instcmd = su_instcmd;
+
+	if (testvar("notransferoids"))
+		disable_transfer_oids();
 
 	/* initialize all other INFO_ fields from list */
 	if (snmp_ups_walk(SU_WALKMODE_INIT))
@@ -111,6 +114,7 @@ void upsdrv_makevartable(void)
   addvar(VAR_VALUE | VAR_SENSITIVE, SU_VAR_COMMUNITY, "Set community name (default=public)");
   addvar(VAR_VALUE, SU_VAR_VERSION, "Set SNMP version (default=v1, allowed v2c)");
   addvar(VAR_VALUE, SU_VAR_POLLFREQ, "Set polling frequency in seconds, to reduce network flow (default=30)");
+  addvar(VAR_FLAG, "notransferoids", "Disable transfer OIDs (use on APCC Symmetras)");
 }
 
 void upsdrv_banner(void)
@@ -184,7 +188,7 @@ void nut_snmp_init(const char *type, const char *hostname, const char *community
 	g_snmp_sess_p = snmp_open(&g_snmp_sess);	/* establish the session */
 	if (g_snmp_sess_p == NULL) {
 		nut_snmp_perror(&g_snmp_sess, 0, NULL, "nut_snmp_init: snmp_open");
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 }
 
@@ -200,7 +204,7 @@ void nut_snmp_cleanup(void)
 
 static int oid_compare(const char *oid_req_in, long *oid_ans, size_t oid_len)
 {
-	int	ctr;
+	unsigned int	ctr;
 	long	val;
 	char	*tmp, *ptr, *oid_req;
 
@@ -493,6 +497,26 @@ void nut_snmp_perror(struct snmp_session *sess, int status,
  * utility functions.
  * ----------------------------------------------------------- */
 
+/* deal with APCC weirdness on Symmetras */
+static void disable_transfer_oids(void)
+{
+	snmp_info_t *su_info_p;
+
+	upslogx(LOG_INFO, "Disabling transfer OIDs");
+
+	for (su_info_p = &snmp_info[0]; su_info_p->info_type != NULL ; su_info_p++) {
+		if (!strcasecmp(su_info_p->info_type, "input.transfer.low")) {
+			su_info_p->flags &= ~SU_FLAG_OK;
+			continue;
+		}
+
+		if (!strcasecmp(su_info_p->info_type, "input.transfer.high")) {
+			su_info_p->flags &= ~SU_FLAG_OK;
+			continue;
+		}
+	}
+}
+
 /* universal function to add or update info element. */
 void su_setinfo(const char *type, const char *value, int flags, int auxdata)
 {
@@ -755,7 +779,7 @@ int su_setvar(const char *varname, const char *val)
 		return STAT_SET_UNKNOWN;
 	}
 
-	if (!(su_info_p->info_flags & FLAG_RW) || su_info_p->OID == NULL) {
+	if (!(su_info_p->info_flags & ST_FLAG_RW) || su_info_p->OID == NULL) {
 		upslogx(LOG_ERR, "su_setvar: not writable %s", varname);
 		return STAT_SET_UNKNOWN; /* STAT_SET_UNHANDLED would be better */
 	}
@@ -797,7 +821,7 @@ int su_instcmd(const char *cmdname, const char *extradata)
 	}
 
 	/* set value. */
-	if (su_info_p->info_flags & FLAG_STRING) {
+	if (su_info_p->info_flags & ST_FLAG_STRING) {
 		status = nut_snmp_set_str(su_info_p->OID, su_info_p->dfl);
 	} else {
 		status = nut_snmp_set_int(su_info_p->OID, su_info_p->info_len);
@@ -894,7 +918,7 @@ static int parse_mibconf_args(int numargs, char **arg)
 	return 1;
 }
 /* called for fatal errors in parseconf like malloc failures */
-void mibconf_err(const char *errmsg)
+static void mibconf_err(const char *errmsg)
 {
 	upslogx(LOG_ERR, "Fatal error in parseconf (*mib.conf): %s", errmsg);
 }
@@ -924,7 +948,7 @@ void read_mibconf(char *mib)
 			continue;
 
 		if (!parse_mibconf_args(ctx.numargs, ctx.arglist)) {
-			int	i;
+			unsigned int	i;
 			char	errmsg[SMALLBUF];
 
 			snprintf(errmsg, sizeof(errmsg), 
@@ -934,7 +958,7 @@ void read_mibconf(char *mib)
 				snprintfcat(errmsg, sizeof(errmsg), " %s", 
 					ctx.arglist[i]);
 
-			upslogx(LOG_WARNING, "%s", errmsg);
+			upslogx(LOG_WARNING, errmsg);
 		}
 	}
 	pconf_finish(&ctx);		
