@@ -74,9 +74,10 @@
                     = 700
 */
 
-#define DRV_VERSION "0.02"
+#define DRV_VERSION "0.03"
 
 #include "main.h"
+#include "serial.h"
 #include <ctype.h>
 
 #define ENDCHAR  '\n'	/* replies end with CR LF -- use LF to end */
@@ -96,14 +97,14 @@ static int hex2d(char *start, int len)
     return strtol(buf, NULL, 16);
 }
 
-int instcmd(const char *cmdname, const char *extra)
+static int instcmd(const char *cmdname, const char *extra)
 {
 	char    buf[256];
 
 	if (!strcasecmp(cmdname, "test.battery.start")) {
-		upssend("%s", ":A\r"); /* Start self test */
+		ser_send(upsfd, "%s", ":A\r"); /* Start self test */
 		sleep(UPSDELAY);
-		upsrecv(buf, sizeof(buf), ENDCHAR, IGNCHARS);
+		ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, IGNCHARS, 3, 0);
 
 		return STAT_INSTCMD_HANDLED;
 	}
@@ -114,7 +115,7 @@ int instcmd(const char *cmdname, const char *extra)
 
 void upsdrv_initinfo(void)
 {
-    char *model;
+    const char *model;
     int  va;
     long w, l;
 
@@ -146,7 +147,7 @@ void upsdrv_initinfo(void)
     setinfo(INFO_REG2,       "%s", l_value);
 #endif
     
-    upsh.new_instcmd = instcmd;     /* Set here instead of upsdrv_initups */
+    upsh.instcmd = instcmd;     /* Set here instead of upsdrv_initups */
     
     printf("Detected %s %s on %s\n",
            dstate_getinfo("ups.mfr"), dstate_getinfo("ups.model"), device_path);
@@ -177,7 +178,7 @@ static void setup_serial(void)
     if (tcsetattr(upsfd, TCSANOW, &tio) == -1) fatal("tcsetattr");
 }
 
-void ups_sync(void)
+static void ups_sync(void)
 {
     char	buf[256];
     int	        tries = 0;
@@ -187,10 +188,10 @@ void ups_sync(void)
         tries++;
         if (tries > MAXTRIES) fatalx("\nFailed - giving up...");
         
-        upssend("%s", ":W\r");
+        ser_send(upsfd, "%s", ":W\r");
         sleep(UPSDELAY);
-        ret = upsrecv(buf, sizeof(buf), ENDCHAR, IGNCHARS);
-        ret = upsrecv(buf, sizeof(buf), ENDCHAR, IGNCHARS);
+	ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, IGNCHARS, 3, 0);
+	ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, IGNCHARS, 3, 0);
         
         if ((ret > 0) && isdigit((unsigned char) buf[0])) break;
     }
@@ -201,58 +202,58 @@ void upsdrv_shutdown(void)      /* FIXME: Not yet tested */
     char	buf[256];
     int	        ret;
     
-    upssend("%s", ":N40\r"); /* Delay 64 seconds */
+    ser_send(upsfd, "%s", ":N40\r"); /* Delay 64 seconds */
     sleep(UPSDELAY);
-    ret = upsrecv(buf, sizeof(buf), ENDCHAR, IGNCHARS);
+    ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, IGNCHARS, 3, 0);
     
-    upssend("%s", ":G\r");   /* Inverter off */
+    ser_send(upsfd, "%s", ":G\r");   /* Inverter off */
     sleep(UPSDELAY);
-    ret = upsrecv(buf, sizeof(buf), ENDCHAR, IGNCHARS);
+    ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, IGNCHARS, 3, 0);
 
 #if 1
     /* FIXME: This may prevent UPS from powering up when utility comes
      * back. */
-    upssend("%s", ":K0\r");  /* Relay off */
+    ser_send(upsfd, "%s", ":K0\r");  /* Relay off */
     sleep(UPSDELAY);
-    ret = upsrecv(buf, sizeof(buf), ENDCHAR, IGNCHARS);
+    ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, IGNCHARS, 3, 0);
 #endif
 }
 
-void ups_ident(void)
+static void ups_ident(void)
 {
     char	buf[256];
     int	        ret;
     
-    upssend("%s", ":W\r");
+    ser_send(upsfd, "%s", ":W\r");
     sleep(UPSDELAY);
-    ret = upsrecv(buf, sizeof(buf), ENDCHAR, IGNCHARS);
-    ret = upsrecv(buf, sizeof(buf), ENDCHAR, IGNCHARS);
+    ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, IGNCHARS, 3, 0);
+    ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, IGNCHARS, 3, 0);
     if (w_value) free(w_value);
     w_value = xstrdup(buf);
 
-    upssend("%s", ":L\r");
+    ser_send(upsfd, "%s", ":L\r");
     sleep(UPSDELAY);
-    ret = upsrecv(buf, sizeof(buf), ENDCHAR, IGNCHARS);
-    ret = upsrecv(buf, sizeof(buf), ENDCHAR, IGNCHARS);
+    ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, IGNCHARS, 3, 0);
+    ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, IGNCHARS, 3, 0);
     if (l_value) free(l_value);
     l_value = xstrdup(buf);
     
-    upssend("%s", ":V\r");
+    ser_send(upsfd, "%s", ":V\r");
     sleep(UPSDELAY);
-    ret = upsrecv(buf, sizeof(buf), ENDCHAR, IGNCHARS);
-    ret = upsrecv(buf, sizeof(buf), ENDCHAR, IGNCHARS);
+    ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, IGNCHARS, 3, 0);
+    ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, IGNCHARS, 3, 0);
     if (v_value) free(v_value);
     v_value = xstrdup(buf);
 
-    upssend("%s", ":X\r");
+    ser_send(upsfd, "%s", ":X\r");
     sleep(UPSDELAY);
-    ret = upsrecv(buf, sizeof(buf), ENDCHAR, IGNCHARS);
-    ret = upsrecv(buf, sizeof(buf), ENDCHAR, IGNCHARS);
+    ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, IGNCHARS, 3, 0);
+    ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, IGNCHARS, 3, 0);
     if (x_value) free(x_value);
     x_value = xstrdup(buf);
 }
 
-void pollfail(char *why)
+static void pollfail(const char *why)
 {
     poll_failures++;
 
@@ -269,10 +270,10 @@ void upsdrv_updateinfo(void)
     int     bp;
     float   bv;
     
-    upssend("%s", ":D\r");
+    ser_send(upsfd, "%s", ":D\r");
     sleep(UPSDELAY); 
-    ret = upsrecv(buf, sizeof(buf), ENDCHAR, IGNCHARS);
-    ret = upsrecv(buf, sizeof(buf), ENDCHAR, IGNCHARS);
+    ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, IGNCHARS, 3, 0);
+    ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, IGNCHARS, 3, 0);
     
     if (strlen(buf) < 21) {
         pollfail("Poll failed: short read from UPS");
@@ -360,10 +361,10 @@ void upsdrv_updateinfo(void)
 
     status_commit();
     
-    upssend("%s", ":B\r");
+    ser_send(upsfd, "%s", ":B\r");
     sleep(UPSDELAY); 
-    ret = upsrecv(buf, sizeof(buf), ENDCHAR, IGNCHARS);
-    ret = upsrecv(buf, sizeof(buf), ENDCHAR, IGNCHARS);
+    ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, IGNCHARS, 3, 0);
+    ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, IGNCHARS, 3, 0);
     bv = hex2d(buf, 2) / 10;
     if (bv <= 11.0)      bp =  10;
     else if (bv <= 11.2) bp =  20;
@@ -379,22 +380,22 @@ void upsdrv_updateinfo(void)
     dstate_setinfo("battery.voltage", "%.1f", bv);
     dstate_setinfo("battery.charge",  "%03d",   bp);
     
-    upssend("%s", ":M\r");
+    ser_send(upsfd, "%s", ":M\r");
     sleep(UPSDELAY); 
-    ret = upsrecv(buf, sizeof(buf), ENDCHAR, IGNCHARS);
-    ret = upsrecv(buf, sizeof(buf), ENDCHAR, IGNCHARS);
+    ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, IGNCHARS, 3, 0);
+    ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, IGNCHARS, 3, 0);
     dstate_setinfo("input.voltage.maximum", "%d", hex2d(buf, 2));
     
-    upssend("%s", ":I\r");
+    ser_send(upsfd, "%s", ":I\r");
     sleep(UPSDELAY); 
-    ret = upsrecv(buf, sizeof(buf), ENDCHAR, IGNCHARS);
-    ret = upsrecv(buf, sizeof(buf), ENDCHAR, IGNCHARS);
+    ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, IGNCHARS, 3, 0);
+    ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, IGNCHARS, 3, 0);
     dstate_setinfo("input.voltage.minimum", "%d", hex2d(buf, 2));
     
-    upssend("%s", ":C\r");
+    ser_send(upsfd, "%s", ":C\r");
     sleep(UPSDELAY);
-    ret = upsrecv(buf, sizeof(buf), ENDCHAR, IGNCHARS);
-    ret = upsrecv(buf, sizeof(buf), ENDCHAR, IGNCHARS);
+    ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, IGNCHARS, 3, 0);
+    ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, IGNCHARS, 3, 0);
     switch (atoi(buf)) {
     case 0:  dstate_setinfo("ups.test.result", "%s", "OK");                    break;
     case 1:  dstate_setinfo("ups.test.result", "%s", "Battery Bad (Replace)"); break;
@@ -422,17 +423,15 @@ void upsdrv_banner(void)
 
 void upsdrv_initups(void)
 {
-    open_serial(device_path, B2400);
+    upsfd = ser_open(device_path);
+    ser_set_speed(upsfd, device_path, B2400);
     setup_serial();
     
-    /* don't let upscommon warn about it since it happens way too often */
-    /* this driver does its own checking with better handling */
-    flag_timeoutfailure = -1;
-    
     ups_sync();
     ups_ident();
 }
 
 void upsdrv_cleanup(void)
 {
+    ser_close(upsfd, device_path);
 }
