@@ -25,7 +25,6 @@
 #include <sys/wait.h>
 
 #include "config.h"
-#include "shared.h"
 #include "proto.h"
 #include "version.h"
 #include "common.h"
@@ -40,29 +39,39 @@ typedef struct {
 	void	*next;
 }	ups_t;
 
-	ups_t	*upstable = NULL;
+static	ups_t	*upstable = NULL;
 
-	int	execwait, verbose = 0, maxsdorder = 0, testmode = 0;
+static	int	verbose = 0, maxsdorder = 0, testmode = 0, exec_error = 0;
 
 	/* timer - keeps us from getting stuck if a driver hangs */
-	int	maxstartdelay = 45;
+static	int	maxstartdelay = 45;
 
 	/* Directory where driver executables live */
-	const char * driverpath;
+static	char	*driverpath = NULL;
 
 	/* passthrough to the drivers: chroot path and new user name */
-	char	*pt_root = NULL, *pt_user = NULL;
+static	char	*pt_root = NULL, *pt_user = NULL;
+
+static	sigset_t	uc_sigmask;
+static	struct	sigaction	sa;
 
 void do_upsconf_args(char *upsname, char *var, char *val)
 {
 	ups_t	*tmp, *last;
 
+	/* handle global declarations */
 	if (!upsname) {
 		if (!strcmp(var, "maxstartdelay"))
 			maxstartdelay = atoi(val);
 
-		if (!strcmp(var, "driverpath"))
+		if (!strcmp(var, "driverpath")) {
+			if (driverpath)
+				free(driverpath);
+
 			driverpath = xstrdup(val);
+		}
+
+		/* ignore anything else - it's probably for main */
 
 		return;
 	}
@@ -128,6 +137,7 @@ static void send_term(char *upsname, char *driver, char *port)
 
 	if (ret != 0) {
 		upslog(LOG_ERR, "Can't open %s", pidfn);
+		exec_error++;
 		return;
 	}
 
@@ -135,6 +145,7 @@ static void send_term(char *upsname, char *driver, char *port)
 
 	if (!pidf) {
 		upslog(LOG_ERR, "Can't open %s", pidfn);
+		exec_error++;
 		return;
 	}
 
@@ -146,6 +157,7 @@ static void send_term(char *upsname, char *driver, char *port)
 	if (pid < 2) {
 		upslogx(LOG_NOTICE, "Ignoring invalid pid %d in %s",
 			pid, pidfn);
+		exec_error++;
 		return;
 	}
 
@@ -159,6 +171,7 @@ static void send_term(char *upsname, char *driver, char *port)
 
 	if (ret < 0) {
 		upslog(LOG_ERR, "kill pid %d failed", pid);
+		exec_error++;
 		return;
 	}
 }
@@ -196,66 +209,95 @@ static void stop_all_drivers(void)
 
 		tmp = tmp->next;
 	}
-
-	exit(0);
 }
 
-/* reap exiting children */
-static void sigchld(int sig)
+static void waitpid_timeout(int sig)
 {
-	execwait = 0;
+	/* do nothing */
+	return;
 }
 
-/* break free of drivers that don't start up quickly */
-static void sigalrm(int sig)
+static void forkexec(const char *prog, char **argv, ups_t *ups)
 {
-	upslogx(LOG_NOTICE, "Startup timer elapsed, continuing...");
-	execwait = 0;
-}
+	int	ret;
+	pid_t	pid;
 
-static void forkexec(const char *prog, ups_t *ups)
-{
-	int	ret, arg = 0;
-	char	*argv[8];
+	pid = fork();
 
-	/* TODO: consider redoing this with wait() depending on portability */
-
-	/* parent spins until the child exits */
-	execwait = 1;
-
-	ret = fork();
-
-	if (ret < 0)
+	if (pid < 0)
 		fatal("fork");
 
-	/* parent */
-	if (ret != 0) {
+	if (pid != 0) {			/* parent */
+		int	wstat;
 
-		/* catching this will unset execwait */
-		signal(SIGCHLD, sigchld);
-
-		/* give us a way out if the child gets stuck */
-		signal(SIGALRM, sigalrm);
+		sigemptyset(&uc_sigmask);
+		sa.sa_mask = uc_sigmask;
+		sa.sa_flags = 0;
+		sa.sa_handler = waitpid_timeout;
+		sigaction(SIGALRM, &sa, NULL);
 
 		if (ups->maxstartdelay != -1)
 			alarm(ups->maxstartdelay);
 		else
 			alarm(maxstartdelay);
 
-		/* spin until the child is done */
-		while (execwait)
-			usleep(250000);
+		ret = waitpid(pid, &wstat, 0);
 
-		alarm(0);
 		signal(SIGALRM, SIG_IGN);
+		alarm(0);
+
+		if (ret == -1) {
+			upslogx(LOG_WARNING, "Startup timer elapsed, continuing...");
+			exec_error++;
+			return;
+		}
+
+		if (WIFEXITED(wstat) == 0) {
+			upslogx(LOG_WARNING, "Driver exited abnormally");
+			exec_error++;
+			return;
+		}
+
+		if (WEXITSTATUS(wstat) != 0) {
+			upslogx(LOG_WARNING, "Driver failed to start"
+			" (exit status=%d)", WEXITSTATUS(wstat));
+			exec_error++;
+			return;
+		}
+
+		/* the rest only work when WIFEXITED is nonzero */
+
+		if (WIFSIGNALED(wstat)) {
+			upslog(LOG_WARNING, "Driver died after signal %d",
+				WTERMSIG(wstat));
+			exec_error++;
+		}
 
 		return;
 	}
 
-	if (verbose)
-		printf("exec: %s -a %s", prog, ups->upsname);
+	/* child */
+
+	ret = execv(prog, argv);
+
+	/* shouldn't get here */
+	fatal("execv");
+}		
+
+static void start_driver(ups_t *ups)
+{
+	char	dfn[SMALLBUF], *argv[8];
+	int	ret, arg = 0;
+	struct	stat	fs;
+
+	snprintf(dfn, sizeof(dfn), "%s/%s", driverpath, ups->driver);
+	ret = stat(dfn, &fs);
+
+	if (ret < 0)
+		fatal("Can't start %s", dfn);
+
+	argv[arg++] = xstrdup(dfn);
 
-	argv[arg++] = xstrdup(prog);
 	argv[arg++] = "-a";
 	argv[arg++] = ups->upsname;
 
@@ -282,30 +324,7 @@ static void forkexec(const char *prog, ups_t *ups)
 	/* tie it off */
 	argv[arg++] = NULL;
 
-	/* child */
-
-	if (testmode)
-		exit(0);
-
-	ret = execv(prog, argv);
-
-	/* should not be reached */
-	fatal("execv");
-}
-
-static void start_driver(ups_t *ups)
-{
-	char	dfn[SMALLBUF];
-	int	ret;
-	struct	stat	fs;
-
-	snprintf(dfn, sizeof(dfn), "%s/%s", driverpath, ups->driver);
-	ret = stat(dfn, &fs);
-
-	if (ret < 0)
-		fatal("Can't start %s", dfn);
-
-	forkexec(dfn, ups);
+	forkexec(dfn, argv, ups);
 }
 
 /* start user-selected driver */
@@ -358,67 +377,26 @@ static void help(const char *progname)
 	printf("  stop <ups>		only stop driver for UPS <ups>\n");
 	printf("  shutdown		shutdown all UPS drivers in ups.conf\n");
 	printf("  shutdown <ups>	only shutdown UPS <ups>\n");
-	exit(0);
+
+	exit(EXIT_SUCCESS);
 }
 
 static void shutdown_driver(ups_t *ups)
 {
-	int	ret;
 	char	*argv[7], dfn[SMALLBUF];
 
-	/* parent spins until the child exits */
-	execwait = 1;
-
-	ret = fork();
-
-	if (ret < 0)
-		fatal("fork");
-
-	/* parent */
-	if (ret != 0) {
-
-		/* wait for child to return */
-		signal(SIGCHLD, sigchld);
-
-		/* give us a way out if the child gets stuck */
-		signal(SIGALRM, sigalrm);
-
-		if (ups->maxstartdelay != -1)
-			alarm(ups->maxstartdelay);
-		else
-			alarm(maxstartdelay);
-
-		while (execwait)
-			usleep(250000);
-
-		alarm(0);
-		signal(SIGALRM, SIG_IGN);
-
-		return;
-	}
-
-	/* child */
-
 	snprintf(dfn, sizeof(dfn), "%s/%s", driverpath, ups->driver);
 
 	if (verbose)
-		printf("exec: %s -a %s -d 0 -k\n", dfn, ups->upsname);
+		printf("exec: %s -a %s -k\n", dfn, ups->upsname);
 
 	argv[0] = dfn;
 	argv[1] = "-a";
 	argv[2] = xstrdup(ups->upsname);
-	argv[3] = "-d";
-	argv[4] = "0";
-	argv[5] = "-k";
-	argv[6] = NULL;
+	argv[3] = "-k";
+	argv[4] = NULL;
 
-	if (testmode)
-		exit(0);
-
-	ret = execv(dfn, argv);
-
-	/* should not be reached */
-	fatal("execv");
+	forkexec(dfn, argv, ups);
 }
 
 static void shutdown_one_driver(char *upsname)
@@ -459,8 +437,30 @@ static void shutdown_all_drivers(void)
 			tmp = tmp->next;
 		}
 	}
+}
 
-	exit(0);
+static void exit_cleanup(void)
+{
+	ups_t	*tmp, *next;
+
+	tmp = upstable;
+
+	while (tmp) {
+		next = tmp->next;
+
+		if (tmp->driver)
+			free(tmp->driver);
+		if (tmp->port)
+			free(tmp->port);
+		if (tmp->upsname)
+			free(tmp->upsname);
+		free(tmp);
+
+		tmp = next;
+	}
+
+	if (driverpath)
+		free(driverpath);
 }
 
 int main(int argc, char **argv)
@@ -474,8 +474,8 @@ int main(int argc, char **argv)
 	prog = argv[0];
 	while ((i = getopt(argc, argv, "+htvu:r:V")) != EOF) {
 		switch(i) {
-			case 'v':
-				verbose++;
+			case 'r':
+				pt_root = optarg;
 				break;
 
 			case 't':
@@ -491,12 +491,12 @@ int main(int argc, char **argv)
 				pt_user = optarg;
 				break;
 
-			case 'r':
-				pt_root = optarg;
+			case 'v':
+				verbose++;
 				break;
 
 			case 'V':
-				exit(0);
+				exit(EXIT_SUCCESS);
 
 			case 'h':
 			default:
@@ -506,7 +506,7 @@ int main(int argc, char **argv)
 	}
 
 	argc -= optind;
-        argv += optind;
+	argv += optind;
 
 	if (argc < 1)
 		help(prog);
@@ -514,45 +514,54 @@ int main(int argc, char **argv)
 	if (testmode)
 		printf("*** Testing mode: not calling exec/kill\n");
 
-    driverpath = xstrdup(DRVPATH);  /* set default */
+	driverpath = xstrdup(DRVPATH);  /* set default */
+
+	atexit(exit_cleanup);
 
 	if (!strcmp(argv[0], "start")) {
-		read_upsconf(1);
+		read_upsconf();
 
 		if (argc == 1)
 			start_all_drivers();
 		else
 			start_one_driver(argv[1]);
 
-		exit(0);
+		if (exec_error)
+			exit(EXIT_FAILURE);
+
+		exit(EXIT_SUCCESS);
 	}
 
 	if (!strcmp(argv[0], "stop")) {
-		read_upsconf(1);
+		read_upsconf();
 
 		if (argc == 1)
 			stop_all_drivers();
 		else
 			stop_one_driver(argv[1]);
 
-		exit(0);
+		if (exec_error)
+			exit(EXIT_FAILURE);
+
+		exit(EXIT_SUCCESS);
 	}
 
 	if (!strcmp(argv[0], "shutdown")) {
-		read_upsconf(1);
+		read_upsconf();
 
 		if (argc == 1)
 			shutdown_all_drivers();
 		else
 			shutdown_one_driver(argv[1]);
 
-		exit(0);
+		if (exec_error)
+			exit(EXIT_FAILURE);
+
+		exit(EXIT_SUCCESS);
 	}
 
 	fatalx("Error: unrecognized command [%s]\n", argv[0]);
 
 	/* NOTREACHED */
-	return 0;
+	exit(EXIT_FAILURE);
 }
-
-
