@@ -33,13 +33,13 @@ typedef struct {
 	int	ch;			/* last character read		*/
 	
 	char	**arglist;		/* array of pointers to words	*/
-	int	*argsize;		/* list of sizes for realloc	*/
-	int	numargs;		/* max usable in arglist	*/
-	int	maxargs;		/* for reallocing arglist	*/
+	size_t	*argsize;		/* list of sizes for realloc	*/
+	size_t	numargs;		/* max usable in arglist	*/
+	size_t	maxargs;		/* for reallocing arglist	*/
 
 	char	*wordbuf;		/* accumulator for current word	*/
 	char	*wordptr;		/* where next char goes in word	*/
-	int	wordbufsize;		/* for reallocing wordbuf	*/
+	size_t	wordbufsize;		/* for reallocing wordbuf	*/
 
 	int	linenum;		/* for good error reporting	*/
 	int	error;			/* set when an error occurred	*/
@@ -50,8 +50,8 @@ typedef struct {
 	int	magic;			/* buffer validity check	*/
 
 	/* these may be redefined by the caller to customize memory use	*/
-	int	arg_limit;		/* halts growth of arglist	*/
-	int	wordlen_limit;		/* halts growth of any wordbuf	*/
+	size_t	arg_limit;		/* halts growth of arglist	*/
+	size_t	wordlen_limit;		/* halts growth of any wordbuf	*/
 
 }	PCONF_CTX;
 
