@@ -51,19 +51,6 @@ int setenv(const char *name, const char *value, int overwrite);
 #define         TIOCM_SR        0020            /* secondary receive */
 #endif
 
-#ifndef HAVE_STRLCPY
-size_t strlcpy(char *dst, const char *src, size_t siz);
-#endif
-
-/* cyberpower needs this to compile on IRIX. 
- *
- * Whether this actually works is another story.
- */
-
-#ifndef CRTSCTS
-   #define CRTSCTS   020000000000
-#endif
-
 #ifdef HAVE_GETPASSPHRASE
 #define GETPASS getpassphrase
 #else
