@@ -20,13 +20,6 @@
 /* structure for funcs that get called by msg parse routine */
 struct ups_handler
 {
-	void	(*setvar)(int, int, char *);
-	void	(*instcmd)(int, int, char *);
-
-	/* temporary names during the driver transition */
-
-	/* these will all lose the "new_" once the old versions are gone! */
-
-	int	(*new_setvar)(const char *, const char *);
-	int	(*new_instcmd)(const char *, const char *);
+	int	(*setvar)(const char *, const char *);
+	int	(*instcmd)(const char *, const char *);
 };
