@@ -20,30 +20,28 @@
 #ifndef ACCESS_H_SEEN
 #define ACCESS_H_SEEN 1
 
-/* possible actions for access controls */
-#define ACTION_GRANT	1
-#define ACTION_DENY	2
+#define ACCESS_REJECT	0
+#define ACCESS_ACCEPT	1
 
 /* ACL structure */
 struct acl_t {
-        char    *name;
-        unsigned int    addr;
-        unsigned int    mask;
-        void    *next;
+	char	*name;
+	unsigned int	addr;
+	unsigned int	mask;
+	void	*next;
 };
 
 /* ACCESS structure */
 struct access_t {
 	int	action;
-	int	level;
 	char	*aclname;
 	void	*next;
 };
 
 int acl_check(const char *aclname, const struct sockaddr_in *addr);
-int access_check(const struct sockaddr_in *addr, int level);
+int access_check(const struct sockaddr_in *addr);
 void acl_add(const char *aclname, char *ipblock);
-void access_add(const char *action, const char *level, const char *aclname);
+void access_add(int type, int numargs, const char **arg);
 void acl_free(void);
 void access_free(void);
 
