@@ -29,7 +29,119 @@
 	extern	char	*statepath, *datapath, *certfile;
 	extern	upstype	*firstups;
 	ups_t	*upstable = NULL;
-	int	reload_flag;
+	int	num_ups = 0;
+
+/* add another UPS for monitoring from ups.conf */
+static void ups_create(const char *fn, const char *name, const char *desc)
+{
+	upstype	*temp, *last;
+
+	temp = last = firstups;
+
+	/* find end of linked list */
+	while (temp != NULL) {
+		last = temp;
+
+		if (!strcasecmp(temp->name, name)) {
+			upslogx(LOG_ERR, "UPS name [%s] is already in use!", 
+				name);
+			return;
+		}
+
+		temp = temp->next;
+	}
+
+	/* grab some memory and add the info */
+	temp = xmalloc(sizeof(upstype));
+
+	temp->name = xstrdup(name);
+	temp->fn = xstrdup(fn);
+
+	if (desc)
+		temp->desc = xstrdup(desc);
+	else
+		temp->desc = NULL;
+
+	temp->stale = 1;
+
+	temp->numlogins = 0;
+	temp->fsd = 0;
+	temp->retain = 1;
+	temp->next = NULL;
+
+	temp->dumpdone = 0;
+	temp->data_ok = 0;
+
+	/* preload this to the current time to avoid false staleness */
+	time(&temp->last_heard);
+
+	temp->last_ping = 0;
+	temp->last_connfail = 0;
+	temp->inforoot = NULL;
+	temp->cmdlist = NULL;
+
+	if (last == NULL)
+		firstups = temp;
+	else
+		last->next = temp;
+
+	temp->sock_fd = sstate_connect(temp);
+
+	num_ups++;
+}
+
+/* change the configuration of an existing UPS (used during reloads) */
+static void ups_update(const char *fn, const char *name, const char *desc)
+{
+	upstype	*temp;
+
+	temp = get_ups_ptr(name);
+
+	if (!temp) {
+		upslogx(LOG_ERR, "UPS %s disappeared during reload", name);
+		return;
+	}
+
+	/* paranoia */
+	if (!temp->fn) {
+		upslogx(LOG_ERR, "UPS %s had a NULL filename!", name);
+
+		/* let's give it something quick to use later */
+		temp->fn = xstrdup("");
+	}
+
+	/* when the filename changes, force a reconnect */
+	if (strcmp(temp->fn, fn) != 0) {
+
+		upslogx(LOG_NOTICE, "Redefined UPS [%s]", name);
+
+		/* release all data */
+		sstate_infofree(temp);
+		sstate_cmdfree(temp);
+		pconf_finish(&temp->sock_ctx);
+
+		close(temp->sock_fd);
+		temp->sock_fd = -1;
+		temp->dumpdone = 0;
+
+		/* now redefine the filename and wrap up */
+		free(temp->fn);
+		temp->fn = xstrdup(fn);
+	}
+
+	/* update the description */
+
+	if (temp->desc)
+		free(temp->desc);
+
+	if (desc)
+		temp->desc = xstrdup(desc);
+	else
+		temp->desc = NULL;
+
+	/* always set this on reload */
+	temp->retain = 1;
+}		
 
 /* return 1 if usable, 0 if not */
 static int parse_upsd_conf_args(int numargs, char **arg)
@@ -71,6 +183,20 @@ static int parse_upsd_conf_args(int numargs, char **arg)
 		return 1;
 	}
 
+	/* ACCEPT <aclname> [<aclname>...] */
+	if (!strcmp(arg[0], "ACCEPT")) {
+		access_add(ACCESS_ACCEPT, numargs - 1, 
+			(const char **) &arg[1]);
+		return 1;
+	}
+
+	/* REJECT <aclname> [<aclname>...] */
+	if (!strcmp(arg[0], "REJECT")) {
+		access_add(ACCESS_REJECT, numargs - 1, 
+			(const char **) &arg[1]);
+		return 1;
+	}
+
 	/* everything below here uses up through arg[2] */
 	if (numargs < 3)
 		return 0;
@@ -84,10 +210,8 @@ static int parse_upsd_conf_args(int numargs, char **arg)
 	if (numargs < 4)
 		return 0;
 
-	/* ACCESS <action> <level> <aclname> */
-	/* any password on the end is now ignored - no more host-based auth */
 	if (!strcmp(arg[0], "ACCESS")) {
-		access_add(arg[1], arg[2], arg[3]);
+		upslogx(LOG_WARNING, "ACCESS in upsd.conf is no longer supported - switch to ACCEPT/REJECT");
 		return 1;
 	}
 
@@ -96,12 +220,12 @@ static int parse_upsd_conf_args(int numargs, char **arg)
 }
 
 /* called for fatal errors in parseconf like malloc failures */
-void upsd_conf_err(const char *errmsg)
+static void upsd_conf_err(const char *errmsg)
 {
 	upslogx(LOG_ERR, "Fatal error in parseconf (upsd.conf): %s", errmsg);
 }
 
-void read_upsdconf(int reload)
+static void load_upsdconf(int reloading)
 {
 	char	fn[SMALLBUF];
 	PCONF_CTX	ctx;
@@ -110,19 +234,16 @@ void read_upsdconf(int reload)
 
 	check_perms(fn);
 
-	reload_flag = reload;
-
 	pconf_init(&ctx, upsd_conf_err);
 
 	if (!pconf_file_begin(&ctx, fn)) {
 		pconf_finish(&ctx);
 
-		if (!reload)
+		if (!reloading)
 			fatalx("%s", ctx.errmsg);
-		else {
-			upslogx(LOG_ERR, "Reload failed: %s", ctx.errmsg);
-			return;
-		}
+
+		upslogx(LOG_ERR, "Reload failed: %s", ctx.errmsg);
+		return;
 	}
 
 	while (pconf_file_next(&ctx)) {
@@ -136,7 +257,7 @@ void read_upsdconf(int reload)
 			continue;
 
 		if (!parse_upsd_conf_args(ctx.numargs, ctx.arglist)) {
-			int	i;
+			unsigned int	i;
 			char	errmsg[SMALLBUF];
 
 			snprintf(errmsg, sizeof(errmsg), 
@@ -146,7 +267,7 @@ void read_upsdconf(int reload)
 				snprintfcat(errmsg, sizeof(errmsg), " %s", 
 					ctx.arglist[i]);
 
-			upslogx(LOG_WARNING, "%s", errmsg);
+			upslogx(LOG_WARNING, errmsg);
 		}
 
 	}
@@ -202,12 +323,11 @@ void do_upsconf_args(char *upsname, char *var, char *val)
 }
 
 /* add valid UPSes from ups.conf to the internal structures */
-void upsconf_add(int reloading)
+static void upsconf_add(int reloading)
 {
 	ups_t	*tmp = upstable, *next;
 	char	statefn[SMALLBUF];
 
-	/* nonfatal for now - eventually this will be required */
 	if (!tmp) {
 		upslogx(LOG_WARNING, "Warning: no UPS definitions in ups.conf");
 		return;
@@ -229,13 +349,14 @@ void upsconf_add(int reloading)
 			upslogx(LOG_WARNING, "Warning: ignoring incomplete configuration for UPS [%s]\n", 
 				tmp->upsname);
 		} else {
-			snprintf(statefn, sizeof(statefn), "%s-%s", tmp->driver, xbasename(tmp->port));
+			snprintf(statefn, sizeof(statefn), "%s-%s", 
+				tmp->driver, xbasename(tmp->port));
 
 			/* if a UPS exists, update it, else add it as new */
-			if ((reloading) && (findups(tmp->upsname) != NULL))
-				redefine_ups(statefn, tmp->upsname, tmp->desc);
+			if ((reloading) && (get_ups_ptr(tmp->upsname) != NULL))
+				ups_update(statefn, tmp->upsname, tmp->desc);
 			else
-				addups(statefn, tmp->upsname, tmp->desc);
+				ups_create(statefn, tmp->upsname, tmp->desc);
 		}
 
 		/* free tmp's resources */
@@ -258,7 +379,7 @@ void upsconf_add(int reloading)
 }
 
 /* remove a UPS from the linked list */
-void delete_ups(upstype *target)
+static void delete_ups(upstype *target)
 {
 	upstype	*ptr, *last;
 
@@ -308,7 +429,7 @@ void delete_ups(upstype *target)
 }			
 
 /* see if we can open a file */
-static int check_file(char *fn)
+static int check_file(const char *fn)
 {
 	char	chkfn[SMALLBUF];
 	FILE	*f;
@@ -327,7 +448,7 @@ static int check_file(char *fn)
 }
 
 /* called after SIGHUP */
-void reload_config(void)
+void conf_reload(void)
 {
 	upstype	*upstmp, *upsnext;
 
@@ -345,7 +466,7 @@ void reload_config(void)
 	}
 
 	/* reload from ups.conf */
-        read_upsconf(1);		/* 1 = required */
+	read_upsconf();
 	upsconf_add(1);			/* 1 = reloading */
 
 	/* flush ACL/ACCESS definitions */
@@ -353,7 +474,7 @@ void reload_config(void)
 	access_free();
 
 	/* now reread upsd.conf */
-	read_upsdconf(1);
+	load_upsdconf(1);		/* 1 = reloading */
 
 	/* now delete all UPS entries that didn't get reloaded */
 
@@ -383,3 +504,17 @@ void reload_config(void)
 	/* and finally reread from upsd.users */
 	user_load();
 }
+
+/* startup: load config files */
+void conf_load(void)
+{
+	/* upsd.conf */
+	load_upsdconf(0);	/* 0 = initial */
+
+	/* handle ups.conf */
+	read_upsconf();
+	upsconf_add(0);
+
+	/* upsd.users */
+	user_load();
+}
