@@ -19,13 +19,14 @@
 
 #include "common.h"
 
-#include "upstype.h"
 #include "upsd.h"
 #include "sstate.h"
 #include "state.h"
 #include "desc.h"
 #include "neterr.h"
 
+#include "netget.h"
+
 static void get_numlogins(ctype *client, const char *upsname)
 {
 	const	upstype	*ups;
@@ -198,7 +199,7 @@ static void get_var(ctype *client, const char *upsname, const char *var)
 	if (!ups_available(ups, client))
 		return;
 
-	val = sstate_getinfo(ups, var, 0);
+	val = sstate_getinfo(ups, var);
 
 	if (!val) {
 		send_err(client, NUT_ERR_VAR_NOT_SUPPORTED);
@@ -212,54 +213,54 @@ static void get_var(ctype *client, const char *upsname, const char *var)
 		sendback(client, "VAR %s %s \"%s\"\n", upsname, var, val);
 }
 
-int net_get(ctype *client, int numarg, char **arg)
+void net_get(ctype *client, int numarg, const char **arg)
 {
 	if (numarg < 2) {
 		send_err(client, NUT_ERR_INVALID_ARGUMENT);
-		return 1;
+		return;
 	}
 
 	/* GET NUMLOGINS UPS */
 	if (!strcasecmp(arg[0], "NUMLOGINS")) {
 		get_numlogins(client, arg[1]);
-		return 1;
+		return;
 	}
 
-	/* GET UPSDESC UPS> */
+	/* GET UPSDESC UPS */
 	if (!strcasecmp(arg[0], "UPSDESC")) {
 		get_upsdesc(client, arg[1]);
-		return 1;
+		return;
 	}
 
 	if (numarg < 3) {
 		send_err(client, NUT_ERR_INVALID_ARGUMENT);
-		return 1;
+		return;
 	}
 
 	/* GET VAR UPS VARNAME */
 	if (!strcasecmp(arg[0], "VAR")) {
 		get_var(client, arg[1], arg[2]);
-		return 1;
+		return;
 	}
 
 	/* GET TYPE UPS VARNAME */
 	if (!strcasecmp(arg[0], "TYPE")) {
 		get_type(client, arg[1], arg[2]);
-		return 1;
+		return;
 	}
 
 	/* GET DESC UPS VARNAME */
 	if (!strcasecmp(arg[0], "DESC")) {
 		get_desc(client, arg[1], arg[2]);
-		return 1;
+		return;
 	}
 
 	/* GET CMDDESC UPS CMDNAME */
 	if (!strcasecmp(arg[0], "CMDDESC")) {
 		get_cmddesc(client, arg[1], arg[2]);
-		return 1;
+		return;
 	}
 
 	send_err(client, NUT_ERR_INVALID_ARGUMENT);
-	return 1;
+	return;
 }
