@@ -40,39 +40,37 @@
 #include "desc.h"
 #include "neterr.h"
 
-	/* pointers to linked lists */
+	/* externally-visible settings and pointers */
+
 	upstype	*firstups = NULL;
-	ctype	*firstclient = NULL, *currclient = NULL, *udpclient = NULL;
-
-static	int	listenfd, udpfd, net_port = PORT, upsnum = 0;
-
-	/* default is to listen on all local interfaces */
-static	struct	in_addr	listenaddr;
 
 	/* default 15 seconds before data is marked stale */
 	int	maxage = 15;
 
-	/* signal handlers */
-	struct sigaction sa;
-	sigset_t upsd_sigmask;
-
-	/* pid file */
-	char	pidfn[SMALLBUF];
-
-	/* set by signal handlers */
-	int	reload_flag = 0, stop_flag = 0;
-
 	/* preloaded to STATEPATH in main, can be overridden via upsd.conf */
 	char	*statepath = NULL;
 
 	/* preloaded to DATADIR in main, can be overridden via upsd.conf */
 	char	*datapath = NULL;
 
-#ifdef SHUT_RDWR
-#define shutdown_how SHUT_RDWR
-#else
-#define shutdown_how 2
-#endif
+	/* everything else */
+
+static	ctype	*firstclient = NULL;
+
+static	int	listenfd, net_port = PORT;
+
+	/* default is to listen on all local interfaces */
+static	struct	in_addr	listenaddr;
+
+	/* signal handlers */
+static	struct sigaction sa;
+static	sigset_t upsd_sigmask;
+
+	/* pid file */
+static	char	pidfn[SMALLBUF];
+
+	/* set by signal handlers */
+static	int	reload_flag = 0, exit_flag = 0;
 
 /* return a pointer to the named ups if possible */
 upstype *get_ups_ptr(const char *name)
@@ -89,31 +87,6 @@ upstype *get_ups_ptr(const char *name)
 	return NULL;
 }
 
-/* search for a ups given the name - and this one allows defaults */
-upstype *findups(const char *upsname)
-{
-	upstype	*temp;
-
-	temp = firstups;
-
-	/* NULL name -> return first one */
-	if (!upsname)
-		return firstups;
-
-	/* return first one if a good name isn't given */
-	if (!strcmp(upsname, ""))
-		return firstups;
-	
-	while (temp != NULL) {
-		if (!strcasecmp(temp->name, upsname))
-			return temp;
-
-		temp = temp->next;
-	}
-
-	return NULL;
-}
-
 /* mark the data stale if this is new, otherwise cleanup any remaining junk */
 static void ups_data_stale(upstype *ups)
 {
@@ -155,136 +128,6 @@ static void check_ups(upstype *ups)
 		ups->sock_fd = sstate_connect(ups);
 }
 
-/* change the configuration of an existing UPS (used during reloads) */
-void redefine_ups(const char *fn, const char *name, const char *desc)
-{
-	upstype	*temp;
-
-	temp = findups(name);
-
-	if (!temp) {
-		upslogx(LOG_ERR, "UPS %s disappeared during reload", name);
-		return;
-	}
-
-	/* paranoia */
-	if (!temp->fn) {
-		upslogx(LOG_ERR, "UPS %s had a NULL filename!", name);
-
-		/* let's give it something quick to use later */
-		temp->fn = xstrdup("");
-	}
-
-	/* when the filename changes, force a reconnect */
-	if (strcmp(temp->fn, fn) != 0) {
-
-		upslogx(LOG_NOTICE, "Redefined UPS [%s]", name);
-
-		/* release all data */
-		sstate_infofree(temp);
-		sstate_cmdfree(temp);
-		pconf_finish(&temp->sock_ctx);
-
-		close(temp->sock_fd);
-		temp->sock_fd = -1;
-		temp->dumpdone = 0;
-
-		/* now redefine the filename and wrap up */
-		free(temp->fn);
-		temp->fn = xstrdup(fn);
-	}
-
-	/* update the description */
-
-	if (temp->desc)
-		free(temp->desc);
-
-	if (desc)
-		temp->desc = xstrdup(desc);
-	else
-		temp->desc = NULL;
-
-	/* always set this on reload */
-	temp->retain = 1;
-}		
-
-/* start monitoring a ups called <name> from a file called <fn> */
-void addups(const char *fn, const char *name, const char *desc)
-{
-	upstype	*temp, *last;
-
-	temp = last = firstups;
-
-	/* find end of linked list */
-	while (temp != NULL) {
-		last = temp;
-
-		if (!strcasecmp(temp->name, name)) {
-			upslogx(LOG_ERR, "UPS name [%s] is already in use!", 
-				name);
-			return;
-		}
-
-		temp = temp->next;
-	}
-
-	/* grab some memory and add the info */
-	temp = xcalloc(1, sizeof(upstype));
-
-	temp->fn = xstrdup(fn);
-	temp->name = xstrdup(name);
-
-	if (desc)
-		temp->desc = xstrdup(desc);
-	else
-		temp->desc = NULL;
-
-	temp->stale = 1;
-
-	temp->numlogins = 0;
-	temp->fsd = 0;
-	temp->retain = 1;
-	temp->next = NULL;
-
-	temp->dumpdone = 0;
-	temp->data_ok = 0;
-
-	/* preload this to the current time to avoid false staleness */
-	time(&temp->last_heard);
-
-	temp->last_ping = 0;
-	temp->last_connfail = 0;
-	temp->inforoot = NULL;
-
-	if (last == NULL)	/* first one */
-		firstups = temp;
-	else			/* not the first */
-		last->next = temp;
-
-	upsnum++;
-
-	temp->sock_fd = sstate_connect(temp);
-}
-
-/* setup a socket for incoming udp requests */
-static void setupudp(void)
-{
-	struct	sockaddr_in	recv;
-
-	if ((udpfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
-		fatal("Can't create socket");
-
-	memset(&recv, '\0', sizeof(recv));
-	recv.sin_addr = listenaddr;
-	recv.sin_family = AF_INET;
-	recv.sin_port = htons(net_port);
-
-	if (bind(udpfd, (struct sockaddr *) &recv, sizeof(recv)) < 0)
-		fatal("Can't bind to udp socket (port %d)", net_port);
-
-	return;
-}
-
 /* create a listening socket for tcp connections */
 static void setuptcp(void)
 {
@@ -325,7 +168,7 @@ static void declogins(const char *upsname)
 {
 	upstype	*ups;
 
-	ups = findups(upsname);
+	ups = get_ups_ptr(upsname);
 
 	if (ups == NULL) {
 		upslogx(LOG_INFO, "Tried to decrement invalid ups name (%s)", upsname);
@@ -333,9 +176,13 @@ static void declogins(const char *upsname)
 	}
 
 	ups->numlogins--;
+
+	if (ups->numlogins < 0)
+		upslogx(LOG_ERR, "Programming error: UPS [%s] has numlogins=%d",
+			ups, ups->numlogins);
 }
 
-/* remove a client struct from the linked list */
+/* disconnect a client connection and free all related memory */
 static void delclient(ctype *dclient)
 {
 	ctype	*tmp, *last;
@@ -366,15 +213,17 @@ static void delclient(ctype *dclient)
 				SSL_free(tmp->ssl);
 #endif
 
+			pconf_finish(&tmp->ctx);
+
 			if (last == NULL)	/* deleting first entry */
 				firstclient = tmp->next;
 			else
 				last->next = tmp->next;
 
 			free(tmp);
-			currclient = NULL;
 			return;
 		}
+
 		last = tmp;
 		tmp = tmp->next;
 	}
@@ -393,7 +242,10 @@ int sendback(ctype *client, const char *fmt, ...)
 	va_list ap;
 
 	if (!client)
-		return -1;
+		return 0;
+
+	if (client->delete)
+		return 0;
 
 	va_start(ap, fmt);
 	vsnprintf(ans, sizeof(ans), fmt, ap);
@@ -401,19 +253,17 @@ int sendback(ctype *client, const char *fmt, ...)
 
 	len = strlen(ans);
 
-	upsdebugx(2, "sendto: [destfd=%d] [len=%d] [%s]", 
+	upsdebugx(2, "write: [destfd=%d] [len=%d] [%s]", 
 		client->fd, len, ans);
 
 	if (client->ssl)
 		res = ssl_write(client, ans, len);
 	else
-		res = sendto(client->fd, ans, len, 0, 
-			(struct sockaddr *) &client->sock,
-			sizeof(struct sockaddr_in));
+		res = write(client->fd, ans, len);
 
 	if (len != res) {
-		upslog(LOG_NOTICE, "sendto() failed for %s", client->addr);
-		delclient(client);
+		upslog(LOG_NOTICE, "write() failed for %s", client->addr);
+		client->delete = 1;
 		return 0;	/* failed */
 	}
 
@@ -432,23 +282,8 @@ int send_err(ctype *client, const char *errtype)
 	return sendback(client, "ERR %s\n", errtype);
 }
 
-/* increment the login counter for this ups */
-static void inclogins(const char *upsname)
-{
-	upstype	*ups;
-
-	ups = findups(upsname);
-
-	if (ups == NULL) {
-		upslogx(LOG_INFO, "Tried to increment invalid ups name");
-		return;
-	}
-
-	ups->numlogins++;
-}
-
 /* disconnect anyone logged into this UPS */
-void kick_login_clients(char *upsname)      
+void kick_login_clients(const char *upsname)
 {
 	ctype   *tmp, *next;
 
@@ -468,7 +303,7 @@ void kick_login_clients(char *upsname)
 				tmp->addr, upsname);
 			delclient(tmp);
 		}
-                
+
 		tmp = next;
 	}
 }
@@ -490,321 +325,78 @@ int ups_available(const upstype *ups, ctype *client)
 	return 1;
 }
 
-/* handler for "VER" */
-void do_sendver(ctype *client, char *arg, char *rest)
+/* check flags and access for an incoming command from the network */
+static void check_command(int cmdnum, ctype *client, int numarg, 
+	const char **arg)
 {
-	sendback(client, 
-		"Network UPS Tools upsd %s - http://www.networkupstools.org/\n", 
-		UPS_VERSION);
-}
-
-/* handler for "HELP" */
-void do_sendhelp(ctype *client, char *arg, char *rest)
-{
-	char	ans[SMALLBUF];
-	int	i;
-
-	memset(ans, '\0', sizeof(ans));
-	for (i = 0; netcmds[i].name != NULL; i++)
-		snprintfcat(ans, sizeof(ans), " %s", netcmds[i].name);
-
-	sendback(client, "Commands:%s\n", ans);
-}
-
-/* handler for "LOGOUT" */
-void do_logout(ctype *client, char *arg, char *rest)
-{
-	if (client->username)
-		upslogx(LOG_INFO, "Client %s@%s logged out", 
-			client->username, client->addr);
-	else
-		upslogx(LOG_INFO, "Client on %s logged out", client->addr);
-
-	/* check return since sendback can call delclient itself */
-	if (sendback(client, "OK Goodbye\n"))
-		delclient(client);
-}		
-
-/* handler for "LOGIN" */
-static void do_login(ctype *client, char *arg, char *rest)
-{
-	int	found = 0;
-	upstype *ups;
-
-	if (client->loginups != NULL) {
-		upslogx(LOG_INFO, "Client on %s tried to login twice", client->addr);
-		send_err(client, NUT_ERR_ALREADY_LOGGED_IN);
-		return;
-	}
-
-	/* "LOGIN" only - no UPS name given as argument */
-	if ((arg == NULL) || (strlen(arg) == 0)) {
-
-		/* no UPSes defined? */
-		if (!firstups) {
-			send_err(client, NUT_ERR_UNKNOWN_UPS);
-			return;
-		}
-
-		/* give it the name of the first one in the .conf */
-		client->loginups = xstrdup(firstups->name);
-
-		/* but warn them that this is going away some day */
-		upslogx(LOG_WARNING, "Warning: LOGIN relied on the default UPS");
-	}
-	else {
-		/* see if <arg> is a valid UPS name */
-
-		for (ups = firstups; ups != NULL; ups = ups->next) {
-			if (!strcasecmp(ups->name, arg))
-				found = 1;
-		}
-
-		if (!found) {
-			send_err(client, NUT_ERR_UNKNOWN_UPS);
-			return;
-		}
-
-		client->loginups = xstrdup(arg);
-	}
-
-	inclogins(client->loginups);
-
-	upslogx(LOG_INFO, "Client %s@%s logged into UPS [%s]%s",
-		client->username, client->addr, client->loginups, 
-		client->ssl ? " (SSL)" : "");
-
-	sendback(client, "OK\n");
-}
-
-/* handler for "PASSWORD" */
-static void do_password(ctype *client, char *arg, char *rest)
-{
-	if ((arg == NULL) || (strlen(arg) == 0)) {
-		upslogx(LOG_INFO, "Client on %s sent a NULL password!", client->addr);
-		send_err(client, NUT_ERR_INVALID_PASSWORD);
-		return;	
-	}
-
-	if (client->password != NULL) {
-		upslogx(LOG_INFO, "Client on %s tried to set a password twice", 
-			client->addr);
-		send_err(client, NUT_ERR_ALREADY_SET_PASSWORD);
-		return;
-	}
-
-	client->password = xstrdup(arg);
-
-	sendback(client, "OK\n");
-}
-
-/* handler for "FSD" */
-static void do_fsd(ctype *client, char *arg, char *rest)
-{
-	upstype *ups;
-
-	ups = findups(arg);
-
-	if (ups == NULL) {
-		send_err(client, NUT_ERR_UNKNOWN_UPS);
-		return;
-	}
-
-	upslogx(LOG_INFO, "Setting FSD on UPS [%s]", ups->name);
-
-	ups->fsd = 1;
-	sendback(client, "OK FSD-SET\n");
-}
-
-/* handler for "MASTER" */
-static void do_master(ctype *client, char *arg, char *rest)
-{
-	upstype *ups;
-
-	ups = findups(arg);
-
-	if (ups == NULL) {
-		send_err(client, NUT_ERR_UNKNOWN_UPS);
-		return;
-	}
-
-	/* really quite simple - this is just an access level check */
-	sendback(client, "OK MASTER-GRANTED\n");
-}
-
-/* handler for "USERNAME" */
-static void do_username(ctype *client, char *arg, char *rest)
-{
-	if ((arg == NULL) || (strlen(arg) ==0)) {
-		upslogx(LOG_INFO, "Client on %s sent a NULL username!",
-		       client->addr);
-		send_err(client, NUT_ERR_INVALID_USERNAME);
-		return;
-	}
-
-	if (client->username != NULL) {
-		upslogx(LOG_INFO, "Client on %s tried to set a username twice",
-		       client->addr);
-		send_err(client, NUT_ERR_ALREADY_SET_USERNAME);
-		return;
-	}
-
-	client->username = xstrdup(arg);
-
-	sendback(client, "OK\n");
-}
-
-static int check_flags(ctype *client, int flags, const char *action, int udp)
-{
-	if (flags & FLAG_TCP) { 
-		if (udp) {
-			send_err(client, NUT_ERR_UNKNOWN_COMMAND);
-			return 0;
-		}
-	}
-
-	if (flags & FLAG_USER) {
+	if (netcmds[cmdnum].flags & FLAG_USER) {
 		if (!client->username) {
 			send_err(client, NUT_ERR_USERNAME_REQUIRED);
-			return 0;
+			return;
 		}
 
 		if (!client->password) {
 			send_err(client, NUT_ERR_PASSWORD_REQUIRED);
-			return 0;
-		}
-
-		/* instcmd is a special case - checked in do_instcmd */
-		if (flags & FLAG_CMD)
-			return 1;
-
-		/* now check this specific action */
-
-		if (!user_checkaction(&client->sock, client->username,
-			client->password, action)) {
-			send_err(client, NUT_ERR_ACCESS_DENIED);
-			return 0;
-		}
-	}
-
-	/* seems good so far... */
-	return 1;
-}
-
-/* this handles new commands separately during 1.4 and will take over in 2.0 */
-static int new_net_parse(ctype *client, int numarg, char **arg, int udp)
-{
-	int	i;
-
-	if (!access_check(&client->sock, LEVEL_BASE)) {
-		send_err(client, NUT_ERR_ACCESS_DENIED);
-		return 1;	
-	}
-
-	if (numarg < 1)
-		return 0;
-
-	/* UDP clients can only do GET */
-	if ((udp) && (strcasecmp(arg[0], "GET") != 0))
-		return 0;
-
-	for (i = 0; new_netcmds[i].name != NULL; i++) {
-		if (!strcasecmp(new_netcmds[i].name, arg[0])) {
-
-			/* see if the flags are satisfied */
-			if (!check_flags(client, new_netcmds[i].flags, arg[0], udp))
-				return 1;
-
-			if (!access_check(&client->sock, new_netcmds[i].level)) {
-				send_err(client, NUT_ERR_ACCESS_DENIED);
-				return 1;
-			}
-
-			/* may drop out with 0 in order to call old funcs */
-			return new_netcmds[i].func(client, numarg - 1, &arg[1]);
-		}
-	}
-
-	return 0;
-}
-
-/* parse requests from the network */
-static void parse_net(ctype *client, int udp)
-{
-	int	i, len;
-	char	*cmd, *arg2, *rest, *ptr, *buf;
-	PCONF_CTX	ctx;
-
-	len = client->rqpos;
-	buf = client->rq;
-
-	/* check for base access if this is a UDP client */
-	if ((udp) && (!access_check(&client->sock, LEVEL_BASE)))
-		return;
-
-	/* FUTURE: maybe switch to pconf_char with per-client context */
-	pconf_init(&ctx, NULL);
-
-	/* paranoia */
-	buf[len] = '\0';
-
-	if (pconf_line(&ctx, buf)) {
-		if (!pconf_parse_error(&ctx)) {
-			if (new_net_parse(client, ctx.numargs, ctx.arglist, udp)) {
-				pconf_finish(&ctx);
-				return;
-			}
-		}
-	}
-
-	pconf_finish(&ctx);
-
-	/* fallthrough: try parsing it the old way */
-
-	arg2 = rest = NULL;
-
-	/* kill any embedded cr/lf chars */
-	for (i = 0; i < len; i++)
-		if ((buf[i] == 13) || (buf[i] == 10))
-			buf[i] = '\0';
-
-	/* parse from buf: <cmd> [<arg2> [<rest>]] */
-
-	cmd = buf;
-	ptr = strchr(cmd, ' ');
-	if (ptr) {
-		*ptr++ = '\0';
-		arg2 = ptr;
-		ptr = strchr(arg2, ' ');
-		if (ptr) {
-			*ptr++ = '\0';
-			rest = ptr;
-		}
-	}
-
-	for (i = 0; netcmds[i].name != NULL; i++) {
-		if (!strcasecmp(netcmds[i].name, cmd)) {
-
-			/* see if the flags are satisfied */
-			if (!check_flags(client, netcmds[i].flags, cmd, udp))
-				return;
-
-			if (!access_check(&client->sock, netcmds[i].level)) {
-				send_err(client, NUT_ERR_ACCESS_DENIED);
-				return;
-			}
-
-			netcmds[i].func(client, arg2, rest);
 			return;
 		}
 	}
 
-	upsdebugx(4, "Unknown command: %s", cmd);
+	/* looks good - call the command */
+	netcmds[cmdnum].func(client, numarg - 1, &arg[1]);
+}
+
+/* parse requests from the network */
+static void parse_net(ctype *client)
+{
+	int	i;
+
+	/* see if this client is still allowed to talk to us */
+	if (!access_check(&client->sock)) {
+		send_err(client, NUT_ERR_ACCESS_DENIED);
+		client->delete = 1;
+		return;
+	}
+
+	/* paranoia */
+	client->rq[client->rqpos] = '\0';
+
+	/* by this point we should always have a usable line */
+	if (pconf_line(&client->ctx, client->rq) != 1) {
+
+		/* shouldn't happen */
+		upslogx(LOG_WARNING, "pconf_line couldn't handle the input");
+		send_err(client, NUT_ERR_UNKNOWN_COMMAND);
+		return;
+	}
+
+	if (pconf_parse_error(&client->ctx)) {
+		upsdebugx(4, "parse error on net read");
+		send_err(client, NUT_ERR_UNKNOWN_COMMAND);
+		return;
+	}
+
+	/* shouldn't happen */
+	if (client->ctx.numargs < 1) {
+		send_err(client, NUT_ERR_UNKNOWN_COMMAND);
+		return;
+	}
+
+	for (i = 0; netcmds[i].name != NULL; i++) {
+		if (!strcasecmp(netcmds[i].name, client->ctx.arglist[0])) {
+			check_command(i, client, client->ctx.numargs, 
+			(const char **) client->ctx.arglist);
+
+			return;
+		}
+	}
+
+	/* fallthrough = not matched by any entry in netcmds */
+
 	send_err(client, NUT_ERR_UNKNOWN_COMMAND);
 }
 
 /* scan the list of UPSes for sanity */
-void check_every_ups(void)
+static void check_every_ups(void)
 {
 	upstype *ups;
 
@@ -830,6 +422,14 @@ static void answertcp(void)
 	if (acc < 0)
 		return;
 
+	if (!access_check(&csock)) {
+		upslogx(LOG_NOTICE, "Rejecting TCP connection from %s", 
+			inet_ntoa(csock.sin_addr));
+		shutdown(acc, shutdown_how);
+		close(acc);
+		return;
+	}
+
 	last = tmp = firstclient;
 
 	while (tmp != NULL) {
@@ -841,11 +441,14 @@ static void answertcp(void)
 
 	tmp->addr = xstrdup(inet_ntoa(csock.sin_addr));
 	tmp->fd = acc;
+	tmp->delete = 0;
 	memcpy(&tmp->sock, &csock, sizeof(struct sockaddr_in));
 
 	tmp->rqpos = 0;
 	memset(tmp->rq, '\0', sizeof(tmp->rq));
 
+	pconf_init(&tmp->ctx, NULL);
+
 	tmp->loginups = NULL;		/* for upsmon */
 	tmp->username = NULL;
 	tmp->password = NULL;
@@ -860,14 +463,6 @@ static void answertcp(void)
 	else
 		last->next = tmp;
 
-	if (!access_check(&tmp->sock, LEVEL_BASE)) {
-		upslogx(LOG_NOTICE, "Rejecting TCP connection from %s", 
-			inet_ntoa(csock.sin_addr));
-
-		delclient(tmp);
-		return;
-	}
-
 	upslogx(LOG_INFO, "Connection from %s", inet_ntoa(csock.sin_addr));
 }
 
@@ -887,7 +482,6 @@ static void readtcp(ctype *client)
 	if (ret < 1) {
 		upslogx(LOG_INFO, "Host %s disconnected (read failure)",
 			client->addr);
-
 		delclient(client);
 		return;
 	}
@@ -906,13 +500,13 @@ static void readtcp(ctype *client)
 
 		/* parse on linefeed ("blah blah\n") */
 		if (buf[i] == 10) {	/* LF */
-			currclient = client;	/* enable tcp replies */
-			parse_net(client, 0);
+			parse_net(client);
 
-			if (currclient == NULL) /* connection was closed */
+			/* bail out if the connection closed in parse_net */
+			if (client->delete) {
+				delclient(client);
 				return;
-
-			currclient = NULL;	/* disable */
+			}
 			
 			/* reset queue */
 			client->rqpos = 0;
@@ -923,15 +517,18 @@ static void readtcp(ctype *client)
 	return;
 }
 
-static void upsd_stop(void)
+static void upsd_cleanup(void)
 {
-	ctype	*tmpcli;
+	ctype	*tmpcli, *tmpnext;
 	upstype	*ups, *unext;
 
 	/* cleanup client fds */
-	for (tmpcli = firstclient; tmpcli != NULL; tmpcli = tmpcli->next) {
-		shutdown(tmpcli->fd, shutdown_how);
-		close(tmpcli->fd);
+	tmpcli = firstclient;
+
+	while (tmpcli) {
+		tmpnext = tmpcli->next;
+		delclient(tmpcli);
+		tmpcli = tmpnext;
 	}
 
 	if (strcmp(pidfn, "") != 0)
@@ -971,12 +568,8 @@ static void upsd_stop(void)
 		free(statepath);
 	if (datapath)
 		free(datapath);
-	if (udpclient)
-		free(udpclient);
 	if (certfile)
 		free(certfile);
-
-	exit(0);
 }
 
 /* service requests and check on new data */
@@ -987,24 +580,19 @@ static void mainloop(void)
 	int	res, maxfd;
 	ctype	*tmpcli, *tmpnext;
 	upstype	*utmp, *unext;
-	socklen_t	fromlen;
-
-	if (stop_flag)
-		upsd_stop();
 
 	if (reload_flag) {
-		reload_config();
+		conf_reload();
 		reload_flag = 0;
 	}
 
 	FD_ZERO(&rfds);
-	FD_SET(udpfd, &rfds);
 	FD_SET(listenfd, &rfds);
 	
 	tv.tv_sec = 2;
 	tv.tv_usec = 0;
 
-	maxfd = (udpfd > listenfd) ? udpfd : listenfd;
+	maxfd = listenfd;
 
 	/* scan through clients and add to FD_SET */
 	for (tmpcli = firstclient; tmpcli != NULL; tmpcli = tmpcli->next) {
@@ -1026,29 +614,6 @@ static void mainloop(void)
 	res = select(maxfd + 1, &rfds, NULL, NULL, &tv);
 
 	if (res > 0) {
-		if (FD_ISSET(udpfd, &rfds)) {
-			fromlen = sizeof(struct sockaddr_in);
-
-			udpclient->fd = udpfd;
-			memset(&udpclient->sock, '\0', fromlen);
-			memset(&udpclient->rq, '\0', SMALLBUF);
-
-			res = recvfrom(udpfd, udpclient->rq, SMALLBUF, 0, 
-				(struct sockaddr *) &udpclient->sock,
-				&fromlen);
-
-			if (res > 0) {
-
-				/* dummy setting, as opposed to NULL */
-				currclient = udpclient;
-
-				udpclient->rqpos = res;
-				parse_net(udpclient, 1);
-
-				currclient = NULL;
-			}
-		}
-
 		if (FD_ISSET(listenfd, &rfds))
 			answertcp();
 
@@ -1094,16 +659,17 @@ static void help(const char *progname)
 	printf("		commands:\n");
 	printf("		 - reload: reread configuration files\n");
 	printf("		 - stop: stop process and exit\n");
-	printf("  -D		raise debugging level\n");           
+	printf("  -D		raise debugging level\n");
 	printf("  -f		stay in the foreground for testing\n");
 	printf("  -h		display this help\n");
 	printf("  -i <address>	binds to interface <address>\n");
-	printf("  -p <port>	sets network port (UDP and TCP)\n");
+	printf("  -p <port>	sets network port (default: TCP port %d)\n", 
+		PORT);
 	printf("  -r <dir>	chroots to <dir>\n");
 	printf("  -u <user>	switch to <user> (if started as root)\n");
 	printf("  -V		display the version of this software\n");
 
-	exit(1);
+	exit(EXIT_SUCCESS);
 }
 
 static void set_reload_flag(int sig)
@@ -1111,10 +677,9 @@ static void set_reload_flag(int sig)
 	reload_flag = 1;
 }
 
-static void set_stop_flag(int sig)
+static void set_exit_flag(int sig)
 {
-	upslogx(LOG_INFO, "Signal %d: exiting", sig);
-	stop_flag = 1;
+	exit_flag = sig;
 }
 
 /* basic signal setup to ignore SIGPIPE */
@@ -1128,7 +693,7 @@ static void setupsignals(void)
 	sigaction(SIGPIPE, &sa, NULL);
 
 	/* handle shutdown signals */
-	sa.sa_handler = set_stop_flag;
+	sa.sa_handler = set_exit_flag;
 	sigaction(SIGINT, &sa, NULL);
 	sigaction(SIGQUIT, &sa, NULL);
 	sigaction(SIGTERM, &sa, NULL);
@@ -1154,6 +719,13 @@ static void initial_dump_wait(void)
 	time(&now);
 
 	while (difftime(now, start) < INITIAL_WAIT_MAX) {
+
+		/* check this now in case the user is trying to ^C us */
+		if (exit_flag) {
+			upsd_cleanup();
+			exit(EXIT_FAILURE);
+		}
+
 		maxfd = 0;
 		numups = 0;
 		numdone = 0;
@@ -1172,7 +744,7 @@ static void initial_dump_wait(void)
 
 				numups++;
 			}
-                }
+		}
 
 		ret = select(maxfd + 1, &rfds, NULL, NULL, &tv);
 
@@ -1207,7 +779,7 @@ static void initial_dump_wait(void)
 		}
 
 		time(&now);
-        }
+	}
 
 	printf(" giving up\n");
 	fflush(stdout);
@@ -1277,7 +849,7 @@ int main(int argc, char **argv)
 			case 'V':
 
 				/* do nothing - we already printed the banner */
-				exit(0);
+				exit(EXIT_SUCCESS);
 
 			case 'c':
 				if (!strncmp(optarg, "reload", strlen(optarg)))
@@ -1302,7 +874,7 @@ int main(int argc, char **argv)
 
 	if (cmd) {
 		sendsignalfn(pidfn, cmd);
-		exit(0);
+		exit(EXIT_SUCCESS);
 	}
 
 	argc -= optind;
@@ -1311,19 +883,13 @@ int main(int argc, char **argv)
 	if (argc != 0)
 		help(progname);
 
-	/* provide a skeletal pseudo-client for all UDP-based requests */
-
-	udpclient = xmalloc(sizeof(ctype));
-	udpclient->ssl = NULL;			/* impossible */
-
 	setupsignals();
-	setupudp();
 	setuptcp();
 
 	open_syslog("upsd");
 
 	/* send logging to the syslog pre-background for later use */
-        xbit_set(&upslog_flags, UPSLOG_SYSLOG);
+	syslogbit_set();
 
 	if (nut_statepath_env)
 		statepath = xstrdup(nut_statepath_env);
@@ -1343,17 +909,11 @@ int main(int argc, char **argv)
 	/* check statepath perms */
 	check_perms(statepath);
 
-	/* read in the ups definitions (required) */
-	read_upsconf(1);
+	/* read data from config files - upsd.conf, ups.conf, upsd.users */
+	conf_load();
 
-	/* ... then do something with them */
-	upsconf_add(0);			/* 0 = initial */
-
-	read_upsdconf(0);		/* 0 = initial */
-	user_load();
-
-	if (upsnum == 0)
-		fatalx("Fatal error: you need to define at least one UPS in your ups.conf!\n");
+	if (num_ups == 0)
+		fatalx("Fatal error: at least one UPS must be defined in ups.conf");
 
 	ssl_init();
 
@@ -1374,9 +934,11 @@ int main(int argc, char **argv)
 		memset(&pidfn, '\0', sizeof(pidfn));
 	}
 
-	for (;;)
+	while (exit_flag == 0)
 		mainloop();
 
-	/* NOTREACHED */
-	return 0;
+	upslogx(LOG_INFO, "Signal %d: exiting", exit_flag);
+	upsd_cleanup();
+
+	exit(EXIT_SUCCESS);
 }
